import QtQuick 1.1
import "constants.js" as UI

Item {
    id: wrapper
    width: parent.width
    height: borders.height + 60
    z: 1
    state: notifyController.Visibility ? "visible" : "hidden"

    Rectangle {
        id: borders
        width: parent.width - 30
        height: text.height + 30
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.bottom: parent.bottom
        border.color: "#67AB16"; border.width: 2
        radius: 6
        color: "#2d292a"
        opacity: 0.9
        Behavior on height { PropertyAnimation { duration: 200 } }
    }

    Text {
        id: text
        height: Math.max (60, undefined)
        anchors.left: borders.left
        anchors.right: borders.right
        anchors.margins: 10
        anchors.verticalCenter: borders.verticalCenter
        font.pixelSize: UI.FONT_SIZE_NORMAL
        font.family: UI.FONT_FAMILY
        text: notifyController.Message
        color: "white"
        wrapMode: Text.WordWrap
        verticalAlignment: Text.AlignVCenter
        Behavior on height { PropertyAnimation { duration: 200 } }
        Behavior on text { PropertyAnimation { duration: 250 } }
    }

    states: [
        State {
            name: "visible"
            PropertyChanges { target: wrapper; y: 0 }
        },
        State {
            name: "hidden"
            PropertyChanges { target: wrapper; y: -(wrapper.height + 10) }
        }
    ]

    transitions: [
        Transition {
            NumberAnimation { property: "y"; easing.type: Easing.InOutQuad; duration: 500 }
        }
    ]

    MouseArea {
        anchors.fill: parent
        onClicked: notifyController.hideBannerAndClearMessageList()
    }
}
