import QtQuick 1.1
import com.meego 1.0
import "constants.js" as UI

Item {
    id: wrapper
    anchors.fill: parent

    property alias content: contentField.children
    property int contentHeight: 0
    property int contentWidth: dialogElements.width
    property string headerText: ""
    property bool showTextArea: false

    state: "hidden"

    Image {
        id: background
        anchors.fill: parent
        source: "/usr/share/spotify/pics/object_menu_background.png"
        state: wrapper.state

        states: [
            State {
                name: "visible"
                PropertyChanges { target: background; opacity: 1 }
            },
            State {
                name: "hidden"
                PropertyChanges { target: background; opacity: 0 }
            }
        ]
        transitions: [
            Transition {
                from: "*"; to: "visible"
                SequentialAnimation {
                    PropertyAction { target: wrapper; property: "visible"; value: true }
                    NumberAnimation { target: background; properties: "opacity"; duration: 500 }
                }
            },
            Transition {
                from: "*"; to: "hidden"
                SequentialAnimation {
                    NumberAnimation { target: background; properties: "opacity"; duration: 500 }
                    PropertyAction { target: wrapper; property: "visible"; value: false }
                }
            }
        ]

        MouseArea {
            anchors.fill: parent
        }

    }

    Item {
        id: dialogElements
        anchors.fill: parent
        anchors.leftMargin: 20
        anchors.rightMargin: 20
        state: parent.state
        transform: Scale {
            id: scale
            origin.x: dialogElements.width / 2
            origin.y: dialogElements.height / 2
        }
        states: [
            State {
                name: "visible"
                PropertyChanges { target: scale; xScale: 1; yScale: 1 }
            },
            State {
                name: "hidden"
                PropertyChanges { target: scale; xScale: 0.5; yScale: 0 }
            }
        ]
        transitions: [
            Transition {
                from: "visible"
                to: "hidden"
                NumberAnimation { target: scale; properties: "xScale,yScale"; duration: 300 }
            },
            Transition {
                from: "hidden"
                to: "visible"
                NumberAnimation { target: scale; properties: "xScale,yScale"; duration: 300 }
            }
        ]

        MouseArea {
            anchors.top: parent.top
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.bottom:  topGreenLine.top
            onClicked: rootWindow.hideDialog()
        }

        Text {
            id: title
            anchors.bottom: topGreenLine.top
            anchors.left: topGreenLine.left
            anchors.right: topGreenLine.right
            anchors.leftMargin: 5

            anchors.bottomMargin: 10

            horizontalAlignment: Text.AlignLeft
            text: headerText

            font {
                pixelSize: 36
                family: UI.FONT_FAMILY
            }
            color: "#86c021"
            wrapMode: Text.Wrap
        }

        Image {
            id: close
            anchors.right: topGreenLine.right
            anchors.verticalCenter: title.verticalCenter
            source: "/usr/share/spotify/pics/dialog_close.png"
        }

        Image {
            id: topGreenLine
            source: "/usr/share/spotify/pics/object_menu_greenline.png"
            width: wrapper.width - 40
            height: 1
            anchors.bottom: showTextArea ? msgLength.top : contentField.top
            anchors.bottomMargin: 20
            anchors.horizontalCenter: parent.horizontalCenter
        }

        Item {
            id: contentField

            anchors.left: parent.left
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            visible: !showTextArea
            enabled: visible

            height: contentHeight
        }

        Text {
            id: msgLength
            font.family: UI.FONT_FAMILY
            font.pixelSize: UI.FONT_SIZE_NORMAL
            anchors.left: parent.left
            anchors.bottom: textAreaContainer.top
            anchors.leftMargin: 15
            anchors.bottomMargin: 5
            text: shareController.MsgLength
            color: shareController.MsgLength < 0 ? "red" : "white"
            visible: textAreaContainer.visible
            enabled: visible
        }

        Flickable {
            id: textAreaContainer
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            height: 200
            contentWidth: textArea.width
            contentHeight: textArea.height
            flickableDirection: Flickable.VerticalFlick
            pressDelay: 100
            clip: true
            visible: showTextArea
            enabled: visible

            TextArea {
                id: textArea
                objectName: "textArea"
                anchors.left: parent.left
                width: textAreaContainer.width
                anchors.top: parent.top
                font.pixelSize: UI.FONT_SIZE_NORMAL
                font.family: UI.FONT_FAMILY
                wrapMode: TextEdit.Wrap
                placeholderText: "Optional message"
                height: 200 //Math.max (200, preferredHeight)
                onTextChanged: shareController.messageLengthChanged(textArea.text)
            }
        }
        ScrollDecorator {
            flickableItem: textAreaContainer
        }

        Button {
            id: okButton
            objectName: "okButton"
            enabled: shareController.MsgLength >= 0 && visible
            anchors.top: textAreaContainer.bottom
            anchors.topMargin: 15
            anchors.horizontalCenter: textAreaContainer.horizontalCenter
            width: 140
            text: "Send"
            platformStyle: ButtonStyle { inverted: true }
            onClicked: {
                textArea.platformCloseSoftwareInputPanel()
                msgLength.focus = true
                rootWindow.hideDialog()
                shareController.share(textArea.text)
            }
            visible: textAreaContainer.visible
        }

        Image {
            id: bottomGreenLine
            source: "/usr/share/spotify/pics/object_menu_greenline.png"
            width: wrapper.width - 40
            height: 1
            anchors.top: showTextArea ? okButton.bottom : contentField.bottom
            anchors.topMargin: 20
            anchors.horizontalCenter: parent.horizontalCenter
        }
        MouseArea {
            anchors.top: bottomGreenLine.bottom
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.bottom:  parent.bottom
            onClicked: rootWindow.hideDialog()
        }

    }
}
