import QtQuick 1.1
import "../Components"
import "constants.js" as UI

Rectangle {
    id: container

    property string labelText1: "";
    property string labelText2: "";
    property string loadingStatus: "online"
    property string trackSender: ""
    property string trackMessage: ""

    property int imageSize: 65
    property int clickStartY: 0
    property int clickStartX: 0
    property int notSeenTracksCount: 0
    property int expandedHeight: 200

    property bool npHighlighther: false
    property bool starred: false
    property bool isSeen: false
    property bool enableAddToQueue: false

    signal clicked;
    signal pressAndHold;
    signal starClicked;
    signal seenClicked;
    signal swiped;

    height: 80
    width: parent.width
    color: "transparent"
    state: "normal"

    Rectangle {
        id: messageShader
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        anchors.right: parent.right
        height: 0
        color: "#404041"
    }

    MouseArea {
        id: mouseArea
        property bool pressed: false

        anchors.fill: parent

        onPressed: {
            pressed = true
            clickStartY = mouseY
            clickStartX = mouseX
            timer.start()
        }
        onClicked: {
            expandedHeight = 123 + messageText.implicitHeight

            var diffX = mouseX - clickStartX
            if (diffX < 50 && diffX > -50) {
                container.color = "lightgrey"
                if (container.state == "normal" && trackMessage != "") {
                    container.state = "expanded"
                    tracksOfPlaylistPageController.listClicked(index)
                    tracksOfPlaylistPageController.setSeen(index, true)
                    container.clicked()
                } else if (container.state == "expanded") {
                    container.state = "normal"
                } else {
                    tracksOfPlaylistPageController.listClicked(index)
                    tracksOfPlaylistPageController.setSeen(index, true)
                    goToPage("NowPlayingPage")
                }
                timer2.start()
            }
        }
        onPressAndHold: {
            var diffX = mouseX - clickStartX
            if (diffX < 50 && diffX > -50) {
                container.pressAndHold();
            }
        }
        onReleased: {
            container.color = "transparent"
            highlightRemoveTimer.start()
            pressed = "false"
        }
        onCanceled: {
            container.color = "transparent"
            addToQueueRect.state = "hidden"
        }
        onMouseXChanged: {
            var diffX = mouseX - clickStartX
            if (diffX > 70 && diffX <= 120) {
                addToQueueRect.state = "visible"
            } else if (diffX > 120 && pressed) {
                pressed = false
                addToQueueRect.state = "highlight"
                highlightRemoveTimer.start()
                container.swiped()
            } else if (pressed) {
                addToQueueRect.state = "hidden"
            }
        }
    }

    Timer {
        id: timer
        interval: 70; running: false; repeat: false
        onTriggered: {
            if (Math.abs(mouseArea.mouseY - clickStartY) < 3) {
                container.color = "lightgrey"
            } else {
                container.color = "transparent"
            }
        }
    }

    Timer {
        id: timer2
        interval: 70; running: false; repeat: false
        onTriggered: container.color = "transparent"
    }

    Timer {
        id: highlightRemoveTimer
        interval: 300; running: false; repeat: false
        onTriggered: addToQueueRect.state = "hidden"
    }

    Rectangle {
        id: addToQueueRect
        visible: container.enableAddToQueue
        enabled: visible
        color: "#67AB16"
        opacity: 0.5
        anchors.top: parent.top
        anchors.right: parent.right
        anchors.left: parent.left
        height: 80
        state: "hidden"

        states: [
            State {
                name: "visible"
                PropertyChanges { target: addToQueueRect; opacity: 0.5 }
            },
            State {
                name: "hidden"
                PropertyChanges { target: addToQueueRect; opacity: 0 }
            },
            State {
                name: "highlight"
                PropertyChanges { target: addToQueueRect; opacity: 1 }
            }
        ]

        transitions: [
            Transition {
                from: "hidden"
                to: "visible"
                NumberAnimation { target: addToQueueRect; properties: "opacity"; duration: 150; easing.type: Easing.InCubic }
            },
            Transition {
                from: "visible"
                to: "hidden"
                NumberAnimation { target: addToQueueRect; properties: "opacity"; duration: 300; easing.type: Easing.OutCubic }
            },
            Transition {
                from: "*"
                to: "highlight"
                NumberAnimation { target: addToQueueRect; properties: "opacity"; duration: 300; easing.type: Easing.OutCubic }
            }
        ]
    }

    Item {
        id: normalItemWrapper
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: parent.top
        anchors.leftMargin: 0
        anchors.rightMargin: 15
        height: 80

        Item {
            id: seenItem
            height: parent.height
            width: 50
            anchors.verticalCenter: parent.verticalCenter

            Rectangle {
                id: seenRect
                anchors.centerIn: parent
                height: 20
                width: 20
                color: isSeen ? "#393536" : "dodgerblue"
                border.color: "Grey"
                border.width: 2
                radius: 10
            }

            Image {
                anchors.top: parent.top
                anchors.left: parent.left
                anchors.topMargin: 6
                anchors.leftMargin: 6
                width: 14
                height: width
                visible: trackMessage == "" ? false : true
                fillMode: Image.PreserveAspectFit
                source: "/usr/share/spotify/pics/icon-m-toolbar-new-message.png"
                smooth: true
            }

            MouseArea {
                objectName: "seenRectMouseArea"
                height: parent.height
                width: parent.width + 40
                onReleased: {
                    container.seenClicked()
                }
            }
        }

        Item {
            height: childrenRect.height
            anchors.verticalCenter: parent.verticalCenter
            anchors.left: seenItem.right
            anchors.right: iconRight.left

            TitlePanel {
                id: text1
                headerText: labelText1
                anchors.left: parent.left
                anchors.right: parent.right
                highlightItem: npHighlighther
            }

            Image {
                id: statusIcon
                anchors.verticalCenter: text2.verticalCenter
                width: loadingStatus == "online" ? 0 : 16
                height: 16
                source: loadingStatus == "downloaded" ? "/usr/share/spotify/pics/Spotify-29.png" :
                        loadingStatus == "downloading" ? "/usr/share/spotify/pics/indicator_playlist_downloading.png" :
                        loadingStatus == "partially" ? "/usr/share/spotify/pics/Spotify-28.png" :
                        ""
            }

            PropertiesDetailPanel {
                id: text2
                visible: loadingStatus != "downloading"
                headerText: labelText2
                anchors.top: text1.bottom
                anchors.left: loadingStatus == "online" ? parent.left : statusIcon.right
                anchors.leftMargin: loadingStatus == "online" ? 0 : 15
                anchors.right: parent.right
                highlightItem: npHighlighther
            }

            ListItemProgress {
                id: progressBar
                visible: loadingStatus == "downloading"
                objectName: "progressBar"
                anchors.left: statusIcon.right
                anchors.leftMargin: 15
                width: 250
                anchors.verticalCenter: statusIcon.verticalCenter
                progress: 0.5
            }
        }

        Image {
            id: iconRight
            width: 40
            height: width
            source: starred ? "/usr/share/spotify/pics/Spotify-19.png" :
                    "/usr/share/spotify/pics/Spotify-23.png"
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter

            MouseArea {
                objectName: "starMouseArea"
                anchors.fill: parent
                anchors.leftMargin: -30
                anchors.rightMargin: -15
                onReleased: {
                    container.starClicked()
                }
            }
        }
    }

    Text {
        id: senderText
        anchors {
            top: normalItemWrapper.bottom
            left: parent.left
            right: parent.right
            leftMargin: seenItem.width
            rightMargin: iconRight.width
            topMargin: 2
        }
        visible: false
        text: trackSender+" says:"
        elide: Text.ElideRight
        font.pixelSize: UI.FONT_SIZE_NORMAL
        font.family: UI.FONT_FAMILY
        font.bold: true
        color: "white"
    }

    Text {
        id: messageText
        anchors.top: senderText.bottom
        anchors.topMargin: 3
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.leftMargin: seenItem.width + 5
        anchors.rightMargin: iconRight.width + 15
        visible: false
        text: trackMessage
        wrapMode: Text.WordWrap
        font.pixelSize: UI.FONT_SIZE_NORMAL
        font.family:  UI.FONT_FAMILY
        color: "grey"
    }

    states: [
        State {
            name: "normal"
            PropertyChanges { target: container; height: 80 }
            PropertyChanges { target: messageText; visible: false }
            PropertyChanges { target: senderText; visible: false }
            PropertyChanges { target: messageShader; height: 0 }
        },
        State {
            name: "expanded"
            PropertyChanges { target: container; height: expandedHeight }
            PropertyChanges { target: messageText; visible: true }
            PropertyChanges { target: senderText; visible: true }
            PropertyChanges { target: messageShader; height: expandedHeight - 80 }
        }
    ]
    transitions: [
        Transition {
            from: "normal"
            to: "expanded"
            NumberAnimation { target: container; properties: "height"; duration: 250; easing.type: Easing.InCubic }
            NumberAnimation { target: messageShader; properties: "height"; duration: 250; easing.type: Easing.InCubic }
            NumberAnimation { target: messageText; properties: "visible"; duration: 500; easing.type: Easing.InCubic }
            NumberAnimation { target: senderText; properties: "visible"; duration: 500; easing.type: Easing.InCubic }
        },
        Transition {
            from: "expanded"
            to: "normal"
            NumberAnimation { target: container; properties: "height"; duration: 500; easing.type: Easing.OutCubic }
            NumberAnimation { target: messageShader; properties: "height"; duration: 500; easing.type: Easing.OutCubic }
            NumberAnimation { target: messageText; properties: "visible"; duration: 250; easing.type: Easing.OutCubic }
            NumberAnimation { target: senderText; properties: "visible"; duration: 250; easing.type: Easing.OutCubic }
        }
    ]
}
