import QtQuick 1.1
import "../Components"
import "constants.js" as UI
Rectangle {
    id: container

    property bool inverted: false
    property string imageSource: ""
    property string labelText1: ""
    property string labelText1Color: ""
    property string labelText2: ""
    property string labelText3: ""
    property int loadingStatus: 0
    property int loadingProgress: 0

    property bool showIcon: true
    property bool showItem: true
    property int imageSize: imageSource ? 65 : 0
    property bool npHighlighther: false
    property int clickStartY: 0
    property int clickStartX: 0

    property bool showNextIcon: false
    property bool starred: false
    property bool allowRightImage: true
    property bool showSeenCount: false
    property int notSeenTracksCount: 0
    property bool listReady: true

    property bool enableAddToQueue: false
    property bool showProductImage: false

    signal clicked;
    signal pressAndHold;
    signal productImageNotFound;
    signal starClicked;
    signal swiped;

    height: 80
    width: parent.width
    color: inverted ? "transparent" : UI.APP_BACKGROUND_COLOR
    MouseArea {
        id: mouseArea
        property bool pressed: false

        anchors.fill: parent
        onPressed: {
            pressed = true
            clickStartY = mouseY
            clickStartX = mouseX
            timer.start()
        }
        onClicked: {
            var diffX = mouseX - clickStartX
            if (diffX < 50 && diffX > -50) {
                container.color = "lightgrey"
                container.clicked()
                timer2.start()
            }
        }
        onPressAndHold: {
            var diffX = mouseX - clickStartX
            if (diffX < 50 && diffX > -50) {
                container.pressAndHold();
            }
        }
        onReleased: {
            container.color = inverted ? "transparent" : UI.APP_BACKGROUND_COLOR
            highlightRemoveTimer.start()
            pressed = "false"
        }
        onCanceled: {
            container.color = inverted ? "transparent" : UI.APP_BACKGROUND_COLOR
            addToQueueRect.state = "hidden"
        }
        onMouseXChanged: {
            var diffX = mouseX - clickStartX
            if (diffX > 70 && diffX <= 120) {
                addToQueueRect.state = "visible"
            } else if (diffX > 120 && pressed) {
                pressed = false
                addToQueueRect.state = "highlight"
                highlightRemoveTimer.start()
                container.swiped()
            } else if (pressed) {
                addToQueueRect.state = "hidden"
            }
        }
    }

    Rectangle {
        id: addToQueueRect
        visible: container.enableAddToQueue
        enabled: visible
        opacity: 0.5
        color: "#67AB16"
        anchors.top: parent.top
        anchors.bottom: parent.bottom
        anchors.right: parent.right
        anchors.left: parent.left
        state: "hidden"

        states: [
            State {
                name: "visible"
                PropertyChanges { target: addToQueueRect; opacity: 0.5 }
            },
            State {
                name: "hidden"
                PropertyChanges { target: addToQueueRect; opacity: 0 }
            },
            State {
                name: "highlight"
                PropertyChanges { target: addToQueueRect; opacity: 1 }
            }
        ]

        transitions: [
            Transition {
                from: "hidden"
                to: "visible"
                NumberAnimation { target: addToQueueRect; properties: "opacity"; duration: 150; easing.type: Easing.InCubic }
            },
            Transition {
                from: "*"
                to: "hidden"
                NumberAnimation { target: addToQueueRect; properties: "opacity"; duration: 300; easing.type: Easing.OutCubic }
            },
            Transition {
                from: "*"
                to: "highlight"
                NumberAnimation { target: addToQueueRect; properties: "opacity"; duration: 300; easing.type: Easing.OutCubic }
            }
        ]
    }

    Timer {
        id: timer
        interval: 70; running: false; repeat: false
        onTriggered: {
            if (Math.abs(mouseArea.mouseY - clickStartY) < 3) {
                container.color = "lightgrey"
            } else {
                container.color = inverted ? "transparent" : UI.APP_BACKGROUND_COLOR
            }
        }
    }

    Timer {
        id: timer2
        interval: 70; running: false; repeat: false
        onTriggered: container.color = inverted ? "transparent" : UI.APP_BACKGROUND_COLOR
    }

    Timer {
        id: highlightRemoveTimer
        interval: 300; running: false; repeat: false
        onTriggered: addToQueueRect.state = "hidden"
    }

    Item {
        id: itemContainer
        anchors.fill: parent
        anchors.verticalCenter: parent.verticalCenter
        anchors.leftMargin: 15
        anchors.rightMargin: 15

        Loader {
            id: _productImageLoader
            sourceComponent: if (imageSource != "" && showProductImage) {
                                _productImageLoader.anchors.verticalCenter = itemContainer.verticalCenter
                                _productImageLoader.anchors.left = itemContainer.left
                                 _productImageLoader.anchors.leftMargin = 13
                                _productImageLoader.height = imageSize
                                _productImageLoader.width = imageSize
                                return _productImageComponent
                             } else {
                                return undefined
                             }
            }

        Loader {
            id: _seenCountRectLoader
            sourceComponent: if (showSeenCount && notSeenTracksCount > 0) {
                                _seenCountRectLoader.anchors.verticalCenter = parent.verticalCenter
                                _seenCountRectLoader.anchors.left = parent.left
                                _seenCountRectLoader.anchors.leftMargin = 150
                                _seenCountRectLoader.width = imageSize
                                _seenCountRectLoader.height = imageSize
                                return _seenCountRectComponent
                             } else {
                                return undefined
                             }
            }

        Column {
            id: detailsColumn
            height: childrenRect.height
            anchors.verticalCenter: parent.verticalCenter
            anchors.left: imageSource != "" ? _productImageLoader.right : parent.left
            anchors.leftMargin: imageSource ? 28 : 0
            anchors.right: allowRightImage ? _iconRightLoader.left : parent.right
            Text {
                property string fontColor: inverted ? "white" : listReady ? labelText1Color : "grey"
                id: text1
                font.pixelSize: UI.FONT_SIZE_LARGE
                font.family: UI.FONT_FAMILY
                font.weight: Font.Bold
                width: parent.width
                elide: Text.ElideRight
                text: labelText1
                color: (fontColor != "") ? fontColor :((!npHighlighther) ? "black" : "#78c043")
                visible: (text != "") ? true : false
                height: visible ? 30 : 0
            }

            Row {
                spacing: 10
                anchors.left: parent.left
                Loader {
                    id: _statusIconLoader
                    sourceComponent: if ( loadingStatus > 0) {
                                         _statusIconLoader.width = 16
                                         _statusIconLoader.height = 16
                                         return _statusIconComponent
                                     } else {
                                         return undefined
                                     }
                }
                Loader {
                    id: _itemProgress
                    sourceComponent: if (loadingStatus == 2) {
                                       // _itemProgress.anchors.left = statusIcon.right
                                        _itemProgress.anchors.leftMargin = 15
                                        _itemProgress.width = 250
                                       // _itemProgress.anchors.anchors.verticalCenter = statusIcon.verticalCenter
                                        return _progressBarComponent
                                     } else {
                                        return undefined
                                     }
                    }
                Text { id: text2
                    visible: if (loadingStatus != 2) {
                                 _itemProgress.width = 0
                                 return true
                             } else {
                                 return false
                             }

                    font.pixelSize: UI.FONT_SIZE_SMALL
                    font.weight: Font.Light
                    font.family: UI.FONT_FAMILY
                    elide: Text.ElideRight
                    text: labelText2
                    height: text!= "" ? 24 : 0
                    width: (labelText3 != "") ? detailsColumn.width - _text3Loader.width : detailsColumn.width
                    color: npHighlighther ? "#51822c" : "grey"

                }
                Loader {
                    id: _text3Loader
                    sourceComponent: if (labelText3 != "") {
                                        return _text3Component
                                    } else {
                                        return undefined
                                    }
                }
            }

        }

        Loader {
            id: _iconRightLoader
            sourceComponent: if (allowRightImage) {
                                 _iconRightLoader.visible = allowRightImage
                                 _iconRightLoader.width = 40
                                 _iconRightLoader.height = 40
                                 _iconRightLoader.anchors.right = itemContainer.right
                                 _iconRightLoader.anchors.verticalCenter = itemContainer.verticalCenter
                                 return _iconRightComponent
                             } else {
                                 return undefined
                             }
        }
    }

    Component {
        id: _iconRightComponent
        Image {
            id: iconRight
            source: if (!showIcon) {
                return ""
            } else if (showNextIcon) {
                return "/usr/share/spotify/pics/drilldown_arrow.png"
            } else {
                return starred ? "/usr/share/spotify/pics/Spotify-19.png" :
                    "/usr/share/spotify/pics/Spotify-23.png"
            }
            MouseArea {
                objectName: "starMouseArea"
                enabled: !showNextIcon
                anchors.right: parent.right
                anchors.left: parent.left
                anchors.rightMargin: -15
                anchors.verticalCenter: parent.verticalCenter
                height: 40
                onReleased: container.starClicked()
            }
        }
    }

    Component {
        id: _statusIconComponent
        Image {
            id: statusIcon
            source: loadingStatus == 1 ? "/usr/share/spotify/pics/indicator_playlist_downloaded.png" :
                    loadingStatus == 2 ? "/usr/share/spotify/pics/indicator_playlist_downloading.png" :
                    loadingStatus == 3 ? "/usr/share/spotify/pics/indicator_playlist_partially_dowloaded.png" :
                    ""
        }
    }

    // seenCountRect
    Component {
           id: _seenCountRectComponent
           Rectangle {
            id: seenCountRect
            color: "Black"
            border.color: "Grey"
            border.width: 2
            radius: 20
            Text {
                text: notSeenTracksCount
                anchors.centerIn: parent
                font.pixelSize: UI.FONT_SIZE_NORMAL
                font.family: UI.FONT_FAMILY
                color: inverted ? "black" : "white"
            }
        }
    }
      // ProductImage
    Component {
         id: _productImageComponent
         Image {
             id: productImage
             asynchronous: true
             source: imageSource
             onStatusChanged: if (productImage.status == Image.Error) {
                 productImageNotFound()
             }
         }
     }
     // ProgressBar
    Component {
            id: _progressBarComponent
            ListItemProgress {
                objectName: "progressBar"
                progress: loadingProgress / 100
            }
        }
    Component {
        id: _text3Component
        PropertiesDetailPanel {
            id: text3
            headerText: labelText3
            alignRight: true
            visible: !allowRightImage
        }
    }
}
