import Qt 4.7
import com.meego 1.0
import "constants.js" as UI

Item {
    MouseArea {
        // Ignore the mouse events from the background
        anchors.fill:  parent
    }

    id: dialog
    property alias button1Label: button1.text
    property alias username: inputUserName.text
    property alias password: inputPassword.text
    property alias tos: tosCheckBox.checked
    property int cellHeight: 50;
    signal logIn

    anchors.fill: parent

    state: "hidden"

    Rectangle {
        anchors.fill: parent
        color: "black"
        opacity: 1
    }

    states: [
        State {
            name: "visible"
            PropertyChanges { target: dialog; opacity: 1 }
            PropertyChanges { target: dialogRectangle; height: 460 }
        },
        State {
            name: "hidden"
            PropertyChanges { target: dialog; opacity: 0 }
            PropertyChanges { target: dialogRectangle; height: 250 }
        },
        State {
            name: "loggingIn"
            PropertyChanges { target: dialog; opacity: 1 }
            PropertyChanges { target: dialogRectangle; height: 250 }
        },
        State {
            name: "loggingOut"
            PropertyChanges { target: dialog; opacity: 1 }
            PropertyChanges { target: dialogRectangle; height: 250 }
        }
    ]

    transitions: [
        Transition {
            from: "*"
            to: "hidden"
            SequentialAnimation {
                NumberAnimation {
                    target: dialog;
                    properties: "opacity";
                    duration: 500
                }
                PropertyAction { target: dialog; property: "visible"; value: false }
            }
        },
        Transition {
            from: "*"
            to: "visible"
            SequentialAnimation {
                NumberAnimation {
                    target: dialogRectangle;
                    properties: "height";
                    duration: 300;
                }
                PropertyAction { target: dialog; property: "visible"; value: true }
                NumberAnimation {
                    target: dialog;
                    properties: "opacity";
                    duration: 300
                }
            }
        },
        Transition {
            from: "*"
            to: "loggingIn,loggingOut"
            NumberAnimation {
                target: dialogRectangle;
                properties: "height";
                duration: 300
            }
            PropertyAction { target: dialog; property: "visible"; value: true }
            NumberAnimation {
                target: dialog;
                properties: "opacity";
                duration: 300
            }
        }
    ]

    Item {
        id: dialogRectangle
        anchors.centerIn: parent
        width: 460
        height: 460

        Rectangle {
            id: background
            anchors.fill: parent
            visible: true
            color: "lightgray"
            radius: 30
        }

        Image {
            id: title
            source: "/usr/share/spotify/pics/spotifylogo_text.png"
            anchors.top: parent.top
            anchors.topMargin: 20
            anchors.horizontalCenter: parent.horizontalCenter
            visible: true
        }

        Item {
            anchors.top: title.bottom
            anchors.bottom: parent.bottom
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.margins: 10

            visible: (dialog.state == "loggingIn" ||
                      dialog.state == "loggingOut" ||
                      dialog.state == "hidden") ? false : true

            Text {
                id: usernameLabel
                height: 30
                text: "Username"
                color: "black"
                anchors.left: inputUserName.left
                anchors.top: parent.top
                verticalAlignment: Text.AlignBottom

                font {
                    family: UI.FONT_FAMILY
                    pixelSize: UI.FONT_SIZE_NORMAL
                }
            }

            TextField {
                objectName: "userNameInputField"
                id: inputUserName
                anchors.top: usernameLabel.bottom
                anchors.horizontalCenter: parent.horizontalCenter
                width: 450
                inputMethodHints: Qt.ImhNoPredictiveText | Qt.ImhNoAutoUppercase
                KeyNavigation.tab: inputPassword
                KeyNavigation.backtab: inputPassword
                font {
                    family: UI.FONT_FAMILY
                    pixelSize: UI.FONT_SIZE_LARGE
                }
                onTextChanged: {
//                    tosCheckBox.checked = false
                }
            }

            Text {
                id: passwordLabel
                height: 40
                text: "Password"
                color: "black"
                anchors.left: inputUserName.left
                anchors.top: inputUserName.bottom
                verticalAlignment: Text.AlignBottom
                font {
                    family: UI.FONT_FAMILY
                    pixelSize: UI.FONT_SIZE_NORMAL
                }
            }

            TextField {
                objectName: "passwordInputField"
                id: inputPassword
                anchors.top: passwordLabel.bottom
                anchors.horizontalCenter: parent.horizontalCenter
                width: 450
                inputMethodHints: Qt.ImhNoPredictiveText
                KeyNavigation.tab: inputUserName
                KeyNavigation.backtab: inputUserName
                echoMode: TextInput.Password
                font {
                    family: UI.FONT_FAMILY
                    pixelSize: UI.FONT_SIZE_LARGE
                }
                onTextChanged: {
//                    tosCheckBox.checked = false
                }

            }

            CheckBox {
                id: tosCheckBox
                anchors.top: inputPassword.bottom
                anchors.topMargin: 25
                anchors.left: inputUserName.left
            }
            Text {
                id: tosText
                anchors.left:  tosCheckBox.right
                anchors.right:  parent.right
                anchors.verticalCenter: tosCheckBox.verticalCenter
                font {
                    family: UI.FONT_FAMILY
                    pixelSize: UI.FONT_SIZE_NORMAL
                }
                wrapMode: Text.WordWrap
                textFormat: Text.RichText
                text: "I have read the <a href=\"http://www.spotify.com/"+settings.GetCountryCode+"/legal/end-user-agreement/\">Terms and Conditions of Use</a> and <a href=\"http://www.spotify.com/"+settings.GetCountryCode+"/legal/mobile-terms-and-conditions/\">Mobile Terms of Use</a>."
                onLinkActivated: {
                    console.log(link + " link activated")
                    settings.openUrlInBrowser(link);
                }
            }

            Button {
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 25
                anchors.horizontalCenter: parent.horizontalCenter
                style: ButtonStyle { inverted: false }
                objectName: "signInButton"
                id: button1
                width: 322
                height: 52
                enabled: tosCheckBox.checked
                onClicked: {
                    inputUserName.platformCloseSoftwareInputPanel();
                    logIn();
                }
            }
        }

        LoadingSpinner {
            id: loadingImage
            visible: (dialog.state == "loggingIn" || dialog.state == "loggingOut") ? true : false
            loadingText: dialog.state == "loggingIn" ? "Logging in..." : "Logging out..."
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            largeSpinner: false
            invertedSpinner: false
        }
    }
}
