import QtQuick 1.1
import com.meego 1.0

Item {
    id: longTapMenuMenu
    anchors.fill: parent
    property bool showing: false
    property bool mouseClickEnabled: true
    property ContextMenuData menuData: ContextMenuData { }
    state: "hidden"

    function open() {
        mouseClickEnabled = true
        state = "visible";
    }

    function close() {
        mouseClickEnabled = false
        state = "hidden";
    }

    Image {
        id: background
        anchors.fill: parent
        source: "/usr/share/spotify/pics/object_menu_background.png"
        state: longTapMenuMenu.state

        states: [
            State {
                name: "visible"; when: showing
                PropertyChanges { target: background; opacity: 1 }
            },
            State {
                name: "hidden"; when: !showing
                PropertyChanges { target: background; opacity: 0 }
            }
        ]
        transitions: [
            Transition {
                from: "*"; to: "visible"
                SequentialAnimation {
                    PropertyAction { target: longTapMenuMenu; property: "visible"; value: true }
                    NumberAnimation { properties: "opacity"; duration: 300; }
                }
            },
            Transition {
                from: "*"; to: "hidden"
                SequentialAnimation {
                    NumberAnimation { properties: "opacity"; duration: 300; }
                    PropertyAction { target: longTapMenuMenu; property: "visible"; value: false }
                }
            }
        ]

        MouseArea {
            anchors.fill: parent
            onClicked:
            {
                close();
            }
        }
    }

    Rectangle {
        id: popupMenuItems
        height: items.height + 20
        width: parent.width
        anchors.centerIn: parent
        color: "transparent"
        state: parent.state
        transform: Scale {
            id: scale
            origin.x: popupMenuItems.width / 2
            origin.y: popupMenuItems.height / 2
        }
        states: [
            State {
                name: "visible"; when: showing
                PropertyChanges { target: scale; xScale: 1; yScale: 1 }
            },
            State {
                name: "hidden"; when: !showing
                PropertyChanges { target: scale; xScale: 0.5; yScale: 0 }
            }
        ]
        transitions: [
            Transition {
                from: "visible"
                to: "hidden"
                NumberAnimation { target: scale; properties: "xScale,yScale"; duration: 150 }
            },
            Transition {
                from: "hidden"
                to: "visible"
                NumberAnimation { target: scale; properties: "xScale,yScale"; duration: 150 }
            }
        ]

        Image {
            id: topGreenLine
            source: "/usr/share/spotify/pics/object_menu_greenline.png"
            width: parent.width - 30
            height: 1
            anchors.bottom: parent.top
            anchors.horizontalCenter: parent.horizontalCenter
        }

        Image {
            id: bottomGreenLine
            source: "/usr/share/spotify/pics/object_menu_greenline.png"
            width: parent.width - 30
            height: 1
            anchors.top: parent.bottom
            anchors.horizontalCenter: parent.horizontalCenter
        }

        ListView {
            id: items
            width: popupMenuItems.width - 30
            height: childrenRect.height
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            interactive: false
            model: menuData.model
            delegate: listDelegate
        }

        Component {
            id: listDelegate
            Rectangle {
                id: desc
                color: "transparent"
                width: items.width
                height: 95
                Text {
                    id: menuItem
                    anchors.bottom: desc.verticalCenter
                    anchors.left: parent.left
                    anchors.leftMargin: 20
                    text: action; color: "White"; font.pointSize: 20; font.bold: true
                }
                CheckBox {
                    id: checkBox
                    anchors.right: parent.right
                    anchors.rightMargin: 10
                    anchors.top: menuItem.top
                    visible: hasCheckBox
                    checked: actionOn
                }

                Text {
                    id: subMenuItem
                    visible: subText != ""
                    anchors.top: menuItem.bottom
                    anchors.left: menuItem.left
                    text: subText
                    color: "Grey"
                    font.pointSize: 18
                }
                MouseArea {
                    anchors.fill: desc
                    enabled: mouseClickEnabled
                    onPressed: desc.color = "lightgrey"
                    onReleased: desc.color = "transparent"
                    onExited: desc.color = "transparent"
                    onClicked: {
                        close();
                        menuData.itemActivated(index)
                    }
                }
            }
        }
    }
}
