import QtQuick 1.1
import com.nokia.meego 1.0
import "constants.js" as UI

Item {
    id: progressBar
    height: 10
    property variant progress: 0
    property variant playedTime: ""
    property variant duration: ""
    property variant songPosition: 0
    property bool tabPressed: false

    signal setPosition;

    function disableDragTarget() {
        mouseRegion.drag.target = undefined
    }

    MouseArea {
        id: progressBarMouseArea
        anchors.horizontalCenter: parent.horizontalCenter
        width: backgroundBar.width
        height: 80
        anchors.top: parent.top
        anchors.topMargin: -20
        onPressed: {
            songPosition = progressBarMouseArea.mouseX / (backgroundBar.x+backgroundBar.width)
            setPosition()
        }
    }

    Rectangle {
        id: backgroundBar
        anchors.centerIn: parent
        height: 10; width: 320
        radius: height/2 - 1
        gradient: Gradient {
            GradientStop { position: 0.9; color: "black" }
            GradientStop { position: 1; color: "lightgrey" }
        }
    }

    Rectangle {
        id: backgroundProgressBar
        anchors.left: backgroundBar.left
        height: parent.height
        width: (progressTab.x <= backgroundBar.x | progressTab.x > (backgroundBar.x+backgroundBar.width)) ? 0: (progressTab.x - x)
        z: 2
        gradient: Gradient {
            GradientStop { position: 0.9; color: "#87c023" }
            GradientStop { position: 1; color: "lightgrey" }
        }
        radius: height/2 - 2
    }

    Item {
        id: progressTab
        height: 50
        width: 1
        z: 3
        anchors.verticalCenter: parent.verticalCenter
        x: backgroundBar.x + (progress < 1 ? backgroundBar.width * progress : backgroundBar.width)

        Rectangle {
            id: progressButton
            visible: duration != ""
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            height: 10; width: 10
            color: tabPressed ? "#DDDDDD" : "White"
            radius: height/2 - 3
            smooth: true
        }

        MouseArea {
            id: mouseRegion
            anchors.horizontalCenter: parent.horizontalCenter
            width: 80
            height: 80
            drag.target: progressTab
            drag.axis: Drag.XAxis
            drag.minimumX: backgroundBar.x
            drag.maximumX: backgroundBar.width + backgroundBar.x
            onPressed: tabPressed = true
            onReleased: {
                if (drag.target == undefined) {
                    drag.target = progressTab
                } else {
                    tabPressed = false
                    songPosition = progressTab.x / (backgroundBar.x+backgroundBar.width)
                    setPosition()
                }
            }
        }
    }

    Text {
        visible: playedTime != "" ? true : false
        text: playedTime
        anchors.verticalCenter: parent.verticalCenter
        anchors.left: parent.left
        color: "grey"
        font.family: UI.FONT_FAMILY
        font.pixelSize: UI.FONT_SIZE_SMALL
        horizontalAlignment: Text.AlignLeft
    }

    Text {
        text: duration
        anchors.verticalCenter: parent.verticalCenter
        anchors.right:  parent.right
        color: "grey"
        font.family: UI.FONT_FAMILY
        font.pixelSize: UI.FONT_SIZE_SMALL
        horizontalAlignment: Text.AlignRight
    }
}
