import QtQuick 1.1

Item {
    property variant scrollArea
    property bool moving: scrollArea.movingVertically
    property bool showHandle: scrollArea.visibleArea.heightRatio < 0.2
    property bool allowFastScrolling: true

    signal scrollingStarted
    signal scrollingStopped

    anchors.right: scrollArea.right
    anchors.top: scrollArea.top
    anchors.bottom: scrollArea.bottom
    visible: moving || handleVisibleTimer.running || mouseArea.pressed
    width: 70
    onMovingChanged: {
        if (!moving)
        handleVisibleTimer.start()
    }

    Timer {
        id: handleVisibleTimer
        interval: 1000; running: false; repeat: false
    }

    Rectangle {
        anchors.right: parent.right
        anchors.top: parent.top
        anchors.bottom: parent.bottom
        width: 7
        smooth: true
        border.color: "dimgrey"; border.width: 1; radius: height/2 - 1
        color: "#2d292a"
    }

    Rectangle {
        id: scrollKnob
        visible: !showHandle || !allowFastScrolling
        enabled: !showHandle || !allowFastScrolling
        height: Math.max(15, scrollArea.visibleArea.heightRatio * scrollArea.height)
        width: 7
        color: "grey"
        radius: 3
        anchors.right: parent.right
        y: scrollArea.visibleArea.yPosition * scrollArea.height < 0 ? 0 :
                scrollArea.visibleArea.yPosition * scrollArea.height
    }


    Image {
        id: scrollKnobHandle
        source: "/usr/share/spotify/pics/fast_scroll.png"
        visible: showHandle && allowFastScrolling
        height: sourceSize.height
        width: sourceSize.width
        anchors.right: parent.right
        y: mouseArea.pressed ? mouseArea.mouseY - height / 2 :
                scrollArea.visibleArea.yPosition * (scrollArea.height - height) +
                scrollArea.visibleArea.yPosition * scrollArea.visibleArea.heightRatio *
                scrollArea.height
    }

    MouseArea {
        id: mouseArea
        visible: showHandle && allowFastScrolling
        enabled: showHandle && allowFastScrolling
        anchors.fill: parent
        drag.axis: Drag.YAxis
        drag.minimumY: 0
        drag.maximumY: parent.height
        onPressed: {
            scrollingStarted()
            scrollingTimer.restart()
        }
        onReleased: {
            scrollingStopped()
            scrollingTimer.stop()
        }
    }

    Timer {
        id: scrollingTimer
        interval: 30; repeat: true; running: false;
        onTriggered: {
            scrollArea.positionViewAtIndex((mouseArea.mouseY / parent.height) * scrollArea.count, ListView.Center)
        }
    }
}
