import QtQuick 1.1
import com.meego 1.0
import "constants.js" as UI

Item {
    id: shareMenu
    width: parent.width
    height: parent.height
    property bool showing: false
    property bool mouseClickEnabled: true
    property string shareType: ""

    state: "hidden"

    signal closeNPPage

    // "0" == ArtistPage, "1" == AlbumPage, "2" == Toolbar (nowplayingpage)
    function open(page) {
        mouseClickEnabled = true
        state = "visible"
        if (page == 1) {
            shareType = "Album"
            listModel.set(1, {"name": "Share Album"})
            listModel.set(1, {"subName": "Share this Album to others"})
        } else if (page == 2) {
            shareType = "Track"
            listModel.set(1, {"name": "Share Track"})
            listModel.set(1, {"subName": "Share this Track to others"})
        }
    }

    function close() {
        mouseClickEnabled = false
        state = "hidden"
    }

    Rectangle {
        id: background
        anchors.fill: parent
        color: "black"
        state: shareMenu.state

        states: [
            State {
                name: "visible"; when: showing
                PropertyChanges { target: background; opacity: 0.8 }
            },
            State {
                name: "hidden"; when: !showing
                PropertyChanges { target: background; opacity: 0 }
            }
        ]
        transitions: [
            Transition {
                from: "*"; to: "visible"
                SequentialAnimation {
                    PropertyAction { target: shareMenu; property: "visible"; value: true }
                    NumberAnimation { properties: "opacity"; duration: 300; }
                }
            },
            Transition {
                from: "*"; to: "hidden"
                SequentialAnimation {
                    NumberAnimation { properties: "opacity"; duration: 300; }
                    PropertyAction { target: shareMenu; property: "visible"; value: false }
                }
            }
        ]

        MouseArea {
            anchors.fill: parent
            onClicked: close();
        }
    }

    ListModel {
        id: listModel
        ListElement {
            name: "Share to Spotify People"
            subName: "Send to a friend's Inbox"
            imagePath: "/usr/share/spotify/pics/spotifylogo.png"
        }
        ListElement {
            name: "Share"
            subName: ""
            imagePath: "/usr/share/spotify/pics/icon-m-toolbar-share.svg"
        }
    }

    Rectangle {
        id: popupMenuItems
        height: items.height + 20
        width: parent.width
        anchors.bottom: parent.bottom
        color: "lightgrey"
        state: parent.state
        transform: Translate {
            id: translate
            x: 0
            y: popupMenuItems.height
        }

        states: [
            State {
                name: "visible"; when: showing
                PropertyChanges { target: translate; y: 0 }
            },
            State {
                name: "hidden"; when: !showing
                PropertyChanges { target: translate; y: popupMenuItems.height }
            }
        ]
        transitions: [
            Transition {
                from: "visible"
                to: "hidden"
                NumberAnimation { target: translate; properties: "y"; duration: 300; easing.type: Easing.InCubic }
            },
            Transition {
                from: "hidden"
                to: "visible"
                NumberAnimation { target: translate; properties: "y"; duration: 300; easing.type: Easing.OutCubic }
            }
        ]

        Rectangle {
            id: bgRect
            height: items.height
            width: parent.width -20
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 10
            color: UI.APP_BACKGROUND_COLOR
            radius: 20

            ListView {
                id: items
                objectName: "listView"
                width: parent.width
                height: 190
                interactive: false
                model: listModel
                delegate: listDelegate
            }

            Component {
                id: listDelegate
                Rectangle {
                    id: desc
                    width: items.width
                    height: 95
                    color: "transparent"
                    Image {
                        id: icon
                        source: imagePath
                        width: 50
                        smooth: true
                        height: width
                        anchors.left: parent.left
                        anchors.leftMargin: 15
                        anchors.verticalCenter: parent.verticalCenter
                    }
                    Text {
                        id: menuItem
                        anchors.bottom: desc.verticalCenter
                        anchors.left: icon.right
                        anchors.leftMargin: 20
                        text: name; color: "black"; font.family: "Nokia Pure Text"; font.pointSize: 20; font.bold: true
                    }
                    Text {
                        id: subMenuItem
                        visible: subName != ""
                        anchors.top: menuItem.bottom
                        anchors.left: menuItem.left
                        text: subName
                        color: "Grey"
                        font.family: "Nokia Pure Text"
                        font.pointSize: 18
                    }
                    Rectangle {
                        id: separator
                        height: 1
                        width: parent.width
                        anchors.bottom: parent.bottom
                        anchors.left: parent.left
                        color: "lightgrey"
                    }
                    MouseArea {
                        anchors.fill: desc
                        enabled: mouseClickEnabled
                        onPressed: desc.color = "lightgrey"
                        onReleased: desc.color = "transparent"
                        onExited: desc.color = "transparent"
                        onClicked: {
                            if (index == 0) {
                                if (shareMenu.shareType == "Track") {
                                    nowPlayingPageController.shareCurrentTrack(true)
                                    closeNPPage()
                                    close()
                                } else if (shareMenu.shareType == "Album") {
                                    albumPageController.shareAlbum(true)
                                    close()
                                }
                                goToPage("SpotifySharePage")
                            } else {
                                if (shareMenu.shareType == "Track") {
                                    nowPlayingPageController.shareCurrentTrack(false)
                                    close()
                                } else if (shareMenu.shareType == "Album") {
                                    albumPageController.shareAlbum(false)
                                    close()
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
