import Qt 4.7
import com.meego 1.0
import "../Components" as Components

Item {
    id: bgImage

    property bool backButtonVisible: pageStack.currentPage.pageName != "PlaylistsPage" &&
                                    pageStack.currentPage.pageName != "SearchPage" &&
                                    pageStack.currentPage.pageName != "DiscoverPage"

    width: parent.width
    state: if (pageStack.currentPage.pageName == "NowPlayingPage") {
        return "np"
    } else if (pageStack.currentPage.pageName == "AlbumPage" ||
        pageStack.currentPage.pageName == "ArtistPage") {
        return "albumAndArtist"
    } else {
        return "full"
    }

    function activateSearch(search) {
        if (pageStack.currentPage.pageName != "SearchPage") {
            goToPage("SearchPage")
            searchPageController.search(search)
        }
    }

    function activatePlaylistsItem() {
        playListsText.color = "white"
        searchText.color = "grey"
        discoverText.color = "grey"
        playListsImageOverlay.visible = true
        searchImageOverlay.visible = false
        discoverImageOverlay.visible = false
    }

    function activateSearchItem() {
        playListsText.color = "grey"
        searchText.color = "white"
        discoverText.color = "grey"
        playListsImageOverlay.visible = false
        searchImageOverlay.visible = true
        discoverImageOverlay.visible = false
    }

    // Regular toolbar
    Image {
        id: fullToolBar
        width: parent.width
        height: parent.height
        source: "/usr/share/spotify/pics/tab_bar_background.png"

        Components.Button {
            id: backBtn
            objectName: "backBtn"
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            imagePath: enabled ? "/usr/share/spotify/pics/icon-m-toolbar-back-white.png" :
                                 "/usr/share/spotify/pics/icon-m-toolbar-back-dimmed-white.png"
            imageSize: 50
            enabled: if (!backButtonVisible || viewMenu.visible) {
                return false
            } else {
                return true
            }
            visible: if (!backButtonVisible) {
                return false
            } else {
                return true
            }
            opacity: enabled ? 1 : 0.3
            Behavior on opacity { NumberAnimation { duration: 250 } }
            onClicked: goToPage("goBack")
        }

        // Playlists button
        Item {
            id: playListsBtn
            anchors.right: searchBtn.left
            anchors.verticalCenter: parent.verticalCenter
            height: parent.height - 10
            width: 95
            enabled: !viewMenu.visible
            opacity: enabled ? 1 : 0.3
            Behavior on opacity { NumberAnimation { duration: 250 } }
            z: 2

            Image {
                id: playListsImage
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.top: parent.top
                anchors.topMargin: 11
                width: sourceSize.width
                height: sourceSize.height
                source: "/usr/share/spotify/pics/tab_bar_playlists.png"
                scale: plMouseRegion.pressed ? 0.8 : 1.0
            }

            Image {
                id: playListsImageOverlay
                anchors.verticalCenter: playListsImage.verticalCenter
                anchors.right: parent.right
                source: "/usr/share/spotify/pics/tab_bar_overlay_l.png"
                visible: true
            }

            Text {
                id: playListsText
                anchors.bottom: playListsBtn.bottom
                anchors.horizontalCenter: parent.horizontalCenter
                text: "Playlists"
                color: "white"
                font.pixelSize: 14
            }
            MouseArea {
                id: plMouseRegion
                objectName: "playListsBtn"
                anchors.fill: parent
                onClicked: {
                    if (pageStack.currentPage.pageName != "PlaylistPage" && !rootWindow.busy) {
                        playListsText.color = "white"
                        searchText.color = "grey"
                        discoverText.color = "grey"
                        playListsImageOverlay.visible = true
                        searchImageOverlay.visible = false
                        discoverImageOverlay.visible = false

                        goToPage("PlaylistsPage")
                    }
                }
            }
        }

        // Search button
        Item {
            id: searchBtn
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            height: parent.height - 10
            width: 95
            enabled: !viewMenu.visible
            opacity: enabled ? 1 : 0.3
            Behavior on opacity { NumberAnimation { duration: 250 } }
            z: 2

            Image {
                id: searchImage
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.top: parent.top
                anchors.topMargin: 11
                width: sourceSize.width
                height: sourceSize.height
                source: "/usr/share/spotify/pics/tab_bar_search.png"
                scale: searchMouseRegion.pressed ? 0.8 : 1.0
            }
            Image {
                id: searchImageOverlay
                anchors.verticalCenter: searchImage.verticalCenter
                anchors.horizontalCenter: parent.horizontalCenter
                source: "/usr/share/spotify/pics/tab_bar_overlay_m.png"
                visible: false
            }

            Text {
                id: searchText
                anchors.bottom: searchBtn.bottom
                anchors.horizontalCenter: parent.horizontalCenter
                text: "Search"
                color: "grey"
                font.pixelSize: 14
            }
            MouseArea {
                id: searchMouseRegion
                objectName: "searchBtn"
                anchors.fill: parent
                onClicked: {
                    if (pageStack.currentPage.pageName != "SearchPage" && !rootWindow.busy) {
                        playListsText.color = "grey"
                        searchText.color = "white"
                        discoverText.color = "grey"
                        playListsImageOverlay.visible = false
                        searchImageOverlay.visible = true
                        discoverImageOverlay.visible = false
                        goToPage("SearchPage")
                    }
                }
            }
        }

        // Discover button
        Item {
            id: discoverBtn
            anchors.left: searchBtn.right
            anchors.verticalCenter: parent.verticalCenter
            height: parent.height - 10
            width: 96
            enabled: !viewMenu.visible
            opacity: enabled ? 1 : 0.3
            Behavior on opacity { NumberAnimation { duration: 250 } }
            z: 2

            Image {
                id: discoverImage
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.top: parent.top
                anchors.topMargin: 11
                width: sourceSize.width
                height: sourceSize.height
                source: "/usr/share/spotify/pics/tab_bar_discover.png"
                scale: discoverMouseRegion.pressed ? 0.8 : 1.0
            }
            Image {
                id: discoverImageOverlay
                anchors.verticalCenter: discoverImage.verticalCenter
                source: "/usr/share/spotify/pics/tab_bar_overlay_r.png"
                visible: false
            }
            Text {
                id: discoverText
                anchors.bottom: discoverBtn.bottom
                anchors.horizontalCenter: parent.horizontalCenter
                text: "Discover"
                color: "grey"
                font.pixelSize: 14
            }
            MouseArea {
                id: discoverMouseRegion
                objectName: "discoverBtn"
                anchors.fill: parent
                onClicked: {
                    if (pageStack.currentPage.pageName != "DiscoverPage" && !rootWindow.busy) {
                        playListsText.color = "grey"
                        searchText.color = "grey"
                        discoverText.color = "white"
                        playListsImageOverlay.visible = false
                        searchImageOverlay.visible = false
                        discoverImageOverlay.visible = true

                        goToPage("DiscoverPage")
                    }
                }
            }
        }

        Components.Button {
            id: menuBtn
            objectName: "menuBtn"
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            imagePath: "/usr/share/spotify/pics/icon-m-toolbar-view-menu-white.svg"
            imageSize: 50
//            visible: (pageStack.currentPage.pageName == "SearchPage" || pageStack.currentPage.pageName == "DiscoverPage") ? false : true
            onClicked: {
                if (pageStack.currentPage.pageName == "TracksOfPlaylistPage") {
                    viewMenu.menuData.model.set(0, {"actionOn": tracksOfPlaylistPageController.PlaylistCollaborative})
                }
                viewMenu.visible ? viewMenu.close() : viewMenu.open("", "", "")
            }
        }
    }

    // Toolbar for NowPlayingPage
    Image {
        id: nowPlayingPageToolBar
        width: parent.width
        height: parent.height
        source: "/usr/share/spotify/pics/toolbar_background.png"

        Components.Button {
            id: npBackBtn
            objectName: "npBackBtn"
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            imagePath: "/usr/share/spotify/pics/icon-m-toolbar-back-white.png"
            imageSize: 50
            enabled: if (viewMenu.visible || shareMenu.visible) {
                return false
            } else {
                return true
            }
            opacity: enabled ? 1 : 0.3
            Behavior on opacity { NumberAnimation { duration: 250 } }
            onClicked: goToPage("goBack")
        }

        Components.Button {
            id: npShareBtn
            objectName: "npShareBtn"
            anchors.left: npBackBtn.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: (parent.width - (width * 4)) / 3
            imagePath: enabled ? "/usr/share/spotify/pics/toolbar_share.png" :
                                 "/usr/share/spotify/pics/toolbar_share.png"
            imageSize: 32
            enabled: !viewMenu.visible
            opacity: enabled ? 1 : 0.3
            Behavior on opacity { NumberAnimation { duration: 250 } }
            onClicked: { shareMenu.visible ? shareMenu.close() : shareMenu.open(2) }
            visible: !nowPlayingPageController.LocalPlayback
        }

        Components.Button {
            id: npFavouriteBtn
            objectName: "npFavouriteBtn"
            anchors.left: npShareBtn.right
            anchors.verticalCenter: parent.verticalCenter
            anchors.leftMargin: (parent.width - (width * 4)) / 3
            imagePath: nowPlayingPageController.Starred ? "/usr/share/spotify/pics/indicator_star_enabled.png" :
                                                          "/usr/share/spotify/pics/indicator_star_disabled.png"
            imageSize: 32
            enabled: if (viewMenu.visible || shareMenu.visible) {
                return false
            } else {
                return true
            }
            opacity: enabled ? 1 : 0.3
            Behavior on opacity { NumberAnimation { duration: 250 } }
            onClicked: nowPlayingPageController.setStarred()
            visible: !nowPlayingPageController.LocalPlayback
        }

        Components.Button {
            id: npMenuBtn
            objectName: "npMenuBtn"
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            imagePath: "/usr/share/spotify/pics/icon-m-toolbar-view-menu-white.svg"
            imageSize: 50
            enabled: !shareMenu.visible
            visible: !nowPlayingPageController.LocalPlayback
            opacity: enabled ? 1 : 0.3
            Behavior on opacity { NumberAnimation { duration: 250 } }
            onClicked: {
                if (bgImage.state == "np") {
                    viewMenu.menuData.model.set(0, {"actionOn": nowPlayingPageController.repeatMode()})
                    viewMenu.menuData.model.set(1, {"actionOn": nowPlayingPageController.isShuffled()})
                }
                viewMenu.visible ? viewMenu.close() : viewMenu.open("", "", "")
            }
        }
    }

    // Toolbar for album- and artistPage
    Image {
        id: albumAndArtistPageToolBar
        width: parent.width
        height: parent.height
        source: "/usr/share/spotify/pics/toolbar_background.png"

        Components.Button {
            id: albumAndArtistBackBtn
            objectName: "albumAndArtistBackBtn"
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            imagePath: "/usr/share/spotify/pics/icon-m-toolbar-back-white.png"
            imageSize: 50
            enabled: !shareMenu.visible
            opacity: enabled ? 1 : 0.3
            Behavior on opacity { NumberAnimation { duration: 250 } }
            onClicked: goToPage("goBack")
        }

        Components.Button {
            id: albumAndArtistShareBtn
            enabled: if (pageStack.currentPage.pageName == "AlbumPage") {
                return !albumPageController.LoadingAlbumData
            } else {
                return !artistPageController.LoadingArtistData
            }
            opacity: enabled ? 1 : 0.3
            objectName: "albumAndArtistShareBtn"
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            imagePath: "/usr/share/spotify/pics/toolbar_share.png"
            imageSize: 32
            onClicked: if (shareMenu.visible) {
                shareMenu.close()
            } else {
                if (pageStack.currentPage.pageName == "AlbumPage") {
                    shareMenu.open(1)
                } else if (pageStack.currentPage.pageName == "ArtistPage") {
                    artistPageController.shareArtist()
                }
            }
        }
    }

    states: [
        State {
            name: "np"
            PropertyChanges { target: bgImage; height: 70 }
            PropertyChanges { target: nowPlayingPageToolBar; opacity: 1 }
            PropertyChanges { target: fullToolBar; opacity: 0 }
            PropertyChanges { target: albumAndArtistPageToolBar; opacity: 0  }
        },
        State {
            name: "full"
            PropertyChanges { target: bgImage; height: 90 }
            PropertyChanges { target: nowPlayingPageToolBar; opacity: 0 }
            PropertyChanges { target: fullToolBar; opacity: 1 }
            PropertyChanges { target: albumAndArtistPageToolBar; opacity: 0 }
        },
        State {
            name: "albumAndArtist"
            PropertyChanges { target: bgImage; height: 70 }
            PropertyChanges { target: nowPlayingPageToolBar; opacity: 0 }
            PropertyChanges { target: fullToolBar; opacity: 0 }
            PropertyChanges { target: albumAndArtistPageToolBar; opacity: 1 }
        }
    ]

    transitions: [
        Transition {
            from: "*"; to: "full"
            NumberAnimation { target: bgImage; properties: "height"; duration: 250 }
            NumberAnimation { target: nowPlayingPageToolBar; properties: "opacity"; duration: 250 }
            NumberAnimation { target: fullToolBar; properties: "opacity"; duration: 500 }
            NumberAnimation { target: albumAndArtistPageToolBar; properties: "opacity"; duration: 250 }
        },
        Transition {
            from: "*"; to: "np"
            NumberAnimation { target: bgImage; properties: "height"; duration: 250 }
            NumberAnimation { target: nowPlayingPageToolBar; properties: "opacity"; duration: 500 }
            NumberAnimation { target: fullToolBar; properties: "opacity"; duration: 250 }
            NumberAnimation { target: albumAndArtistPageToolBar; properties: "opacity"; duration: 250 }
        },
        Transition {
            from: "*"; to: "albumAndArtist"
            NumberAnimation { target: bgImage; properties: "height"; duration: 250 }
            NumberAnimation { target: nowPlayingPageToolBar; properties: "opacity"; duration: 250 }
            NumberAnimation { target: fullToolBar; properties: "opacity"; duration: 250 }
            NumberAnimation { target: albumAndArtistPageToolBar; properties: "opacity"; duration: 500 }
        }
    ]
}
