import QtQuick 1.1
import com.meego 1.0
import "../Components"
import "constants.js" as UI

Item {
    id: viewMenu
    width: parent.width
    height: parent.height
    property bool showing: false
    property bool mouseClickEnabled: true
    property ContextMenuData menuData: ContextMenuData { }
    property string headerText1: "";
    property string headerText2: "";
    property string headerImage: "";
    property bool elideForheaderText1: false

    onElideForheaderText1Changed: {
        if (elideForheaderText1) {
            text1.elide = Text.ElideRight
            text1.wrapMode = Text.NoWrap
        } else {
            text1.wrapMode = Text.WordWrap
            text1.elide = Text.ElideNone
        }
    }

    state: "hidden"

    function open(text1, text2, imgSrc) {
        headerText1 = text1
        headerText2 = text2
        headerImage = imgSrc
        mouseClickEnabled = true
        state = "visible";
    }

    function close() {
        mouseClickEnabled = false
        state = "hidden";
        elideForheaderText1 = false
    }

    Image {
        id: background
        anchors.fill: parent
        source:  "/usr/share/spotify/pics/context_menu_background.png"
        state: viewMenu.state

        states: [
            State {
                name: "visible"; when: showing
                PropertyChanges { target: background; opacity: 1 }
            },
            State {
                name: "hidden"; when: !showing
                PropertyChanges { target: background; opacity: 0 }
            }
        ]
        transitions: [
            Transition {
                from: "*"; to: "visible"
                SequentialAnimation {
                    PropertyAction { target: viewMenu; property: "visible"; value: true }
                    NumberAnimation { properties: "opacity"; duration: 300; }
                }
            },
            Transition {
                from: "*"; to: "hidden"
                SequentialAnimation {
                    NumberAnimation { properties: "opacity"; duration: 300; }
                    PropertyAction { target: viewMenu; property: "visible"; value: false }
                    ScriptAction { script: menuData.menuClosed(); }
                }
            }
        ]

        MouseArea {
            anchors.fill: parent
            onClicked:
            {
                close();
            }
        }
    }

    Rectangle {
        id: popupMenuItems
        height: items.height + headerItem.height + 20
        width: parent.width
        anchors.bottom: parent.bottom
        color: "lightgrey"
        state: parent.state
        transform: Translate {
            id: translate
            x: 0
            y: popupMenuItems.height
        }

//        Text {
//            id: text1
//            text: headerText
//            wrapMode: Text.WordWrap
//            anchors.left: parent.left
//            anchors.leftMargin: 15
//            anchors.right: parent.right
//            anchors.rightMargin: 15
//            anchors.bottom: bgRect.top
//            anchors.bottomMargin: 15
//            color: "black"
//            font.pixelSize: UI.FONT_SIZE_LARGE
//            font.family: UI.FONT_FAMILY
//        }
        Item {
            id: headerItem
            visible: headerText1 != ""
            anchors.left: parent.left
            anchors.leftMargin: 10
            anchors.right: parent.right
            anchors.rightMargin: 10
            anchors.bottom: bgRect.top
            anchors.bottomMargin: 5
            height: headerText1 != "" ?  texts.height < 90 ? 80 : texts.height+10 : 0

            Image {
                id: image
                anchors.verticalCenter: headerItem.verticalCenter
                anchors.left: parent.left
                height: 64
                width: 64
                source: headerImage
                asynchronous: true
            }
            Image {
                id: closeImage
                anchors.right: headerItem.right
                anchors.top:  headerItem.top
                anchors.topMargin: 10
                source: "/usr/share/spotify/pics/context_menu_close.png"
            }

            Item {
                id: texts
                anchors.verticalCenter: image.verticalCenter
                anchors.left: image.right
                anchors.leftMargin: 15
                anchors.right: closeImage.left
                anchors.rightMargin: 5
                height: text1.height + text2.height
                Text {
                    id: text1
                    text: headerText1
                    anchors.left: parent.left
                    anchors.right: parent.right
                    color: "black"
                    font.pixelSize: UI.FONT_SIZE_LARGE
                    font.family: UI.FONT_FAMILY
                    font.weight: Font.Bold
                    wrapMode: Text.WordWrap
                    //elide: Text.ElideRight
                }
                Text {
                    id: text2
                    text: headerText2
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.top: text1.bottom
                    color: "grey"
                    font.pixelSize: UI.FONT_SIZE_SMALL
                    font.family: UI.FONT_FAMILY
                    font.weight: Font.Light
                    elide: Text.ElideRight
                }
             }
        }

        states: [
            State {
                name: "visible"; when: showing
                PropertyChanges { target: translate; y: 0 }
            },
            State {
                name: "hidden"; when: !showing
                PropertyChanges { target: translate; y: popupMenuItems.height }
            }
        ]
        transitions: [
            Transition {
                from: "visible"
                to: "hidden"
                NumberAnimation { target: translate; properties: "y"; duration: 300; easing.type: Easing.InCubic }
            },
            Transition {
                from: "hidden"
                to: "visible"
                NumberAnimation { target: translate; properties: "y"; duration: 300; easing.type: Easing.OutCubic }
            }
        ]

        Rectangle {
            id: bgRect
            height: items.height
            width: parent.width -20
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 10
            color: UI.APP_BACKGROUND_COLOR
            radius: 20

            ListView {
                id: items
                width: parent.width
                height: childrenRect.height
                interactive: false
                model: menuData.model
                delegate: listDelegate
            }

            Component {
                id: listDelegate
                Rectangle {
                    id: desc
                    width: items.width
                    height: 95
                    color: "transparent"

                    property bool itemPressed: false

                    Rectangle {
                        id: topCorners
                        visible: itemPressed
                        anchors.top: parent.top
                        width: desc.width
                        height: 70
                        radius: index == 0 ? 20 : 0
                        color: "#b2b2b4"
                    }
                    Rectangle {
                        id: bottomCorners
                        visible: itemPressed
                        anchors.bottom: parent.bottom
                        width: desc.width
                        height: 70
                        radius: index == items.count - 1 ? 20 : 0
                        color: "#b2b2b4"
                    }
                    Text {
                        id: menuItem
                        anchors.bottom: desc.verticalCenter
                        anchors.bottomMargin: (subText == "") ? -(menuItem.height/2) : 0
                        anchors.left: parent.left
                        anchors.leftMargin: 20
                        text: action; color: "black"; font.family: UI.FONT_FAMILY; font.pointSize: 20; font.bold: true
                    }
                    CheckBox {
                        id: checkBox
                        anchors.right: parent.right
                        anchors.rightMargin: 10
                        anchors.top: menuItem.top
                        visible: hasCheckBox
                        z: 1
                        checked: actionOn
                        onClicked: {
                            menuData.itemActivated(index)
                            close();
                        }
                    }

                    Text {
                        id: subMenuItem
                        visible: subText != ""
                        anchors.top: menuItem.bottom
                        anchors.left: menuItem.left
                        text: subText
                        color: "Grey"
                        font.family: UI.FONT_FAMILY
                        font.pointSize: 18
                    }

                    Rectangle {
                        id: separator
                        height: 1
                        width: parent.width
                        anchors.bottom: parent.bottom
                        anchors.left: parent.left
                        color: "lightgrey"
                    }

                    MouseArea {
                        anchors.fill: desc
                        enabled: mouseClickEnabled
                        onPressed: itemPressed = true
                        onReleased: itemPressed = false
                        onExited: itemPressed = false
                        onClicked: {
                            menuData.itemActivated(index)
                            close();
                        }
                    }
                }
            }
        }
    }
}
