import QtQuick 1.1
import com.meego 1.0
import "../Components" as Components
import "../Components/constants.js" as UI

Page {
    id: wrapper
    property string pageName: "ArtistPage"
    property int longTappedIndex
    property string currentTab: "overview"
    property string currentArtist: ""
    property bool addingTrackToPlaylist

    orientationLock: 1 // PageOrientation.LockPortrait

    onCurrentArtistChanged: {
        viewTitlePanel.headerText = currentArtist
    }

    onStatusChanged: {
        switch(status) {
            case PageStatus.Activating: {
                viewTitlePanel.headerText = currentArtist
            }
        }
    }

    Components.ContextMenuData {
        id: longTapMenuDataTracks
        model: ListModel {
            ListElement {
                action: "Queue"
                subText: "Add this track to Queue"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Add to Playlist"
                subText: "Save this track to Playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "View Album"
                subText: "View tracks of this album"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share Track"
                subText: "Share this track with others"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share to Spotify People"
                subText: "Send to a friend's Inbox"
                hasCheckBox: false
                actionOn: false
            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Queue": {
                    artistPageController.handleAddToQueue(longTappedIndex)
                }
                case "Add to Playlist": {
                    addingTrackToPlaylist = true
                    rootWindow.showDialog("Save to playlist", saveToPlaylistContent)
                }
                case "View Album": {
                    albumPageController.resetAlbumPage()
                    goToPage("AlbumPage")
                    artistPageController.getAlbumData(longTappedIndex)
                }
                case "Share track": {
                    artistPageController.shareLongTappedTrack(longTappedIndex, false)
                }
                case "Share to Spotify People": {
                    artistPageController.shareLongTappedTrack(longTappedIndex, true)
                    goToPage("SpotifySharePage")
                }
            }
        }
    }

    Components.ContextMenuData {
        id: longTapMenuDataAlbums
        model: ListModel {
            ListElement {
                action: "Queue"
                subText: "Add this album to play Queue"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Add to Playlist"
                subText: "Save this album to Playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share Album"
                subText: "Share this album with others"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share to Spotify People"
                subText: "Send to a friend's Inbox"
                hasCheckBox: false
                actionOn: false
            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Queue": {
                    artistPageController.handleAddAlbumToQueue(longTappedIndex)
                }
                case "Add to Playlist": {
                    addingTrackToPlaylist = false
                    rootWindow.showDialog("Save to playlist", saveToPlaylistContent)
                }
                case "Share Album": {
                    artistPageController.shareLongTappedAlbum(longTappedIndex, false)
                }
                case "Share to Spotify People": {
                    artistPageController.shareLongTappedAlbum(longTappedIndex, true)
                    goToPage("SpotifySharePage")
                }
            }
        }
    }

    Components.LoadingSpinner {
        z: 1
        invertedSpinner: true
        visible: artistPageController.LoadingArtistData
        anchors.fill:  parent

    }

    Components.ViewTitlePanel {
        id: viewTitlePanel
        z: 2
        objectName: "viewTitlePanel"
        anchors.top: parent.top
        anchors.topMargin: rootWindow.statusBarHeight
        height: 72
        headerText: "ArtistPage"
        onlineMode: engineLoggedIn
        imageSource: nowPlayingPageController.HasValidTrack ?
                "/usr/share/spotify/pics/Spotify-31.png" : ""
        imageSourcePressed: nowPlayingPageController.HasValidTrack ?
                        "/usr/share/spotify/pics/Spotify-61.png" : ""
        onClicked: {
            if (nowPlayingPageController.HasValidTrack) {
                rootWindow.showNPSheet()
            }
        }
    }

    Item {
        id: headerArea
        width: parent.width
        anchors.top: viewTitlePanel.bottom
        height: childrenRect.height

        Rectangle {
            id: details
            color:  UI.APP_BACKGROUND_COLOR
            height: (artistPageController.ImagePath == "" ) ? 0 : 200

            width: parent.width
            anchors.top: parent.top
            Behavior on height {
                     NumberAnimation {
                         duration: 450
                     }
                 }
            PathView {
                id: artistArts
                objectName: "artistArts"
                anchors.fill: parent
                anchors.leftMargin:5
                anchors.rightMargin: 5
                MouseArea {
                    anchors.fill: parent
                    onClicked:  {
                        if (artistArts.height == 420) {
                            pathIdBehavior.duration = 500
                            pathIdPLBehavior.duration = 500
                            details.height = 200
                            artistArts.height = 200
                            pathId.startX = -150
                            pathIdPL.x = details.width + 150
                        } else {
                            pathIdBehavior.duration = 350
                            pathIdPLBehavior.duration = 350
                            details.height = 420
                            artistArts.height = 420
                            pathId.startX = -details.width*2
                            pathIdPL.x = details.width*3
                        }
                    }
                }
                preferredHighlightBegin: 0.5
                preferredHighlightEnd: 0.5
                model: artistArtsModel
                path: Path {
                    id: pathId
                    startX: -150
                    Behavior on  startX{
                             NumberAnimation {
                                 id: pathIdBehavior
                                 duration: 300
                             }
                         }
                    startY: details.height / 2
                    PathAttribute { name: "iconScale"; value: 0.8 }
                    PathLine { x: details.width / 2; y: details.height / 2 }
                    PathAttribute { name: "iconScale"; value: (artistArts.height != 420) ? 1.2 : 1.0}
                    PathLine {
                        id:pathIdPL
                        x: details.width + 150
                        y: details.height / 2
                        Behavior on x{
                                 NumberAnimation {
                                     id: pathIdPLBehavior
                                     duration: 300
                                 }
                             }
                    }
                    PathAttribute { name: "iconScale"; value: 0.8 }
                 }
                pathItemCount: (artistArts.height != 420) ? 5 : 5
                delegate: Image {
                            id: artistImage
                            source:modelData
                            scale: PathView.iconScale
                            fillMode: Image.PreserveAspectFit
                            smooth: true
                            height: parent.height
                            width: height
                        }
            }
        }

        Rectangle {
            id: tabs
            color: "transparent"
            height: 50
            width: parent.width
            anchors.top: details.bottom

            Rectangle {
                id: vertSeparator1
                z:2
                objectName: "vertSeparator1"
                height: 1
                width: parent.width
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: parent.top
                color: "grey"
            }

            Image {
                id: arrow
                anchors.verticalCenter: tabs.top
                anchors.horizontalCenter: if (currentTab == "topTracks") {
                    return topTracks.horizontalCenter
                } else if (currentTab == "overview") {
                    return overview.horizontalCenter
                } else if (currentTab == "biography") {
                    return biography.horizontalCenter
                } else {
                    return related.horizontalCenter
                }
                source: "/usr/share/spotify/pics/icon-m-framework-viewmenu-arrow.svg"
                asynchronous: true
            }

            Rectangle {
                id: topTracks
                width: parent.width / 4
                height: parent.height
                anchors.left: parent.left
                anchors.top: vertSeparator1.top
                color: currentTab == "topTracks" ? "#4f4f4f" : "#232323"
                Text {
                    id: topTracksText
                    anchors.bottom: parent.bottom
                    anchors.bottomMargin: 10
                    anchors.horizontalCenter: parent.horizontalCenter
                    text: "Top Hits"
                    color: currentTab == "topTracks" ? "white" : "grey"
                    font.pixelSize: UI.FONT_SIZE_NORMAL
                    font.family: UI.FONT_FAMILY
                }
                Rectangle {
                    width: 1
                    height: parent.height
                    anchors.right: parent.right
                    anchors.bottom: parent.bottom
                    color: "grey"
                }
                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        currentTab = "topTracks"
                    }
                }
            }
            Rectangle {
                id: overview
                width: parent.width / 4
                height: parent.height
                anchors.left: topTracks.right
                anchors.top: vertSeparator1.top
                color: currentTab == "overview" ? "#4f4f4f" : "#232323"
                Text {
                    id: overviewText
                    anchors.bottom: parent.bottom
                    anchors.bottomMargin: 10
                    anchors.horizontalCenter: parent.horizontalCenter
                    text: "Music"
                    color: currentTab == "overview" ? "white" : "grey"
                    font.pixelSize: UI.FONT_SIZE_NORMAL
                    font.family: UI.FONT_FAMILY
                }
                Rectangle {
                    width: 1
                    height: parent.height
                    anchors.right: parent.right
                    anchors.bottom: parent.bottom
                    color: "grey"
                }
                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        currentTab = "overview"
                    }
                }
            }
            Rectangle {
                id: related
                width: parent.width / 4
                height: parent.height
                anchors.left: overview.right
                anchors.top: vertSeparator1.top
                color: currentTab == "related" ? "#4f4f4f" : "#232323"
                Text {
                    id: relatedText
                    anchors.bottom: parent.bottom
                    anchors.bottomMargin: 10
                    anchors.horizontalCenter: parent.horizontalCenter
                    text: "Related"
                    color: currentTab == "related" ? "white" : "grey"
                    font.pixelSize: UI.FONT_SIZE_NORMAL
                    font.family: UI.FONT_FAMILY
                }
                Rectangle {
                    width: 1
                    height: parent.height
                    anchors.right: parent.right
                    anchors.bottom: parent.bottom
                    color: "grey"
                }
                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        currentTab = "related"
                    }
                }
            }
            Rectangle {
                id: biography
                width: parent.width / 4
                height: parent.height
                anchors.left: related.right
                anchors.top: vertSeparator1.top
                color: currentTab == "biography" ? "#4f4f4f" : "#232323"
                Text {
                    id: biographyText
                    anchors.bottom: parent.bottom
                    anchors.bottomMargin: 10
                    anchors.horizontalCenter: parent.horizontalCenter
                    text: "Bio"
                    color: currentTab == "biography" ? "white" : "grey"
                    font.pixelSize: UI.FONT_SIZE_NORMAL
                    font.family: UI.FONT_FAMILY
                }
                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        currentTab = "biography"
                    }
                }
            }
        }
        Rectangle {
            width: parent.width
            height: 2
            anchors.left: parent.left
            anchors.bottom: tabs.bottom
            color: "grey"
        }
    }

    Item {
        id: artistDataArea
        width: parent.width
        anchors.top: headerArea.bottom
        anchors.bottom: parent.bottom
        anchors.bottomMargin: albumAndArtistPageToolBar.height

        Item {
            id: topTracksArea
            visible: currentTab == "topTracks"
            anchors.fill: parent

            Components.NoContent {
                anchors.fill: parent
                visible: myList.count  == 0
                noContentText: "No top tracks found."
            }
            Components.ScrollBar {
                z: 2
                scrollArea: myList
            }

            ListView {
                id: myList
                anchors.fill: parent
                model: topTracksOfArtistModel
                clip: true
                delegate: Components.ListItem {
                    labelText1: TrackTitle
                    labelText2: TrackDurationInString
                    starred: TrackStarred
                    enableAddToQueue: true
                    showProductImage: true
                    onClicked: {
                        artistPageController.topListClicked(index)
                        goToPage("NowPlayingPage")
                    }
                    onStarClicked: {
                        artistPageController.setStarred(index)
                    }
                    onPressAndHold: {
                        viewMenu.menuData = longTapMenuDataTracks
                        longTappedIndex = index;
                        viewMenu.open(labelText1, TrackArtist, TrackAlbumArt)
                    }
                    onSwiped: artistPageController.handleAddToQueue(index)
                }
            }
        }

        Item {
            id: overviewArea
            visible: currentTab == "overview"
            anchors.fill: parent
            Components.NoContent {
                anchors.fill: parent
                visible: albumsList.count == 0
                noContentText: "No albums found."
            }
            Components.ScrollBar {
                z: 2
                scrollArea: albumsList
                onScrollingStarted: searchPageModel.setFastScroll(true)
                onScrollingStopped: {
                    albumsOfArtistModel.setFastScroll(false)
                    albumsOfArtistModel.setMoving(false)
                    albumsOfArtistModel.loadAlbumArts()
                }
            }

            ListView {
                id: albumsList
                anchors.fill: parent
                model: albumsOfArtistModel
                clip: true
                section.property: "AlbumType"
                section.criteria: ViewSection.FullString
                section.delegate: Components.GroupHeaderDivider {
                    headerText: section
                }
                property real contentBottom: contentY + height
                onMovementEnded: {
                    albumsOfArtistModel.setMoving(false)
                    albumsOfArtistModel.loadAlbumArts()
                }
                onMovementStarted: albumsOfArtistModel.setMoving(true)

                delegate: Components.ListItem {
                    id: listItem
                    imageSource: AlbumArt == "" ? "/usr/share/spotify/pics/Spotify-03.png" : AlbumArt
                    labelText1: AlbumTitle
                    labelText2: AlbumReleaseYear
                    allowRightImage: false
                    showProductImage: true
                    onClicked: {
                        albumPageController.resetAlbumPage()
                        goToPage("AlbumPage")
                        artistPageController.albumListClicked(index)
                    }
                    property bool isVisible : (((index * height) >= albumsList.contentY &&
                                                (index * height) <= albumsList.contentBottom) ||
                                               ((index * height) + height >= albumsList.contentY &&
                                                (index * height) + height <= albumsList.contentBottom))
                    onIsVisibleChanged: {
                        if (isVisible && albumsOfArtistModel) {
                            albumsOfArtistModel.setItemVisible(isVisible, index);
                        } else if (!isVisible && albumsOfArtistModel) {
                            albumsOfArtistModel.setItemVisible(isVisible, index);
                        }
                    }
                    onPressAndHold: {
                        viewMenu.menuData = longTapMenuDataAlbums
                        longTappedIndex = index;
                        viewMenu.open(labelText1, currentArtist, imageSource)
                    }
                }
            }
        }

        Components.NoContent {
            anchors.fill: biographyArea
            visible: currentTab == "biography" && artistPageController.Biography == ""
            noContentText: "No biography found."
        }
        Components.ScrollBar {
            z: 2
            scrollArea: biographyArea
            allowFastScrolling: false
        }

        Flickable {
            id: biographyArea
            visible: currentTab == "biography"
            anchors.fill: parent
            contentHeight: bgText.height + 20
            maximumFlickVelocity: 800
            flickableDirection: Flickable.VerticalFlick
            clip: true

            Text {
                id: bgText
                text: artistPageController.Biography
                wrapMode: Text.WordWrap
                textFormat: Text.RichText
                anchors.left: parent.left
                anchors.leftMargin: 15
                anchors.right: parent.right
                anchors.rightMargin: 15
                anchors.top: parent.top
                anchors.topMargin: 15
                anchors.bottomMargin: 15
                color: "black"
                font.pixelSize: UI.FONT_SIZE_NORMAL
                font.family: UI.FONT_FAMILY
                onLinkActivated: artistPageController.linkActivated(link)
            }
        }

        Connections {
            target: artistPageController
            onAlbumLinkClicked: {
                albumPageController.resetAlbumPage()
                goToPage("AlbumPage")
            }
            onArtistLinkClicked: {
                artistPageController.clearData()
                currentTab = "overview"
            }
            onArtistNameChanged: currentArtist = name
            onSearchLinkClicked: {
                //toolbar.activateSearch(search)
            }
            onCurrentTabChanged: currentTab = tab
        }

        Item {
            id: relatedArea
            visible: currentTab == "related"
            anchors.fill: parent
            Components.NoContent {
                anchors.fill: parent
                visible: relatedList.count == 0
                noContentText: "No related artists found."
            }
            Components.ScrollBar {
                z: 2
                scrollArea: relatedList
            }

            ListView {
                id: relatedList
                anchors.fill: parent
                model: similarArtistsModel
                clip: true
                delegate: Components.ListItem {
                    id: listItem
                    imageSource: AlbumArt == "" ? "/usr/share/spotify/pics/Spotify-03.png" : AlbumArt
                    labelText1: ArtistTitle
                    labelText2: ""// TODO: ArtistReleaseYears
                    allowRightImage: false
                    showProductImage: true
                    onClicked: {
                            artistPageController.clearData()
                            currentTab = "overview"
                            artistPageController.relatedListClicked(index)
                        }
                    }
                }
            }

    }

    Component {
        id: saveToPlaylistContent
        Item {
            height: childrenRect.height
            width: dialogLoader.status ==  Loader.Ready ? dialogLoader.item.contentWidth : 0

            ListView {
                id: playlists
                objectName: "playlists"
                clip: true
                width: parent.width
                height: Math.max(150, Math.min(600, count * 80 + 80))
                model: editablePlaylistsModel
                header: headerComponent
                delegate: Components.ListItem {
                    inverted: true
                    labelText1: playlistName
                    showIcon: false
                    onClicked: {
                        artistPageController.saveToPlaylist(index, longTappedIndex, addingTrackToPlaylist)
                        rootWindow.hideDialog()
                    }
                }

                Component {
                    id: headerComponent
                    Components.ListItem {
                        inverted: true
                        labelText1: "<Add as a new playlist>"
                        labelText2: artistPageController.longTappedItem(longTappedIndex, addingTrackToPlaylist)
                        showIcon: false
                        onClicked: {
                            artistPageController.saveToPlaylist(-1, longTappedIndex, addingTrackToPlaylist)
                            rootWindow.hideDialog()
                        }
                    }
                }
            }
        }
    }
}
