import QtQuick 1.1
import com.meego 1.0
import "../Components"

Page {
    id: wrapper
    property string pageName: "DiscoverPage"
    property string albumArtDefaultImg: "/usr/share/spotify/pics/Spotify-01.png"
    property string artistArtDefaultImg: "/usr/share/spotify/pics/Spotify-01.png"
    property int longTappedIndex
    property string artistOfLongTappedItem: ""
    property bool addingTrackToPlaylist
    orientationLock: 1 // PageOrientation.LockPortrait

    onStatusChanged: {
        switch(status) {
            case PageStatus.Active: {
                viewTitlePanel.headerText = "What's New"
            }
            case PageStatus.Activating: {
                discoverPageController.setVisible(true)
            }
            case PageStatus.Deactivating: {
                discoverPageController.setVisible(false)
            }
        }
    }

    ContextMenuData {
        id: longTapMenuDataTracks
        model: ListModel {
            ListElement {
                action: "Queue"
                subText: "Add this track to Queue"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Add to Playlist"
                subText: "Save this track to Playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "View Album"
                subText: "View tracks of this album"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "View Artist"
                subText: "More from this artist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share Track"
                subText: "Share this track with others"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share to Spotify People"
                subText: "Send to a friend's Inbox"
                hasCheckBox: false
                actionOn: false
            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Queue": {
                    discoverPageController.handleAddToQueue(longTappedIndex)
                }
                case "Add to Playlist": {
                    addingTrackToPlaylist = true
                    rootWindow.showDialog("Save to playlist", saveToPlaylistContent)
                }
                case "View Album": {
                    albumPageController.resetAlbumPage()
                    goToPage("AlbumPage")
                    discoverPageController.getAlbumData(longTappedIndex)
                }
                case "View Artist": {
                    artistPageController.clearData()
                    goToPage("ArtistPage")
                    discoverPageController.getArtistData(false, longTappedIndex)
                }
                case "Share Track": {
                    discoverPageController.shareLongTappedTrack(longTappedIndex, false)
                }
                case "Share to Spotify People": {
                    discoverPageController.shareLongTappedTrack(longTappedIndex, true)
                    goToPage("SpotifySharePage")
                }
            }
        }
    }

    ContextMenuData {
        id: longTapMenuDataAlbums
        model: ListModel {
            ListElement {
                action: "Queue"
                subText: "Add this album to Queue"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Add to Playlist"
                subText: "Save this album to Playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "View Artist"
                subText: "More from this artist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share Album"
                subText: "Share this album with others"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share to Spotify People"
                subText: "Send to a friend's Inbox"
                hasCheckBox: false
                actionOn: false
            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Queue": {
                    discoverPageController.handleAddAlbumToQueue(longTappedIndex)
                }
                case "Add to Playlist": {
                    addingTrackToPlaylist = false
                    rootWindow.showDialog("Save to playlist", saveToPlaylistContent)
                }
                case "View Artist": {
                    artistPageController.clearData()
                    goToPage("ArtistPage")
                    discoverPageController.getArtistData(true, longTappedIndex)
                }
                case "Share Album": {
                    discoverPageController.shareLongTappedAlbum(longTappedIndex, false)
                }
                case "Share to Spotify People": {
                    discoverPageController.shareLongTappedAlbum(longTappedIndex, true)
                    goToPage("SpotifySharePage")
                }
            }
        }
    }

    ScrollBar {
        id: scrollBar
        z: 2
        objectName: "scrollBar"
        scrollArea: discoverView
        allowFastScrolling: false
    }

    ViewTitlePanel {
        id: viewTitlePanel
        z: 2
        objectName: "viewTitlePanel"
        anchors.top: parent.top
        anchors.topMargin: rootWindow.statusBarHeight
        height: 72
        headerText: "What's New"
        onlineMode: engineLoggedIn
        imageSource: nowPlayingPageController.HasValidTrack ?
                "/usr/share/spotify/pics/Spotify-31.png" : ""
        imageSourcePressed: nowPlayingPageController.HasValidTrack ?
                        "/usr/share/spotify/pics/Spotify-61.png" : ""
        onClicked: {
            if (nowPlayingPageController.HasValidTrack) {
                rootWindow.showNPSheet()
            }
        }
    }

    Flickable {
        id: discoverView
        objectName: "discoverView"
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: viewTitlePanel.bottom
        anchors.bottom:  parent.bottom
        flickableDirection: Flickable.VerticalFlick
        contentHeight: childrenRect.height

        Item {
            id: flickableComponent
            objectName: "flickableComponent"
            height: childrenRect.height
            width: parent.width

            GroupHeaderDivider {
                id: topTracksHeader
                objectName: "topTracksHeader"
                headerText: "Top Tracks"
            }

            PathView {
                id: topTracks
                objectName: "topTracks"
                anchors.top: topTracksHeader.bottom
                anchors.topMargin: 3
                width: parent.width
                height: 120
                model: topTracksModel
                pathItemCount: 6
                path: Path {
                    startX: -60; startY: topTracks.height / 2
                    PathLine { x: topTracks.width + 180; y: topTracks.height / 2 }
                }

                delegate: OverlayLabel {
                    id: topTrackItem
                    objectName: "topTrackItem"
                    imageSource: AlbumArt == "" ? albumArtDefaultImg : AlbumArt
                    labelText: TitleRole
                    width: 120; height: 120;
                    visible: if ((index == topTracks.currentIndex || index == topTracks.currentIndex + 5)
                                 && wrapper.status != PageStatus.Active) {
                                 return false
                             } else { return true }
                    onClicked:  {
                        discoverPageController.topTracksClicked(index)
                        goToPage("NowPlayingPage")
                    }
                    onPressAndHold: {
                        viewMenu.menuData = longTapMenuDataTracks
                        artistOfLongTappedItem = ArtistRole
                        longTappedIndex = index;
                        viewMenu.open(labelText, SubTitleRole, imageSource)
                    }
                }
            }

            GroupHeaderDivider {
                id: topAlbumsHeader
                objectName: "topAlbumsHeader"
                anchors.top: topTracks.bottom
                headerText: "Top Albums"
            }

            PathView {
                id: topAlbums
                objectName: "topAlbums"
                anchors.top: topAlbumsHeader.bottom
                anchors.topMargin: 3
                width: parent.width
                height: 120
                model: topAlbumsModel
                pathItemCount: 6
                path: Path {
                    startX: -60; startY: topAlbums.height / 2
                    PathLine { x: topAlbums.width + 180; y: topAlbums.height / 2 }
                }

                delegate: OverlayLabel {
                    id: topAlbumsItem
                    objectName: "topAlbumsItem"
                    imageSource: AlbumArt == "" ? albumArtDefaultImg : AlbumArt
                    labelText: TitleRole
                    width: 120; height: 120;
                    visible: if ((index == topAlbums.currentIndex || index == topAlbums.currentIndex + 5)
                                 && wrapper.status != PageStatus.Active) {
                             return false
                         } else { return true }
                    onClicked:  {
                        albumPageController.resetAlbumPage()
                        goToPage("AlbumPage")
                        discoverPageController.topAlbumsClicked(index)
                    }
                    onPressAndHold: {
                        viewMenu.menuData = longTapMenuDataAlbums
                        artistOfLongTappedItem = ArtistRole
                        longTappedIndex = index;
                        viewMenu.open(labelText, ArtistRole, AlbumArt)
                    }
                }
            }

            GroupHeaderDivider {
                id: topArtistsHeader
                objectName: "topArtistsHeader"
                anchors.top: topAlbums.bottom
                headerText: "Top Artists"
            }

            PathView {
                id: topArtists
                objectName: "topArtists"
                anchors.top: topArtistsHeader.bottom
                anchors.topMargin: 3
                width: parent.width
                height: 120
                model: topArtistsModel
                pathItemCount: 6
                path: Path {
                    startX: -60; startY: topArtists.height / 2
                    PathLine { x: topArtists.width + 180; y: topArtists.height / 2 }
                }

                delegate: OverlayLabel {
                    id: topArtistItem
                    objectName: "topAlbumsItem"
                    imageSource: AlbumArt == "" ? artistArtDefaultImg : AlbumArt
                    labelText: TitleRole
                    width: 120; height: 120;
                    visible: if ((index == topArtists.currentIndex || index == topArtists.currentIndex + 5)
                                 && wrapper.status != PageStatus.Active) {
                                 return false
                             } else { return true }

                    onClicked:  {
                        artistPageController.clearData()
                        goToPage("ArtistPage")
                        artistPageController.setTab("overview")
                        discoverPageController.topArtistsClicked(index)
                    }
                }
            }

            GroupHeaderDivider {
                id: peopleHeader
                objectName: "peopleHeader"
                anchors.top: topArtists.bottom
                headerText: "People"
            }
            ListItem {
                id: spotifySocial
                objectName: "spotifySocial"
                anchors.top: peopleHeader.bottom
                imageSource: "/usr/share/spotify/pics/Spotify-07.png"
                labelText1: "Spotify Social"
                labelText2: discoverPageController.FriendsCount + " Friends"
                allowRightImage: true
                showNextIcon: true
                showProductImage: true
                listReady: discoverPageController.FriendsCount > 0 ? true : false
                onClicked: {
                    if (listReady) {
                        goToPage("FriendListPage")
                        engine.getFriendList()
                    }
                }
            }

//            GroupHeaderDivider {
//                id: feedHeader
//                objectName: "feedHeader"
//                anchors.top: spotifySocial.bottom
//                headerText: "Feed"
//            }
        }
    }

    Component {
        id: saveToPlaylistContent
        Item {
            height: childrenRect.height
            width: dialogLoader.status ==  Loader.Ready ? dialogLoader.item.contentWidth : 0

            ListView {
                id: playlists
                objectName: "playlists"
                clip: true
                width: parent.width
                height: Math.max(150, Math.min(600, count * 80 + 80))
                model: editablePlaylistsModel
                header: headerComponent
                cacheBuffer: 800
                delegate: ListItem {
                    inverted: true
                    labelText1: playlistName
                    showIcon: false
                    onClicked: {
                        discoverPageController.saveToPlaylist(index, longTappedIndex, addingTrackToPlaylist)
                        rootWindow.hideDialog()
                    }
                }

                Component {
                    id: headerComponent
                    ListItem {
                        inverted: true
                        labelText1: "<Add as a new playlist>"
                        labelText2: discoverPageController.longTappedItem(longTappedIndex, addingTrackToPlaylist)
                        showIcon: false
                        onClicked: {
                            discoverPageController.saveToPlaylist(-1, longTappedIndex, addingTrackToPlaylist)
                            rootWindow.hideDialog()
                        }
                    }
                }
            }
        }
    }
}
