import QtQuick 1.1
import com.meego 1.0
import "../Components"

Page {
    id: wrapper
    property string pageName: "FriendListPage"
    orientationLock: 1 // PageOrientation.LockPortrait

    onStatusChanged: {
        switch(status) {
            case PageStatus.Activating: {
                viewTitlePanel.headerText = "Spotify Social"
                friendListController.setVisible(true)
            }
            case PageStatus.Deactivating: {
                friendListController.setVisible(false)
            }
        }
    }

    ScrollBar {
        id: scrollBar
        z: 1
        objectName: "scrollBar"
        scrollArea: myList
    }

    ViewTitlePanel {
        id: viewTitlePanel
        z: 2
        objectName: "viewTitlePanel"
        anchors.top: parent.top
        anchors.topMargin: rootWindow.statusBarHeight
        height: 72
        headerText: "FriendListPage"
        onlineMode: engineLoggedIn
        imageSource: nowPlayingPageController.HasValidTrack ?
                "/usr/share/spotify/pics/Spotify-31.png" : ""
        imageSourcePressed: nowPlayingPageController.HasValidTrack ?
                        "/usr/share/spotify/pics/Spotify-61.png" : ""
        onClicked: {
            if (nowPlayingPageController.HasValidTrack) {
                rootWindow.showNPSheet()
            }
        }
    }

    ListView {
        id: myList
        objectName: "friendList"
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: viewTitlePanel.bottom
        anchors.bottom:  parent.bottom
        anchors.bottomMargin: albumAndArtistPageToolBar.height
        model: friendListModel
        cacheBuffer: 800

        delegate: ListItem {
            objectName: "friendItem"
            imageSource: FriendProfilePicture ? FriendProfilePicture : "/usr/share/spotify/pics/Spotify-07.png"
            labelText1: FriendTitle
            labelText2: FriendPlaylistCount > 0 ? FriendPlaylistCount + " playlists" : ""
            allowRightImage: false
            showProductImage: true
            listReady: FriendLoaded ? true : false
            onClicked: {
                if (listReady) {
                    friendListController.listClicked(index)
                    goToPage("ProfilePage")
                }
            }
        }
    }
}
