import QtQuick 1.1
import com.meego 1.0
import "../Components"
import "../Components/constants.js" as UI

Page {
    orientationLock: 1 // PageOrientation.LockPortrait
    property string pageName: "LocalSongsPage"

    onStatusChanged: {
        switch(status) {
            case PageStatus.Activating: {
                viewTitlePanel.headerText = "Local Files"
                //console.log("### activating")
            }
            case PageStatus.Active: {
                //console.log("### active")
            }
            case PageStatus.Deactivating: {
                //console.log("### deactivating")
                //localPlayer.stop()
            }
            case PageStatus.Inactive: {
                //console.log("### inactive")
            }
        }
    }

    ScrollBar {
        id: scrollBar
        z: 1
        objectName: "scrollBar"
        scrollArea: trackList
    }

    ViewTitlePanel {
        id: viewTitlePanel
        z: 2
        objectName: "viewTitlePanel"
        anchors.top: parent.top
        anchors.topMargin: rootWindow.statusBarHeight
        height: 72
        headerText: "LocalSongsPage"
        onlineMode: engineLoggedIn
        imageSource: nowPlayingPageController.HasValidTrack ?
                "/usr/share/spotify/pics/Spotify-31.png" : ""
        imageSourcePressed: nowPlayingPageController.HasValidTrack ?
                        "/usr/share/spotify/pics/Spotify-61.png" : ""
        onClicked: {
            if (nowPlayingPageController.HasValidTrack) {
                rootWindow.showNPSheet()
            }
        }
    }
    Rectangle {
        id: searchContainer
        z:2
        color: "black"
        width: parent.width
        anchors.top: viewTitlePanel.bottom
        height: 90

        TextField {
            objectName: "textField"
            id: textField
            width: parent.width - 40
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            placeholderText: "Search"
            style: TextFieldStyle { paddingRight: clearButton.width }
            font {
                family: UI.FONT_FAMILY
                pixelSize: UI.FONT_SIZE_LARGE
            }
            platformSipAttributes: SipAttributes {
                actionKeyLabel: "Search"
                actionKeyHighlighted: true
            }
            Keys.onReturnPressed: {
                textField.platformCloseSoftwareInputPanel()
                if (textField.text != "") {

                    // Start search here

                }
            }
            onTextChanged: text == "" ? clearButton.state = "hidden" :
                                        clearButton.state = "visible"

            Image {
                id: clearButton
                objectName: "clearButton"
                anchors.verticalCenter: parent.verticalCenter
                anchors.right: parent.right
                anchors.rightMargin: 5
                source: "/usr/share/spotify/pics/icon-m-input-clear.svg"
                scale: mouseArea.pressed ? 0.8 : 1.0
                opacity: 0
                state: "hidden"

                MouseArea {
                    id: mouseArea
                    anchors.fill: parent
                    onClicked: textField.text = ""
                }

                states:  [
                    State {
                        name: "visible"
                        PropertyChanges { target: clearButton; opacity: 1 }
                    },
                    State {
                        name: "hidden"
                        PropertyChanges { target: clearButton; opacity: 0 }
                    }
                ]
                transitions: [
                    Transition {
                        NumberAnimation { property: "opacity"; easing.type: Easing.Linear; duration: 300 }
                    }
                ]
            }
        }

        Rectangle {
            id: separator
            objectName: "separator"
            width: parent.width
            height: 2
            anchors.bottom: searchContainer.bottom
            color: "dimgrey"
        }
    }

    ListView {
        id: trackList
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: searchContainer.bottom
        anchors.bottom:  parent.bottom
        anchors.bottomMargin: albumAndArtistPageToolBar.height
        model: localSongsModel
        cacheBuffer: 800

        delegate: ListItem {
            objectName: "playlistItem"
            visible: index == 0 || index == localSongsModel.rowCount() - 1 ? false : true
            height: index == 0 || index == localSongsModel.rowCount() - 1 ? 0 : 80
            labelText1: TrackTitle
            labelText2: TrackArtist + " | " + TrackAlbum
            labelText3: lengthColumn
            allowRightImage: false
            onClicked: {
                localPlayer.loadMedia(index - 1, true)
                goToPage("NowPlayingPage")
            }
        }
    } //end ListView
} //end Page
