import QtQuick 1.1
import com.meego 1.0
import "../Components" as Components
import "../Components/constants.js" as UI

Sheet {
    id: nowPlayingPage
//    property string pageName: "NowPlayingPage"

    style: SheetStyle {
        inverted: true
    }

    property bool cancelDragEvent: nowPlayingPageController.DragCancelingNeeded
    property int npIndex: nowPlayingPageController.NowPlayingIndex
    property int artIndex: 0
    property int oldNpIndex: 0
    property int highlightMoveDuration: 400
    property int timeFromPreviousIndexChange: 0
    property int correctIndex: -1
    property bool previousButtonClicked: false
    property bool nextButtonClicked: false
    property bool albumArtInit: false

//    orientationLock: 1 // PageOrientation.LockPortrait
    acceptButtonText: "Hide"

    onNpIndexChanged: {
        timeIncreaser.start()
        if (timeFromPreviousIndexChange == 0) {
            nowPlayingPage.highlightMoveDuration = 400
        } else {
            nowPlayingPage.highlightMoveDuration = 80
        }

        if (oldNpIndex == 0 && npIndex == nowPlayingPageController.PlaylistTrackCount - 1 && albumArtInit) {
            if (previousButtonClicked) {
                indexSetter.start()
            }
            if (nextButtonClicked && nowPlayingPageController.PlaylistTrackCount == 2) {
                artIndex = npIndex + 1
                albumArtImage.playIndex = artIndex
                albumArtImage.currentIndex = artIndex
            } else {
                correctIndex = npIndex
                artIndex--
                albumArtImage.playIndex = artIndex
                albumArtImage.currentIndex = artIndex
            }
        } else if (oldNpIndex == nowPlayingPageController.PlaylistTrackCount - 1 && npIndex == 0 && albumArtInit) {
            if (nextButtonClicked) {
                indexSetter.start()
            }
            if (previousButtonClicked && nowPlayingPageController.PlaylistTrackCount == 2) {
                artIndex = npIndex + 1
                albumArtImage.playIndex = artIndex
                albumArtImage.currentIndex = artIndex
            } else {
                correctIndex = npIndex
                artIndex++
                albumArtImage.playIndex = artIndex
                albumArtImage.currentIndex = artIndex
            }
        } else {
            artIndex = npIndex + 1
            albumArtImage.playIndex = artIndex
            albumArtImage.currentIndex = artIndex
        }
        previousButtonClicked = false
        nextButtonClicked = false

        if (currentTrackIndexText.state == "hidden") {
            currentTrackIndexText.state = "visible"
            timer.start()
        } else {
            timer.restart()
        }
        oldNpIndex = npIndex
    }

    Timer {
        id: indexSetter
        interval: nowPlayingPage.highlightMoveDuration; running: false; repeat: false
        onTriggered: {
            artIndex = correctIndex + 1
            albumArtImage.playIndex = artIndex
            albumArtImage.positionViewAtIndex(artIndex, ListView.Center)
            correctIndex = -1
        }
    }

    Timer {
        id: timeIncreaser
        interval: 10; running: false; repeat: true
        onTriggered: {
            timeFromPreviousIndexChange += 10
            if (timeFromPreviousIndexChange > 500) {
                timeFromPreviousIndexChange = 0
                timeIncreaser.stop()
            }
        }
    }

    onStatusChanged: {
        switch(status) {
            case PageStatus.Activating: {
//                    viewMenu.menuData = menuDataNP
                    nowPlayingPageController.setVisible(true)
                }
            case PageStatus.Deactivating: {
                    nowPlayingPageController.setVisible(false)
                    rootWindow.hideNPSheet()
                }
        }
    }

    onCancelDragEventChanged: {
        progressBar.tabPressed = false
        progressBar.disableDragTarget()
    }

    Timer {
        id: timer
        interval: 3000; running: false; repeat: false
        onTriggered: {
            currentTrackIndexText.state = "hidden"
        }
    }

    Components.ContextMenuData {
        id: menuDataNP
        model: ListModel {
            ListElement {
                action: "Repeat"
                subText: ""
                hasCheckBox: true
                actionOn: false
            }
            ListElement {
                action: "Shuffle"
                subText: ""
                hasCheckBox: true
                actionOn: false
            }
            ListElement {
                action: "Add To Playlist"
                subText: "Save current track to Playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Play Queue"
                subText: "Show tracks of Play Queue"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Save Play Queue"
                subText: "Create Playlist from Play Queue"
                hasCheckBox: false
                actionOn: false
            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Repeat": {
                    nowPlayingPageController.changeRepeatMode()
                }
                case "Shuffle": {
                    nowPlayingPageController.shuffle()
                }
                case "Add To Playlist": {
                    rootWindow.showDialog("Save to playlist", saveToPlaylistContent)
                }
                case "Play Queue": {
                    goToPage("PlayQueuePage")
                }
                case "Save Play Queue": {
                    nowPlayingPageController.allowPlayQueueSaving()
                    rootWindow.showDialog("Save to playlist", saveToPlaylistContent)
                }
            }
        }
    }

    content: Rectangle {
        id: mainContainer
        anchors.fill: parent
        radius: 20
        color: "#282828"

        // Album art
        ListView {
            id: albumArtImage
            property bool songChangeAllowed: false
            property bool flickStarted: false
            property int playIndex: -1

            anchors.left: parent.left
            anchors.right: parent.right
            anchors.top: parent.top
            height: parent.width
    //        anchors.bottom: parent.bottom
            model: nowPlayingAlbumArtModel
            orientation: ListView.Horizontal
            snapMode: ListView.SnapOneItem
            clip: true
            highlightRangeMode: ListView.StrictlyEnforceRange
            highlightMoveDuration: nowPlayingPage.highlightMoveDuration
            boundsBehavior: Flickable.StopAtBounds
            interactive: nowPlayingPageController.PlaylistTrackCount > 1

            Components.LoadingSpinner {
                id: albumArtLoadingSpinner
                fillParent: false
                invertedSpinner: true
                loadingText: "Loading album art..."
                anchors.top: parent.top
                anchors.left: parent.left
                anchors.right: parent.right
                height: albumArtImage.currentItem != null ? albumArtImage.currentItem.height : 0
                visible: !nowPlayingPageController.LocalPlayback &&
                         albumArtImage.currentItem != null &&
                         !albumArtImage.currentItem.imageReady ? true : false
            }

            delegate:
            Item {
                id: nowPlayingPageAlbumArt
                width: nowPlayingPage.width
                height: width
                property bool imageReady: false
                state: imageReady ? "visible" : "hidden"

                Image {
                    id: albumArt
                    anchors.top: parent.top
                    width: parent.width - 40
                    height: width
                    source: TrackAlbumArt
                    anchors.centerIn: parent
                    smooth: true
                    onStatusChanged: {
                        if (status == Image.Ready) {
                            nowPlayingPageAlbumArt.imageReady = true
                        } else if (status == Image.Error) {
                            nowPlayingPageAlbumArt.imageReady = true
                            console.log("\n\n\n IMAGE NOT FOUND FOR NP PUT DEFAULT\n\n\n")
                        }
                    }
                }
    /*            Image {
                    id: mirroredAlbumArt
                    anchors.top: albumArt.bottom
                    anchors.horizontalCenter: albumArt.horizontalCenter
                    anchors.topMargin: 2
                    opacity: 0.9
                    source: TrackAlbumArt
                    width: parent.width - 40
                    height: width
                    smooth: true
                    transform: [
                        Rotation { origin.x: mirroredAlbumArt.width/2; origin.y: mirroredAlbumArt.height/2; axis { x: 1; y: 0; z: 0 } angle: 180 },
                        Rotation { origin.x: mirroredAlbumArt.width/2; origin.y: 0; axis { x: 1; y: 0; z: 0 } angle: 60 }
                    ]
                }
                Rectangle {
                    id: mirrorAlbumArtShader
                    anchors.fill: mirroredAlbumArt
                    transform: [
                        Rotation { origin.x: mirrorAlbumArtShader.width/2; origin.y: mirrorAlbumArtShader.height/2; axis { x: 0; y: 0; z: 1; } angle: 0; },
                        Rotation { origin.x: mirroredAlbumArt.width/2; origin.y: 0; axis { x: 1; y: 0; z: 0 } angle: 60 }
                    ]
                    gradient: Gradient {
                        GradientStop { position: 0.0; color: "transparent" }
                        GradientStop { position: 0.6; color: "#393536" }
                    }
                }
    */            states: [
                    State {
                        name: "visible"
                        PropertyChanges { target: albumArt; opacity: 1 }
    //                    PropertyChanges { target: mirroredAlbumArt; opacity: 1 }
                    },
                    State {
                        name: "hidden"
                        PropertyChanges { target: albumArt; opacity: 0 }
    //                    PropertyChanges { target: mirroredAlbumArt; opacity: 0 }
                    }
                ]
                transitions: [
                    Transition {
                        from: "hidden"; to: "visible"
                        NumberAnimation { target: albumArt; properties: "opacity"; duration: 400; easing.type: Easing.InCubic }
    //                    NumberAnimation { target: mirroredAlbumArt; properties: "opacity"; duration: 400; easing.type: Easing.InCubic }
                    }
                ]
            }
            onModelChanged: {
                songChangeAllowed = false
                artIndex = npIndex + 1
                currentIndex = artIndex
                playIndex = artIndex
                positionViewAtIndex(artIndex, ListView.Center)
                albumArtInit = true
                songChangeAllowed = true
            }
            onCurrentIndexChanged: {
                if (currentItem != null) {
                    currentItem.z = 2
                }

                if (songChangeAllowed) {
                    artIndex = currentIndex
                    if (currentIndex < playIndex - 1) {
                        currentIndex = playIndex - 1
                    } else if (currentIndex > playIndex + 1) {
                        currentIndex = playIndex + 1
                    }
                }
            }
            onMovementStarted: playIndex = currentIndex
            onMovementEnded: {
                if (playIndex != artIndex && !flickStarted) {
                    if (playIndex > artIndex && songChangeAllowed) {
                        nowPlayingPageController.previous()
                        playIndex = artIndex
                        currentIndex = artIndex
                    } else if (playIndex < artIndex && songChangeAllowed) {
                        nowPlayingPageController.next()
                        playIndex = artIndex
                        currentIndex = artIndex
                    }
                    if (correctIndex != -1) {
                        artIndex = correctIndex + 1
                        playIndex = artIndex
                        currentIndex = artIndex
                        albumArtImage.positionViewAtIndex(artIndex, ListView.Center)
                        correctIndex = -1
                    }
                }
            }

            Timer {
                id: flickEndedTimer
                interval: 10; running: false; repeat: false
                onTriggered: {
                    if (!albumArtImage.moving) {
                        albumArtImage.flickStarted = false
                    } else {
                        flickEndedTimer.start()
                    }
                }
            }

            onFlickStarted: {
                if (!moving) {
                    flickStarted = true
                }
            }
            onFlickEnded: {
                flickEndedTimer.start()
                if (playIndex > artIndex && songChangeAllowed) {
                    nowPlayingPageController.previous()
                    playIndex = artIndex
                    currentIndex = artIndex
                } else if (playIndex < artIndex && songChangeAllowed) {
                    nowPlayingPageController.next()
                    playIndex = artIndex
                    currentIndex = artIndex
                }
                if (correctIndex != -1) {
                    artIndex = correctIndex + 1
                    playIndex = artIndex
                    currentIndex = artIndex
                    albumArtImage.positionViewAtIndex(artIndex, ListView.Center)
                    correctIndex = -1
                }
            }
        }

        // Progress bar
        Components.Progress {
            id: progressBar
            objectName: "progressBar"
            progress: nowPlayingPageController.NowPlayingProgressBarPosition
            playedTime: nowPlayingPageController.NowPlayingProgressBarPlayedTime
            duration: nowPlayingPageController.NowPlayingProgressBarSongDuration
            anchors.left: albumArtImage.left
            anchors.leftMargin: 25
            anchors.right: albumArtImage.right
            anchors.rightMargin: 25
            anchors.top: albumArtImage.bottom
            anchors.topMargin: 5
            onSetPosition: nowPlayingPageController.seek(progressBar.songPosition)
        }

        // Buttons
        Item {
            id: buttonPanel
            objectName: "buttonPanel"
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.top: progressBar.bottom
            anchors.topMargin: 10
            height: 80
            Components.Button {
                id: button1
                objectName: "previous"
                enabled: nowPlayingPageController.PlaylistTrackCount > 1 || nowPlayingPageController.Repeat ? true : false
                imagePath: enabled ? "/usr/share/spotify/pics/pb_control-prev.png" : "/usr/share/spotify/pics/pb_control-prev-dimmed.png"
                anchors.left: buttonPanel.left
                width: parent.width / 3
                imageSize: 48
                onClicked: {
                    if (!indexSetter.running) {
                        previousButtonClicked = true
                        nowPlayingPageController.previous()
                    }
                }
            }

            Components.Button {
                id: button2
                objectName: "pause"
                imagePath: {
                    if (nowPlayingPageController.PlayingState) {
                        return "/usr/share/spotify/pics/pb_control-pause.png"
                    } else {
                        return "/usr/share/spotify/pics/pb_control-play.png"
                    }
                }
                imageSize: 48
                anchors.horizontalCenter: buttonPanel.horizontalCenter
                width: parent.width / 3
                onClicked: nowPlayingPageController.pauseOrResume()
            }

            Components.Button {
                id: button3
                objectName: "next"
                enabled: nowPlayingPageController.PlaylistTrackCount > 1 || nowPlayingPageController.Repeat ? true : false
                imagePath: enabled ? "/usr/share/spotify/pics/pb_control-next.png" : "/usr/share/spotify/pics/pb_control-next-dimmed.png"
                anchors.right: buttonPanel.right
                width: parent.width / 3
                imageSize: 48
                onClicked: {
                    if (!indexSetter.running) {
                        nextButtonClicked = true
                        nowPlayingPageController.next()
                    }
                }
            }
        }

        // Track title
        Text {
            id: songText
            objectName: "songText"
            anchors.top: buttonPanel.bottom; anchors.left: parent.left; anchors.right: parent.right
            anchors.topMargin: 5; anchors.leftMargin: 10; anchors.rightMargin: 10
            text: nowPlayingPageController.NowPlayingTrack
            color: "white"
            font.pixelSize: 32
            font.bold: true
            elide: Text.ElideRight
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignBottom
        }

        // Artist text
        Text {
            id: artistText
            objectName: "artistText"
            anchors.top: songText.bottom; anchors.left: parent.left; anchors.right: parent.right
            anchors.topMargin: 5; anchors.leftMargin: 10; anchors.rightMargin: 10
            text: "by " + nowPlayingPageController.NowPlayingArtist
            color: "white"
            font.pixelSize: UI.FONT_SIZE_SMALL
            font.family: UI.FONT_FAMILY
            elide: Text.ElideRight
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignBottom
        }

        // Album text
        Text {
            id: albumText
            objectName: "albumText"
            anchors.top: artistText.bottom; anchors.left: parent.left; anchors.right: parent.right
            anchors.topMargin: 5; anchors.leftMargin: 10; anchors.rightMargin: 10
            text: nowPlayingPageController.NowPlayingAlbum
            color: "grey"
            font.pixelSize: UI.FONT_SIZE_SMALL
            elide: Text.ElideRight
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignBottom
        }

//      //   Toolbar fake
////        Image {
////            id: toolbar
////            source: "/usr/share/spotify/pics/np_toolbar.png"
////            anchors.bottom: parent.bottom
////        }

//        ToolBar {
//            id: npToolbar
//          //  z: 2
//            platformStyle: ToolBarStyle { inverted: true; visibilityTransitionDuration: 200 }
//            anchors.bottom: parent.bottom
//            anchors.bottomMargin: inputContext.softwareInputPanelVisible == true ? -npToolbar.height : 0


//            tools: ToolBarLayout {
//                ToolIcon {
//                    id: npBackBtn
//                    objectName: "npBackBtn"
//                    iconSource: "/usr/share/spotify/pics/icon-m-toolbar-back-white.png"
//                    onClicked: {
//                       goToPage("goBack")
//                    }
//                    enabled: if (viewMenu.visible || shareMenu.visible) {
//                        return false
//                    } else {
//                        return true
//                    }
//                }

//                ToolIcon {
//                    id: npShareBtn
//                    objectName: "npShareBtn"
//                    iconSource: enabled ? "/usr/share/spotify/pics/toolbar_share.png" :
//                    "/usr/share/spotify/pics/toolbar_share.png"
//                    enabled: !viewMenu.visible
//                    onClicked: {
//                        if (pageStack.currentPage.pageName != "SearchPage" && !rootWindow.busy) {
//                            playListsText.color = "grey"
//                            searchText.color = "white"
//                            discoverText.color = "grey"
//                            goToPage("SearchPage")
//                        }
//                    }
//                    onClicked: { shareMenu.visible ? shareMenu.close() : shareMenu.open(2) }
//                    visible: !nowPlayingPageController.LocalPlayback
//                }

//                ToolIcon {
//                    id: npFavouriteBtn
//                    objectName: "npFavouriteBtn"
//                    iconSource:nowPlayingPageController.Starred ? "/usr/share/spotify/pics/indicator_star_enabled.png" :
//                    "/usr/share/spotify/pics/indicator_star_disabled.png"
//                    visible: !nowPlayingPageController.LocalPlayback
//                    enabled: if (viewMenu.visible || shareMenu.visible) {
//                        return false
//                    } else {
//                        return true
//                    }

//                    onClicked: {
//                        nowPlayingPageController.setStarred()
//                    }

//                }

//                ToolIcon {
//                    id: npMenuBtn
//                    objectName: "npMenuBtn"
//                    iconSource:"/usr/share/spotify/pics/icon-m-toolbar-view-menu-white.svg"
//                    enabled: !shareMenu.visible
//                    visible: !nowPlayingPageController.LocalPlayback
//                    onClicked: {
//                        if (bgImage.state == "np") {
//                            viewMenu.menuData.model.set(0, {"actionOn": nowPlayingPageController.repeatMode()})
//                            viewMenu.menuData.model.set(1, {"actionOn": nowPlayingPageController.isShuffled()})
//                        }
//                        viewMenu.visible ? viewMenu.close() : viewMenu.open("")
//                    }

//                }
//            }
//        }


        // Show info area
        MouseArea {
            id: mouseRegion
            objectName: "mouseRegion"
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.top: songText.top
            anchors.bottom: albumText.bottom
            onClicked: detailsMenu.open()
            enabled: !nowPlayingPageController.LocalPlayback
        }

        // Current track index
        Text {
            id: currentTrackIndexText
            anchors.top: albumArtImage.top
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.topMargin: 40
            height: 100
            width: 300
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            text: nowPlayingPageController.CurrentPositionInPlaylist
            color: "white"
            font.pixelSize: 60
            font.family: UI.FONT_FAMILY
            font.bold: true
            style: Text.Outline; styleColor: "#393536"
            z: 2
            state: "hidden"

            states: [
                State {
                    name: "visible"
                    PropertyChanges { target: currentTrackIndexText; opacity: 1 }
                },
                State {
                    name: "hidden"
                    PropertyChanges { target: currentTrackIndexText; opacity: 0 }
                }
            ]

            transitions: [
                Transition {
                    from: "visible"; to: "hidden"
                    NumberAnimation { target: currentTrackIndexText; properties: "opacity"; duration: 1500; easing.type: Easing.OutCubic }
                },
                Transition {
                    from: "hidden"; to: "visible"
                    NumberAnimation { target: currentTrackIndexText; properties: "opacity"; duration: 500; easing.type: Easing.InCubic }
                }
            ]
        }

        // This prevents user to pan more than 1 index per time.
        MouseArea {
            anchors.fill: albumArtImage
            visible: albumArtImage.moving
        }

        // Info menu
        ContextMenu {
            id: detailsMenu
            z:3

            MenuLayout {
                MenuItem {
                    text: nowPlayingPageController.NowPlayingArtist
                    onClicked: {
                        if (nowPlayingPageController.artistDataReady()) {
                            detailsMenu.close()
                            artistPageController.clearData()
                            goToPage("ArtistPage")
                            artistPageController.setTab("overview")
                            nowPlayingPageController.getArtistData()
                        }
                    }
                }
                MenuItem {
                    text: nowPlayingPageController.NowPlayingAlbum
                    onClicked: {
                        if (nowPlayingPageController.albumDataReady()) {
                            detailsMenu.close()
                            albumPageController.resetAlbumPage()
                            goToPage("AlbumPage")
                            nowPlayingPageController.getAlbumData()
                        }
                    }
                }
                MenuItem {
                    text: "See related Artists"
                    onClicked: {
                        if (nowPlayingPageController.artistDataReady()) {
                            detailsMenu.close()
                            artistPageController.clearData()
                            goToPage("ArtistPage")
                            artistPageController.setTab("related")
                            nowPlayingPageController.getArtistData()
                        }
                    }
                }
            }
        }

    }



    // Save as playlist dialog
    Component {
        id: saveToPlaylistContent
        Item {
            height: childrenRect.height
            width: dialogLoader.status ==  Loader.Ready ? dialogLoader.item.contentWidth : 0

            ListView {
                id: playlists
                objectName: "playlists"
                clip: true
                width: parent.width
                height: Math.max(150, Math.min(600, count * 80 + 80))
                model: editablePlaylistsModel
                header: headerComponent
                delegate: Components.ListItem {
                    labelText1: playlistName
                    showIcon: false
                    onClicked: {
                        nowPlayingPageController.saveToPlaylist(index)
                        rootWindow.hideDialog()
                    }
                }

                Component {
                    id: headerComponent
                    Components.ListItem {
                        labelText1: "<Add as a new playlist>"
                        labelText2: nowPlayingPageController.NowPlayingTrack
                        showIcon: false
                        onClicked: {
                            nowPlayingPageController.saveToPlaylist(-1)
                            rootWindow.hideDialog()
                        }
                    }
                }
            }
        }
    }
}
