import QtQuick 1.1
import com.meego 1.0
import "../Components" as Components

Page {
    id: wrapper
    property string pageName: "PlayQueuePage"

    orientationLock: 1 // PageOrientation.LockPortrait

    onStatusChanged: {
        switch(status) {
            case PageStatus.Activating: {
                viewTitlePanel.headerText = "Play Queue"
            }
            case PageStatus.Active: {
                myList.positionViewAtIndex(nowPlayingPageController.NowPlayingIndex,
                                               ListView.Beginning);
            }
        }
    }

    Components.ScrollBar {
        id: scrollBar
        z: 2
        objectName: "scrollBar"
        scrollArea: myList
    }

    Components.ViewTitlePanel {
        id: viewTitlePanel
        z: 2
        objectName: "viewTitlePanel"
        anchors.top: parent.top
        anchors.topMargin: rootWindow.statusBarHeight
        height: 72
        headerText: "PlayQueuePage"
        onlineMode: engineLoggedIn
        imageSource: nowPlayingPageController.HasValidTrack ?
                "/usr/share/spotify/pics/Spotify-31.png" : ""
        imageSourcePressed: nowPlayingPageController.HasValidTrack ?
                        "/usr/share/spotify/pics/Spotify-61.png" : ""
        onClicked: {
            if (nowPlayingPageController.HasValidTrack) {
                rootWindow.showNPSheet()
            }
        }
    }

    ListView {
        id: myList
        objectName: "tracksOfPlayQueue"
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: viewTitlePanel.bottom
        anchors.bottom:  parent.bottom
        anchors.bottomMargin: albumAndArtistPageToolBar.height
        model: nowPlayingQueueModel
        cacheBuffer: 800
        delegate: Components.ListItem {
            objectName: "playQueueItem"
            visible: if (nowPlayingPageController.LocalPlayback) {
                index == 0 || index == localSongsModel.rowCount() - 1 ? false : true
            }

            height: visible ? 80 : 0
            labelText1: TrackTitle
            labelText2: TrackSubHeader
            allowRightImage: false
            listReady: TrackAvailable
            npHighlighther: if (nowPlayingPageController.LocalPlayback) {
                index == nowPlayingPageController.NowPlayingIndex + 1
            } else {
                index == nowPlayingPageController.NowPlayingIndex
            }

            onClicked: {
                if (nowPlayingPageController.LocalPlayback) {
                    localPlayer.loadMedia(index - 1, true)
                } else {
                    nowPlayingPageController.queueListClicked(index)
                }
                goToPage("NowPlayingPage")
            }
        }
    }
}
