import QtQuick 1.1
import com.meego 1.0
import "../Components" as Components
import "../Components/constants.js" as UI

Page {
    id: wrapper
    property string pageName: "PlaylistsPage"
    property int longTappedIndex
    property bool allowClicks: rootWindow.loginIn
    orientationLock: PageOrientation.LockPortrait

    onAllowClicksChanged: {
        if (allowClicks)
            pageItem.visible = false
    }

    Item {
        id: pageItem
        anchors.fill: parent
        visible: true
        z: 10
        MouseArea {
            anchors.fill: parent
            onClicked: console.log("Wait")
        }
    }

    onStatusChanged: {
        switch(status) {
            case PageStatus.Activating: {
                viewTitlePanel.headerText = "Playlists"
                pageItem.visible = true
            }
            case PageStatus.Active: {
                if (rootWindow.loginIn)
                    pageItem.visible = false

                playlistsPageController.setVisible(true)
            }
        }
    }

    // Context menu for own playlists
    Components.ContextMenuData {
        id: longTapMenuData
        model: ListModel {
            ListElement {
                action: "Delete"
                subText: "Delete this playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Rename"
                subText: "Rename this playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Add to Queue"
                subText: "Add this playlist to Queue"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share playlist"
                subText: "Share this playlist with others"
                hasCheckBox: false
                actionOn: false
            }
//            ListElement {
//                action: "Available offline"
//                subText: "Playable while offline"
//                hasCheckBox: true
//                actionOn: false
//            }
//            ListElement {
//                action: "Collaborative Playlist"
//                subText: "Other can edit this playlist"
//                hasCheckBox: true
//                actionOn: false
//            }
        }


        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Delete": {
                    if (playlistsPageController.handleDeletePlaylist(longTappedIndex)) {
                        rootWindow.showDialog("Delete playlist", deleteContent)
                    } else {
                        notifyController.showMessage("Removing failed")
                    }
                }
                case "Rename": {
                    if (playlistsPageController.handleRenamePlaylist(longTappedIndex)) {
                        rootWindow.showDialog("Rename playlist", renameContent)
                    } else {
                        notifyController.showMessage("Renaming failed")
                    }
                }
                case "Add to Queue": {
                    playlistsPageController.handleAddPlaylistToQueue(longTappedIndex)
                }
                case "Share playlist": {
                    playlistsPageController.shareLongTappedPlaylist(longTappedIndex)
                }
                case "Available offline": {
                    //playlistsPageController.changePlaylistOfflineMode(longTappedIndex)
                }
//                case "Collaborative Playlist": {

//                }
            }
        }
    }

    // Context menu for subscribed playlists
    Components.ContextMenuData {
        id: longTapMenuDataSubscribed
        model: ListModel {
            ListElement {
                action: "Unsubscribe"
                subText: "Unsubscribe this playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Add to Queue"
                subText: "Add this playlist to Queue"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share playlist"
                subText: "Share this playlist with others"
                hasCheckBox: false
                actionOn: false
            }
//            ListElement {
//                action: "Available offline"
//                subText: "Playable while offline"
//                hasCheckBox: true
//                actionOn: false
//            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Unsubscribe": {
                    playlistsPageController.unsubscribe(longTappedIndex)
                }
                case "Add to Queue": {
                    playlistsPageController.handleAddPlaylistToQueue(longTappedIndex)
                }
                case "Share playlist": {
                    playlistsPageController.shareLongTappedPlaylist(longTappedIndex)
                }
                case "Available offline": {
                    //playlistsPageController.changePlaylistOfflineMode(longTappedIndex)
                }
            }
        }
    }

    // Context menu for private items
    Components.ContextMenuData {
        id: longTapMenuDataForPrivateItems
        model: ListModel {
            ListElement {
                action: "Add to Queue"
                subText: "Add this playlist to Queue"
                hasCheckBox: false
                actionOn: false
            }
//            ListElement {
//                action: "Available offline"
//                subText: "Playable while offline"
//                hasCheckBox: true
//                actionOn: false
//            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Add to Queue": {
                    playlistsPageController.handleAddPlaylistToQueue(longTappedIndex)
                }
                case "Available offline": {
                    //playlistsPageController.changePlaylistOfflineMode(longTappedIndex)
                }
            }
        }
    }

    // Context menu for Play Queue saving
    Components.ContextMenuData {
        id: longTapMenuDataForPlayQueueSaving
        model: ListModel {
            ListElement {
                action: "Save Play Queue"
                subText: "Create Playlist from Play Queue"
                hasCheckBox: false
                actionOn: false
            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Save Play Queue": {
                    nowPlayingPageController.allowPlayQueueSaving()
                    rootWindow.showDialog("Save to playlist", saveToPlaylistContent)
                }
            }
        }
    }

    // Scroll bar for playlist view
    Components.ScrollBar {
        id: scrollBar
        z: 1
        objectName: "scrollBar"
        scrollArea: myList
    }

    // Title panel
    Components.ViewTitlePanel {
        id: viewTitlePanel
        z: 2
        objectName: "viewTitlePanel"
        anchors.top: parent.top
        anchors.topMargin: rootWindow.statusBarHeight
        height: 72
        headerText: "PlaylistsPage"
        onlineMode: engineLoggedIn
        imageSource: nowPlayingPageController.HasValidTrack ?
                "/usr/share/spotify/pics/Spotify-31.png" : ""
        imageSourcePressed: nowPlayingPageController.HasValidTrack ?
                        "/usr/share/spotify/pics/Spotify-61.png" : ""
        onClicked: {
            if (nowPlayingPageController.HasValidTrack) {
                rootWindow.showNPSheet()
            }
        }
    }

    // Main list for playlists
    ListView {
        id: myList
        objectName: "playlists"
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: viewTitlePanel.bottom
        anchors.bottom:  parent.bottom
        anchors.bottomMargin: toolbar.height
        header: listHeader
        model: playlistsModel

        delegate: Components.ListItem {
            objectName: "playlistItem"
            imageSource: PlaylistCollaborativity ? "/usr/share/spotify/pics/Spotify-30.png" : "/usr/share/spotify/pics/Spotify-21.png";
            loadingStatus: PlaylistOfflineStatus
            loadingProgress: PlaylistOfflineProgress
            showProductImage: true
            listReady: if (playlistsPageController.offlineMode) {
                            PlaylistOfflineStatus == 2
                        } else if (PlaylistTracksLoaded == PlaylistTrackCount )  {
                            true
                        } else {
                            false
                        }
            labelText1: if (PlaylistName == "") {
                            "initializing playlist..."
                        } else {
                            PlaylistName
                        }
            labelText1Color:  PlaylistCollaborativity ? "#86c021": ""

            labelText2: if (PlaylistTracksLoaded <= 0) {
                            ""
                        } else if (PlaylistTracksLoaded < PlaylistTrackCount) {
                            PlaylistOwner ? PlaylistTracksLoaded + "/" + PlaylistTrackCount + " tracks | by " + PlaylistOwner : + PlaylistTracksLoaded + "/" + PlaylistTrackCount + " tracks"
                        } else {
                            PlaylistOwner ? PlaylistTrackCount + " tracks | by " + PlaylistOwner : PlaylistTrackCount + " tracks"
                        }
            showNextIcon: true
            imageSize: 40
            onClicked: {
                if (listReady) {
                    if (!rootWindow.busy) {
                        playlistsPageController.listClicked(index)
                        goToPage("TracksOfPlaylistPage")
                    }
                }
            }
            onPressAndHold: {
                if (listReady) {
                    if (PlaylistOwner) {
                        viewMenu.menuData = longTapMenuDataSubscribed
                        //longTapMenuDataSubscribed.model.set(3, {"actionOn": PlaylistOfflineStatus != 0})
                    } else {
                        viewMenu.menuData = longTapMenuData
                        //longTapMenuData.model.set(4, {"actionOn": PlaylistOfflineStatus != 0})
                    }
                    longTappedIndex = index;
                    viewMenu.open(labelText1, labelText2, PlaylistCollaborativity ? "/usr/share/spotify/pics/Spotify-30_large.png" : "/usr/share/spotify/pics/Spotify-21_large.png")
                }
            }
        }
    }

    // Main list header for Inbox, Starred, Local files and Play queue
    Component {
        id: listHeader
        Item {
            objectName: "listHeader"
            width: parent.width
            height: childrenRect.height

            Components.ListItem {
                id: starred
                objectName: "starred"
                imageSource: "/usr/share/spotify/pics/Spotify-13.png"
                labelText1: "Starred"
                labelText2: playlistsPageController.StarredCount + " tracks"
                showNextIcon: true
                imageSize: 40
                showProductImage: true
                listReady: if (playlistsPageController.StarredReady) {
                               true
                           } else {
                               false
                           }
                onClicked: {
                    if (!rootWindow.busy && listReady) {
                        playlistsPageController.listClicked(-2)
                        goToPage("TracksOfPlaylistPage")
                    }
                }
                onPressAndHold: {
                    if (listReady) {
                        viewMenu.menuData = longTapMenuDataForPrivateItems
                        longTappedIndex = -2;
                        viewMenu.open(labelText1, labelText2, "/usr/share/spotify/pics/Spotify-10_large.png")
                    }
                }
            }

            Components.ListItem {
                id: inbox
                objectName: "inbox"
                imageSource: "/usr/share/spotify/pics/Spotify-12.png"
                anchors.top: starred.bottom
                labelText1: "Inbox"
                labelText2: playlistsPageController.InboxMessageCount + " tracks"
                showSeenCount: true
                showNextIcon: true
                showProductImage: true
                imageSize: 40
                notSeenTracksCount: playlistsPageController.NotSeenTrackCount
                listReady: if (playlistsPageController.InboxReady) {
                               true
                           } else {
                               false
                           }
                onClicked: {
                    if (!rootWindow.busy && listReady) {
                        playlistsPageController.listClicked(-3)
                        goToPage("TracksOfPlaylistPage")
                    }
                }
                onPressAndHold: {
                    if (listReady) {
                        viewMenu.menuData = longTapMenuDataForPrivateItems
                        longTappedIndex = -3;
                        viewMenu.open(labelText1, labelText2, "/usr/share/spotify/pics/Spotify-12_large.png")
                    }
                }
            }


            Components.ListItem {
                id: localFiles
                objectName: "localFiles"
                anchors.top: inbox.bottom
                imageSource: "/usr/share/spotify/pics/Spotify-20.png"
                labelText1: "Local Files"
                labelText2: playlistsPageController.LocalTracksCount + " tracks"
                showNextIcon: true
                showProductImage: true
                imageSize: 40
                onClicked: goToPage("LocalSongsPage")
            }

            Components.ListItem {
                id: playQueue
                objectName: "playQueue"
                anchors.top: localFiles.bottom
                listReady: (nowPlayingPageController.PlaylistTrackCount > 0)
                enabled: listReady
                imageSource: "/usr/share/spotify/pics/Spotify-18.png"
                labelText1: "Play Queue"
                labelText2: enabled ? (nowPlayingPageController.PlaylistTrackCount + " tracks") : ""
                showNextIcon: true
                showProductImage: true
                imageSize: 40
                onClicked: goToPage("PlayQueuePage")
                onPressAndHold: {
                    if (listReady && nowPlayingPageController.PlaylistTrackCount > 0 && !nowPlayingPageController.LocalPlayback) {
                        viewMenu.menuData = longTapMenuDataForPlayQueueSaving
                        longTappedIndex = -3;
                        viewMenu.open(labelText1, labelText2, "/usr/share/spotify/pics/Spotify-18_large.png")
                    }
                }
            }
            Image {
                id: dividerrow
                anchors.top: playQueue.bottom
                source: "/usr/share/spotify/pics/group_separator.png"
            }
        }
    }

    // Delete dialog
    Component {
        id: deleteContent
        Item {
            height: childrenRect.height
            width: dialogLoader.status ==  Loader.Ready ? dialogLoader.item.contentWidth : 0

            Text {
                id: text
                width: parent.width
                text: "Are you sure you want to delete '"+playlistsPageController.DeletePlaylist+"'-playlist?"
                font.pixelSize: UI.FONT_SIZE_NORMAL
                font.family: UI.FONT_FAMILY
                color: "white"

                wrapMode: Text.WordWrap
            }

            Item {
                anchors.top: text.bottom
                anchors.topMargin: 30
                width: parent.width
                height: 60
                anchors.left: parent.left
                anchors.leftMargin: 30

                Button {
                    id: ok
                    objectName: "okButton"
                    anchors.left: parent.left
                    anchors.leftMargin: 80
                    width: 140
                    text: "Ok"
                    platformStyle: ButtonStyle { inverted: true }
                    onClicked: {
                        notifyController.deleteAccepted()
                        rootWindow.hideDialog()
                    }
                }
            }
        }
    }

    // Rename dialog
    Component {
        id: renameContent
        Item {
            height: childrenRect.height
            width: dialogLoader.status ==  Loader.Ready ? dialogLoader.item.contentWidth : 0

            TextField {
                id: text
                width: parent.width
                font.pixelSize: UI_FONT_SIZE_NORMAL
                onTextChanged: notifyController.messageLengthChanged(text)
                text: notifyController.RenameOldName
                Keys.onReturnPressed: {
                    if (notifyController.MsgAcceptable) {
                        notifyController.renameAccepted(text.text)
                        text.platformCloseSoftwareInputPanel()
                        text.focus = false
                        rootWindow.hideDialog()
                    }
                }
            }

            Item {
                anchors.top: text.bottom
                anchors.topMargin: 30
                width: parent.width
                height: 60
                anchors.left: parent.left
                anchors.leftMargin: 30

                Button {
                    id: ok
                    objectName: "okButton"
                    enabled: notifyController.MsgAcceptable
                    anchors.left: parent.left
                    anchors.leftMargin: 80
                    width: 140
                    text: "Ok"
                    platformStyle: ButtonStyle { inverted: true }
                    onClicked: {
                        notifyController.renameAccepted(text.text)
                        text.platformCloseSoftwareInputPanel()
                        text.focus = false
                        rootWindow.hideDialog()
                    }
                }
            }
        }
    }
}
