import QtQuick 1.1
import com.meego 1.0
import "../Components"

Page {
    id: wrapper
    property string pageName: "ProfilePage"
    property string albumArtDefaultImg: "/usr/share/spotify/pics/Spotify-01.png"
    property string artistArtDefaultImg: "/usr/share/spotify/pics/Spotify-01.png"
    property int longTappedIndex
    property string artistOfLongTappedItem: ""
    property string profileName: profilePageController.ProfileName
    orientationLock: 1 // PageOrientation.LockPortrait

    onProfileNameChanged: {
        viewTitlePanel.headerText = profileName
    }

    onStatusChanged: {
        switch(status) {
            case PageStatus.Activating: {
                viewTitlePanel.headerText = profileName
            }
            case PageStatus.Active: {
            }
        }
    }

    ContextMenuData {
        id: longTapMenuDataTracks
        model: ListModel {
            ListElement {
                action: "Queue"
                subText: "Add this track to Queue"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Add to Playlist"
                subText: "Save this album to Playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "View Album"
                subText: "View tracks of this album"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "View Artist"
                subText: "More from this artist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share Track"
                subText: "Share this track with others"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share to Spotify People"
                subText: "Send to a friend's Inbox"
                hasCheckBox: false
                actionOn: false
            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Queue": {
                    profilePageController.handleAddToQueue(longTappedIndex)
                }
                case "Add to Playlist": {
                    rootWindow.showDialog("Save to playlist", saveToPlaylistContent)
                }
                case "View Album": {
                    albumPageController.resetAlbumPage()
                    goToPage("AlbumPage")
                    profilePageController.getAlbumData(longTappedIndex)
                }
                case "View Artist": {
                    artistPageController.clearData()
                    goToPage("ArtistPage")
                    profilePageController.getArtistData(longTappedIndex)
                }
                case "Share Track": {
                    profilePageController.shareLongTappedTrack(longTappedIndex, false)
                }
                case "Share to Spotify People": {
                    profilePageController.shareLongTappedTrack(longTappedIndex, true)
                    goToPage("SpotifySharePage")
                }
            }
        }
    }

    ContextMenuData {
        id: longTapMenuDataPlaylists
        model: ListModel {
            ListElement {
                action: "Queue"
                subText: "Add this playlist to Queue"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share playlist"
                subText: "Share this playlist with others"
                hasCheckBox: false
                actionOn: false
            }
//            ListElement {
//                action: "Available offline"
//                subText: "Playable while offline"
//                hasCheckBox: true
//                actionOn: false
//            }
//            ListElement {
//                action: "Subscribed playlist"
//                subText: ""
//                hasCheckBox: true
//                actionOn: false
//            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Queue": {
                    profilePageController.handleAddPlaylistToQueue(longTappedIndex)
                }
                case "Share playlist": {
                    profilePageController.shareLongTappedPlaylist(longTappedIndex)
                }
                case "Available offline": {

                }
                case "Subscribed playlist": {

                }
            }
        }
    }

    ScrollBar {
        id: scrollBar
        z: 2
        objectName: "scrollBar"
        scrollArea: profileView
        allowFastScrolling: false
    }

    ViewTitlePanel {
        id: viewTitlePanel
        z: 2
        objectName: "viewTitlePanel"
        anchors.top: parent.top
        anchors.topMargin: rootWindow.statusBarHeight
        height: 72
        headerText: "ProfilePage"
        onlineMode: engineLoggedIn
        imageSource: nowPlayingPageController.HasValidTrack ?
                "/usr/share/spotify/pics/Spotify-31.png" : ""
        imageSourcePressed: nowPlayingPageController.HasValidTrack ?
                        "/usr/share/spotify/pics/Spotify-61.png" : ""
        onClicked: {
            if (nowPlayingPageController.HasValidTrack) {
                rootWindow.showNPSheet()
            }
        }
    }

    Flickable {
        id: profileView
        objectName: "flickableComponent"
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: viewTitlePanel.bottom
        anchors.bottom:  parent.bottom
        anchors.bottomMargin: albumAndArtistPageToolBar.height
        flickableDirection: Flickable.VerticalFlick
        contentHeight: flickableComponent.height

        Item {
            id: flickableComponent
            objectName: "flickableArea"
            height: childrenRect.height
            width: parent.width

            GroupHeaderDivider {
                id: playlistsHeader
                visible: playlists.count > 0
                objectName: "playlistsHeader"
                headerText: "Playlists"
            }

            ListView {
                id: playlists
                visible: count > 0
                objectName: "playlists"
                anchors.top: playlistsHeader.bottom
                interactive: false
                width: parent.width
                height: count * 80
                model: profilePlaylistsModel
                delegate: ListItem {
                    id: listItem
                    imageSource: "/usr/share/spotify/pics/Spotify-21.png"
                    imageSize: 49
                    labelText1: {
                        if (playlistName == "") {
                            "initializing playlist..."
                        } else {
                            playlistName
                        }
                    }
                    labelText2: {
                        if (tracksLoaded <= 0) {
                            ""
                        } else if (tracksLoaded < trackCount) {
                            tracksLoaded + "/" + trackCount + " Tracks"
                        } else {
                            trackCount + " Tracks"
                        }
                    }
                    showNextIcon: true
                    showProductImage: true
                    listReady: {
                        if (tracksLoaded == trackCount && trackCount > 0) {
                            true
                        } else {
                            false
                        }
                    }
                    onClicked: {
                        if (listReady) {
                            if (profilePageController.playlistClicked(index)) {
                                goToPage("TracksOfPlaylistPage")
                            } else {
                                notifyController.showMessage("... failed")
                            }
                        }
                    }
                    onPressAndHold: {
                        if (listReady) {
                            viewMenu.menuData = longTapMenuDataPlaylists
                            longTappedIndex = index;
                            viewMenu.open(labelText1, labelText2, imageSource)
                        }
                    }
                }
            }
            GroupHeaderDivider {
                id: topArtistsHeader
                visible: topArtists.count > 0
                objectName: "topArtistsHeader"
                anchors.top: playlists.bottom
                headerText: "Top Artists"
            }

            GridView {
                id: topArtists
                visible: count > 0
                objectName: "topArtists"
                anchors.top: topArtistsHeader.bottom
                interactive: false
                width: parent.width
                height: 120
                cellWidth: 120;
                cellHeight: cellWidth
                clip: true
                model: profileTopArtistsModel
                delegate: OverlayLabel {
                    id: topArtistItem
                    imageSource: AlbumArt != "" ? AlbumArt : artistArtDefaultImg
                    labelText: TitleRole
                    width: 119; height: 119;
                    onReleased:  {
                        artistPageController.clearData()
                        goToPage("ArtistPage")
                        artistPageController.setTab("overview")
                        profilePageController.topArtistsClicked(index)
                    }
                }
            }

            GroupHeaderDivider {
                id: topTracksHeader
                visible: topTracks.count > 0
                objectName: "topTracksHeader"
                anchors.top: topArtists.count > 0 ? topArtists.bottom : playlists.bottom
                headerText: "Top Tracks"
            }

            ListView {
                id: topTracks
                objectName: "topTracks"
                anchors.top: topTracksHeader.bottom
                interactive: false
                width: parent.width
                height: count * 80
                model: profileTopTracksModel
                cacheBuffer: 800
                delegate: ListItem {
                    id: listItem
                    imageSource: AlbumArt != "" ? AlbumArt : albumArtDefaultImg
                    labelText1: TitleRole
                    labelText2: SubTitleRole
                    showProductImage: true
                    onClicked: {
                        if (profilePageController.topTracksClicked(index)) {
                            goToPage("NowPlayingPage")
                        }
                    }
                    onPressAndHold: {
                        viewMenu.menuData = longTapMenuDataTracks
                        artistOfLongTappedItem = ArtistRole
                        longTappedIndex = index;
                        viewMenu.open(labelText1, labelText2, AlbumArt)
                    }
                }
            }
        }
    }

    Component {
        id: saveToPlaylistContent
        Item {
            height: childrenRect.height
            width: dialogLoader.status ==  Loader.Ready ? dialogLoader.item.contentWidth : 0

            ListView {
                id: playlists
                objectName: "playlists"
                clip: true
                width: parent.width
                height: Math.max(150, Math.min(600, count * 80 + 80))
                model: editablePlaylistsModel
                header: headerComponent
                cacheBuffer: 800
                delegate: ListItem {
                    inverted: true
                    labelText1: playlistName
                    showIcon: false
                    onClicked: {
                        profilePageController.saveToPlaylist(index, longTappedIndex)
                        rootWindow.hideDialog()
                    }
                }

                Component {
                    id: headerComponent
                    ListItem {
                        inverted: true
                        labelText1: "<Add as a new playlist>"
                        labelText2: profilePageController.longTappedItem(longTappedIndex)
                        showIcon: false
                        onClicked: {
                            profilePageController.saveToPlaylist(-1, longTappedIndex)
                            rootWindow.hideDialog()
                        }
                    }
                }
            }
        }
    }
}
