import QtQuick 1.1
import com.meego 1.0
import "../Components"
import "../Components/constants.js" as UI

Page {
    id: wrapper
    property string pageName: "SearchPage"
    property string currentSearch: "tracks"
    property int defaultSearchAmount: 10
    property int tracksType: 0
    property int albumsType: 1
    property int artistsType: 2
    property int longTappedIndex
    property string artistOfLongTappedItem: ""
    property bool addingTrackToPlaylist

    orientationLock: 1 // PageOrientation.LockPortrait

    onStatusChanged: {
        switch(status) {
            case PageStatus.Activating: {
//                viewTitlePanel.headerText = "Search"

                if (currentSearch == "tracks") {
                    if (searchPageModel) {
                        searchPageModel.setCurrentType(tracksType)
                    }
                } else if (currentSearch == "albums") {
                    if (searchPageModel) {
                        searchPageModel.setCurrentType(albumsType)
                    }
                } else {
                    if (searchPageModel) {
                        searchPageModel.setCurrentType(artistsType)
                    }
                }
                if (searchPageController.getKeyword() != "") {
                    textField.text = searchPageController.getKeyword()
                }
                searchPageController.setVisible(true)
            }
            default: textField.platformCloseSoftwareInputPanel();
        }
    }

    Connections {
        target: searchPageController
        onActivateSearch: {
            textField.platformCloseSoftwareInputPanel();
            textField.text = search
            myList.model = 0
            loadingSpinner.visible = true
            engine.search(search, 0, defaultSearchAmount,
                          0, defaultSearchAmount,
                          0, defaultSearchAmount,
                          false, currentSearch)
        }
    }

    ContextMenuData {
        id: menuDataTracks
        model: ListModel {
            ListElement {
                action: "Queue"
                subText: "Add this track to Queue"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Add to Playlist"
                subText: "Save this track to Playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "View Album"
                subText: "View tracks of this album"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "View Artist"
                subText: "More from this artist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share Track"
                subText: "Share this track with others"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share to Spotify People"
                subText: "Send to a friend's Inbox"
                hasCheckBox: false
                actionOn: false
            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Queue": {
                    searchPageController.handleAddToQueue(longTappedIndex)
                }
                case "Add to Playlist": {
                    addingTrackToPlaylist = true
                    rootWindow.showDialog("Save to playlist", saveToPlaylistContent)
                }
                case "View Album": {
                    albumPageController.resetAlbumPage()
                    goToPage("AlbumPage")
                    searchPageController.getAlbumData(longTappedIndex)
                }
                case "View Artist": {
                    artistPageController.clearData()
                    goToPage("ArtistPage")
                    searchPageController.getArtistData(longTappedIndex)
                }
                case "Share Track": {
                    searchPageController.shareLongTappedTrack(longTappedIndex, false)
                }
                case "Share to Spotify People": {
                    searchPageController.shareLongTappedTrack(longTappedIndex, true)
                    goToPage("SpotifySharePage")
                }
            }
        }
    }

    ContextMenuData {
        id: menuDataAlbums
        model: ListModel {
            ListElement {
                action: "Queue"
                subText: "Add this album to Queue"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Add to Playlist"
                subText: "Save this album to Playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "View Artist"
                subText: "More from this artist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share Album"
                subText: "Share this album with others"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share to Spotify People"
                subText: "Send to a friend's Inbox"
                hasCheckBox: false
                actionOn: false
            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Queue": {
                    searchPageController.handleAddAlbumToQueue(longTappedIndex)
                }
                case "Add to Playlist": {
                    addingTrackToPlaylist = false
                    rootWindow.showDialog("Save to playlist", saveToPlaylistContent)
                }
                case "View Artist": {
                    artistPageController.clearData()
                    goToPage("ArtistPage")
                    searchPageController.getArtistData(longTappedIndex)
                }
                case "Share Album": {
                    searchPageController.shareLongTappedAlbum(longTappedIndex, false)
                }
                case "Share to Spotify People": {
                    searchPageController.shareLongTappedAlbum(longTappedIndex, true)
                    goToPage("SpotifySharePage")
                }
            }
        }
    }

    ScrollBar {
        id: scrollBar
        z: 1
        objectName: "scrollBar"
        scrollArea: myList
        onScrollingStarted: searchPageModel.setFastScroll(true)
        onScrollingStopped: {
            searchPageModel.setFastScroll(false)
            searchPageModel.setMoving(false)
            searchPageModel.loadAlbumArts()
        }
    }
/*
    ViewTitlePanel {
        id: viewTitlePanel
        z: 2
        objectName: "viewTitlePanel"
        anchors.top: parent.top
        anchors.topMargin: rootWindow.statusBarHeight
        height: 72
        headerText: "SearchPage"
        onlineMode: engineLoggedIn
        imageSource: nowPlayingPageController.HasValidTrack ?
                "/usr/share/spotify/pics/Spotify-31.png" : ""
        imageSourcePressed: nowPlayingPageController.HasValidTrack ?
                        "/usr/share/spotify/pics/Spotify-61.png" : ""
        onClicked: {
            if (nowPlayingPageController.HasValidTrack) {
                rootWindow.showNPSheet()
            }
        }
    }
*/
    Rectangle {
        id: headerArea
        objectName: "headerArea"
        color: "black"
        width: parent.width
        anchors.top: parent.top
        height: childrenRect.height + 24
        Image {
            id: searchField
            objectName: "searchField"
            source: "/usr/share/spotify/pics/viewtitlepanel_background.png"
            height: sourceSize.height
            width: sourceSize.width
            anchors.top: parent.top

            Image {
                id: npImage
                visible: true
                anchors.right: parent.right
                anchors.rightMargin: 12
                anchors.verticalCenter: textField.verticalCenter
                width: nowPlayingPageController.HasValidTrack ? 42 : 0
                height: width
                source: nowPlayingPageController.HasValidTrack ?
                            (npImageArea.pressed ? "/usr/share/spotify/pics/Spotify-61.png" : "/usr/share/spotify/pics/Spotify-31.png") : ""
//                scale: npImageArea.pressed ? 0.8 : 1.0
                MouseArea {
                    id: npImageArea
                    anchors.fill: parent
                    onClicked: {
                        if (nowPlayingPageController.HasValidTrack) {
                            rootWindow.showNPSheet()
                        }
                    }
                }
            }

            TextField {
                objectName: "textField"
                id: textField
                anchors.left: parent.left
                anchors.right: npImage.left
                anchors.top: searchField.top
                anchors.leftMargin: 12
                anchors.topMargin: 12
                anchors.rightMargin: 12
                placeholderText: "Search"
                inputMethodHints: Qt.ImhNoPredictiveText | Qt.ImhNoAutoUppercase
                style: TextFieldStyle { paddingRight: clearButton.width }
                platformSipAttributes: SipAttributes {
                    actionKeyLabel: "Search"
//                    actionKeyHighlighted: true
                }
                font {
                    family: UI.FONT_FAMILY
                    pixelSize: UI.FONT_SIZE_LARGE
                }
                Keys.onReturnPressed: {
                    textField.platformCloseSoftwareInputPanel()
                    myList.focus = true
                    if (textField.text == "") {
                        myList.model = 0
                        searchPageController.clearModel()
                    } else {
                        searchPageController.search(textField.text)
                    }
                }
                onTextChanged: text == "" ? clearButton.state = "hidden" :
                                            clearButton.state = "visible"

                Image {
                    id: clearButton
                    objectName: "clearButton"
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.right: parent.right
                    anchors.rightMargin: 5
                    source: "/usr/share/spotify/pics/icon-m-input-clear.svg"
                    scale: mouseArea.pressed ? 0.8 : 1.0
                    opacity: 0
                    state: "hidden"

                    MouseArea {
                        id: mouseArea
                        anchors.fill: parent
                        onClicked: {
                            searchPageController.clearKeyword()
                            textField.text = ""
                        }
                    }

                    states:  [
                        State {
                            name: "visible"
                            PropertyChanges { target: clearButton; opacity: 1 }
                        },
                        State {
                            name: "hidden"
                            PropertyChanges { target: clearButton; opacity: 0 }
                        }
                    ]

                    transitions: [
                        Transition {
                            NumberAnimation { property: "opacity"; easing.type: Easing.Linear; duration: 300 }
                        }
                    ]
                }
            }
        }

            Item {
                id: buttonContainer
                anchors.bottom: headerArea.bottom
                anchors.topMargin: 12
                width: parent.width
                height: 45
/*                Rectangle {
                    id: vertSeparator1
                    z:2
                    objectName: "vertSeparator1"
                    height: 1
                    width: parent.width
                    anchors.left: buttonContainer.left
                    anchors.right: buttonContainer.right
                    anchors.top: buttonContainer.top
                    color: "grey"
                }
  */              Rectangle {
                    id: tracks
                    objectName: "tracks"
                    width: parent.width / 3
                    height: parent.height
                    anchors.left: parent.left
                    anchors.top: parent.top
                    color: currentSearch == "tracks" ? "#4f4f4f" : "#232323"
                    Text {
                        id: tracksText
                        objectName: "tracksText"
                        anchors.bottom: parent.bottom
                        anchors.bottomMargin: 10
                        anchors.horizontalCenter: parent.horizontalCenter
                        text: "Tracks"
                        color: currentSearch == "tracks" ? "white" : "grey"
                        font.pixelSize: UI.FONT_SIZE_NORMAL
                        font.family: UI.FONT_FAMILY
                    }
                    MouseArea {
                        objectName: "tracksMouseArea"
                        anchors.fill: parent
                        onClicked: {
                            currentSearch = "tracks"
                            if (textField.text == "") {
                                myList.model = 0
                            } else {
                                myList.positionViewAtIndex(0, ListView.Beginning);
                                if (searchPageModel != null) {
                                    searchPageModel.setCurrentType(tracksType)
                                    if (searchPageModel.getCurrentTypeCount() > 0) {
                                        myList.model = searchPageModel
                                        myList.delegate = searchDelegate
                                    } else {
                                        myList.model = 1
                                        myList.delegate = didYouMeanDelegate
                                    }
                                }
                                if (textField.text != searchPageController.getKeyword()) {
                                    searchPageController.search(textField.text)
                                }
                            }
                        }
                    }
                }
                Rectangle {
                    id: albums
                    objectName: "albums"
                    width: parent.width / 3
                    height: parent.height
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.top: parent.top
                    color: currentSearch == "albums" ? "#4f4f4f" : "#232323"
                    Text {
                        id: albumsText
                        objectName: "albumsText"
                        anchors.bottom: parent.bottom
                        anchors.bottomMargin: 10
                        anchors.horizontalCenter: parent.horizontalCenter
                        text: "Albums"
                        color: currentSearch == "albums" ? "white" : "grey"
                        font.pixelSize: UI.FONT_SIZE_NORMAL
                        font.family: UI.FONT_FAMILY
                    }
                    Rectangle {
                        id: horizSeparator1
                        objectName: "horizSeparator1"
                        width: 1
                        height: parent.height
                        anchors.left: albums.left
                        anchors.bottom: parent.bottom
                        color: "grey"
                    }
                    Rectangle {
                        id: horizSeparator2
                        objectName: "horizSeparator2"
                        width: 1
                        height: parent.height
                        anchors.right: albums.right
                        anchors.bottom: parent.bottom
                        color: "grey"
                    }
                    MouseArea {
                        objectName: "albumsMouseArea"
                        anchors.fill: parent
                        onClicked: {
                            currentSearch = "albums"
                            if (textField.text == "") {
                                myList.model = 0
                            } else {
                                myList.positionViewAtIndex(0, ListView.Beginning);
                                if (searchPageModel != null) {
                                    searchPageModel.setCurrentType(albumsType)
                                    if (searchPageModel.getCurrentTypeCount() > 0) {
                                        myList.model = searchPageModel
                                        myList.delegate = searchDelegate
                                    } else {
                                        myList.model = 1
                                        myList.delegate = didYouMeanDelegate
                                    }
                                }
                                if (textField.text != searchPageController.getKeyword()) {
                                    searchPageController.search(textField.text)
                                }
                            }
                        }
                    }
                }
                Rectangle {
                    id: artists
                    objectName: "artists"
                    width: parent.width / 3
                    height: parent.height
                    anchors.right: parent.right
                    anchors.top: parent.top
                    color: currentSearch == "artists" ? "#4f4f4f" : "#232323"
                    Text {
                        id: artistsText
                        objectName: "artistsText"
                        anchors.bottom: parent.bottom
                        anchors.bottomMargin: 10
                        anchors.horizontalCenter: parent.horizontalCenter
                        text: "Artists"
                        color: currentSearch == "artists" ? "white" : "grey"
                        font.pixelSize: UI.FONT_SIZE_NORMAL
                        font.family: UI.FONT_FAMILY
                    }
                    MouseArea {
                        objectName: "artistsMouseArea"
                        anchors.fill: parent
                        onClicked: {
                            currentSearch = "artists"
                            if (textField.text == "") {
                                myList.model = 0
                            } else {
                                myList.positionViewAtIndex(0, ListView.Beginning);
                                if (searchPageModel != null) {
                                    searchPageModel.setCurrentType(artistsType)
                                    if (searchPageModel.getCurrentTypeCount() > 0) {
                                        myList.model = searchPageModel
                                        myList.delegate = searchDelegate
                                    } else {
                                        myList.model = 1
                                        myList.delegate = didYouMeanDelegate
                                    }
                                }
                                if (textField.text != searchPageController.getKeyword()) {
                                    searchPageController.search(textField.text)
                                }
                            }
                        }
                    }
                }
            }
            Rectangle {
                id: separator
                objectName: "separator"
                width: parent.width
                height: 2
                anchors.bottom: headerArea.bottom
                color: "dimgrey"
            }

    }

    ListView {
        id: myList
        objectName: "searchResults"
        clip: true
        anchors.fill: parent
        anchors.topMargin: headerArea.height
        anchors.bottomMargin: toolbar.height
        property real contentBottom: contentY + height
        //cacheBuffer : 2000

        onContentYChanged: {
            if (contentY > contentHeight - height - 400 && searchPageModel
                && searchPageModel.hasMoreItems() && !searchPageModel.isWaitingForMore())
            {
                searchPageModel.fetchMoreItems(defaultSearchAmount);
            }
        }
        onMovementEnded: {
            searchPageModel.setMoving(false)
            searchPageModel.loadAlbumArts()
        }
        onMovementStarted: searchPageModel.setMoving(true)
    }

    Connections {
        target: searchPageController
        onSearchReady: {
            loadingSpinner.visible = false
            if (searchPageModel.getCurrentTypeCount() > 0) {
                myList.delegate = searchDelegate
                myList.model = searchPageModel
                searchPageModel.loadAlbumArts()
            } else {
                myList.delegate = didYouMeanDelegate
                myList.model = 1
            }
        }
    }

    LoadingSpinner {
        id: loadingSpinner
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: myList.verticalCenter
        visible:false
    }

    Component {
        id: searchDelegate
        ListItem {
            objectName: "searchResultItem"
            imageSource: currentSearch == "artists" ? (AlbumArt == "" ? "/usr/share/spotify/pics/Spotify-07.png" : AlbumArt) : (AlbumArt == "" ? "/usr/share/spotify/pics/Spotify-07.png" : AlbumArt)
            labelText1: TitleRole
            labelText2: TrackSubHeader
            allowRightImage: currentSearch == "tracks"
            starred: TrackStarred
            enableAddToQueue: currentSearch == "tracks" ? true : false
            showProductImage: true
            onClicked: {
                if (currentSearch == "tracks") {
                    searchPageController.listClicked(index)
                    goToPage("NowPlayingPage")
                } else if (currentSearch == "albums") {
                    albumPageController.resetAlbumPage()
                    goToPage("AlbumPage")
                    searchPageController.listClicked(index)
                } else {
                    searchPageController.setCurrentArtist(index)
                    artistPageController.clearData()
                    goToPage("ArtistPage")
                    artistPageController.setTab("overview")
                    searchPageController.listClicked(index)
                }
            }
            onStarClicked: {
                searchPageController.setStarred(index)
            }
            property bool isVisible : (((index * height) >= myList.contentY &&
                                        (index * height) <= myList.contentBottom) ||
                                       ((index * height) + height >= myList.contentY &&
                                        (index * height) + height <= myList.contentBottom))
            onIsVisibleChanged: {
                if (searchPageModel) {
                    searchPageModel.setItemVisible(isVisible, index);
                }
            }

            onPressAndHold: {
                if (currentSearch == "tracks") {
                    viewMenu.menuData = menuDataTracks
                    artistOfLongTappedItem = ArtistRole
                    longTappedIndex = index;
                    viewMenu.open(labelText1, ArtistRole, imageSource)
                } else if (currentSearch == "albums") {
                    viewMenu.menuData = menuDataAlbums
                    artistOfLongTappedItem = ArtistRole
                    longTappedIndex = index;
                    viewMenu.open(labelText1, ArtistRole, imageSource)
                }
            }
            onSwiped: if (currentSearch == "tracks") {
                searchPageController.handleAddToQueue(index)
            }
        }
    }

    Component {
        id: didYouMeanDelegate

        Item {
            width: parent.width
            height: 450

            Text {
                id: didYouMeanText
                width: parent.width - 100
                anchors.centerIn: parent
                font.pixelSize: 40
                font.family: UI.FONT_FAMILY
                color: "black"
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                text: {
                    if (searchPageModel.DidYouMean != "") {
                        "Did you mean <u><font color=\"#67AB16\">" + searchPageModel.DidYouMean + "</font></u> ?"
                    } else {
                        "No " + currentSearch + " found."
                    }
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        if (searchPageModel.DidYouMean != "") {
                            textField.text = searchPageModel.DidYouMean
                            searchPageController.search(textField.text)
                        }
                    }
                }
            }
        }
    }

    Component {
        id: saveToPlaylistContent
        Item {
            height: childrenRect.height
            width: dialogLoader.status ==  Loader.Ready ? dialogLoader.item.contentWidth : 0

            ListView {
                id: playlists
                objectName: "playlists"
                clip: true
                width: parent.width
                height: Math.max(150, Math.min(600, count * 80 + 80))
                model: editablePlaylistsModel
                header: headerComponent
                cacheBuffer: 800
                delegate: ListItem {
                    inverted: true
                    labelText1: playlistName
                    showIcon: false
                    onClicked: {
                        searchPageController.saveToPlaylist(index, longTappedIndex, addingTrackToPlaylist)
                        rootWindow.hideDialog()
                    }
                }

                Component {
                    id: headerComponent
                    ListItem {
                        inverted: true
                        labelText1: "<Add as a new playlist>"
                        labelText2: searchPageController.longTappedItem(longTappedIndex)
                        showIcon: false
                        onClicked: {
                            searchPageController.saveToPlaylist(-1, longTappedIndex, addingTrackToPlaylist)
                            rootWindow.hideDialog()
                        }
                    }
                }
            }
        }
    }
}
