import QtQuick 1.1
import com.meego 1.0
import "../Components" as Components
import "../Components/constants.js" as UI

Page {
    id: settingsPage
    property string pageName: "SettingsPage"
    property string cacheSizeString: "Updating..."

    orientationLock: 1

    onStatusChanged: {
        switch(status) {
            case PageStatus.Activating: {
                viewTitlePanel.headerText = "Settings"
            }
            case PageStatus.Active: {
                cacheSizeString = "Cache size: " + settings.GetCacheSize
            }
        }
    }

    Components.ViewTitlePanel {
        id: viewTitlePanel
        z: 2
        objectName: "viewTitlePanel"
        anchors.top: parent.top
        anchors.topMargin: rootWindow.statusBarHeight
        height: 72
        headerText: "SettingsPage"
        onlineMode: engineLoggedIn
        imageSource: nowPlayingPageController.HasValidTrack ?
                "/usr/share/spotify/pics/Spotify-31.png" : ""
        imageSourcePressed: nowPlayingPageController.HasValidTrack ?
                        "/usr/share/spotify/pics/Spotify-61.png" : ""
        onClicked: {
            if (nowPlayingPageController.HasValidTrack) {
                rootWindow.showNPSheet()
            }
        }
    }

    ListView {
        id: settingsList
        objectName: "settings"
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: viewTitlePanel.bottom
        anchors.bottom:  parent.bottom
       // model:list
        header: listHeader
        interactive: false
    }
    Component {
        id: listHeader
        Column {
            objectName: "listHeader"
            width: parent.width
            height: childrenRect.height

            Components.GroupHeaderDivider {
                id: audioQualityHeader
                objectName: "audioQualityHeader"
                headerText: "Audio Quality"
            }
            Components.ListItem {
                id: rate
                objectName: "bitRate"
                showIcon: false
                labelText1: "Streaming"
                labelText2: if (settings.streamBitRate) {
                                return "High bandwidth 320k"
                            } else {
                                return "Standard bandwidth 160k"
                            }
                onClicked: rootWindow.showDialog("Streaming", changeBitrateContent)
            }

            Components.ListItem {
                id: syncrate
                objectName: "syncrate"
                showIcon: false
                labelText1: "Sync"
                labelText2: if (settings.syncBitRate) {
                                return "High bandwidth 320k"
                            } else {
                                return "Standard bandwidth 160k"
                            }
                onClicked: rootWindow.showDialog("Sync", changeSyncBitrateContent)
            }

            Components.GroupHeaderDivider {
                id: lfmHeader
                objectName: "lfmHeader"
                headerText: "Last.fm"
            }

            Components.ListItem {
                id: scrobbling
                objectName: "scrobbling"
                showIcon: false
                labelText1: "Scrobble to Last.fm"
                onClicked: scrobbleSheet.open() //rootWindow.showDialog("Scrobble to Last.fm", scrobbleContent)
                Switch {
                    id: scrobblingSwitch
                    anchors.verticalCenter: scrobbling.verticalCenter
                    anchors.right: parent.right
                    anchors.rightMargin: 15
                    objectName: "scrobblingSwitch"
                    checked: settings.scrobbling
                }
                MouseArea {
                    anchors.fill: scrobblingSwitch
                    onClicked: {
                        if (!settings.scrobbling) {
                            nowPlayingPageController.authenticateLastFMSession()
                        } else {
                            notifyController.showMessage("Last.fm scrobbling switched off")
                        }
                        settings.setScrobbling(!settings.scrobbling)
                    }
                }
            }

            Components.GroupHeaderDivider {
                id: applicationHeader
                objectName: "applicationHeader"
                headerText: "Application"
            }

            Components.ListItem {
                id: clear
                objectName: "Clear"
                showIcon: false
                labelText1: "Clear saved data"
                labelText2: cacheSizeString
                onClicked: clearCacheContent.open()
            }
            Components.ListItem {
                id: about
                objectName: "About"
                showIcon: false
                labelText1: "About Spotify"
                labelText2: "Disk usage, version number, ..."
                onClicked: rootWindow.showDialog("About Spotify", aboutContent)
            }

            Components.ListItem {
                id: logoff
                objectName: "logoff"
                showIcon: false
                labelText1: "Log off"
                labelText2: "Log off to switch user"
                onClicked: { goToPage("PlaylistsPage"); engine.logout() }
            }
        }
    }

    ListModel {
        id: bitrateModel

        ListElement {
            desc: "Standard Bandwidth"
            bitrate: "Stream music in 160kbit/s"
        }
        ListElement {
            desc: "High Quality"
            bitrate: "Stream music in 320kbit/s"
        }
    }

    ListModel {
        id: syncBitrateModel

        ListElement {
            desc: "Standard Bandwidth"
            bitrate: "Sync music in 160kbit/s"
        }
        ListElement {
            desc: "High Quality"
            bitrate: "Sync music in 320kbit/s"
        }
    }

    Component {
        id: changeBitrateContent

        Item {
            height: childrenRect.height
            width: dialogLoader.status ==  Loader.Ready ? dialogLoader.item.contentWidth : 0

            ListView {
                id: bitrateList
                clip: true
                width: parent.width
                height: 160
                model: bitrateModel
                interactive: false
                delegate: Components.ListItem {
                    inverted: true
                    labelText1: desc
                    labelText2: bitrate
                    showIcon: false
                    onClicked: {
                        settings.setStreamBitRate(index)
                        rootWindow.hideDialog()
                    }
                }
            }
        }
    }

    Component {
        id: changeSyncBitrateContent

        Item {
            height: childrenRect.height
            width: dialogLoader.status ==  Loader.Ready ? dialogLoader.item.contentWidth : 0

            ListView {
                id: syncBitrateList
                clip: true
                width: parent.width
                height: 160
                model: syncBitrateModel
                interactive: false
                delegate: Components.ListItem {
                    inverted: true
                    labelText1: desc
                    labelText2: bitrate
                    showIcon: false
                    onClicked: {
                        settings.setSyncBitRate(index)
                        rootWindow.hideDialog()
                    }
                }
            }
        }
    }

    QueryDialog {
        id: clearCacheContent

        titleText: "Clear Saved Data?"
        message: "Are you sure you want to clear the cache? All offline playlists and cached music will be deleted, which may cause unnecessary bandwidth usage.\n\nWe recommed that you only do this if you are about to uninstall Spotify from your phone.\n\nIf you wish to remove offline playlists, you may do so from the Playlists view (clicking the Offline playlists button) instead."
        acceptButtonText: "Delete"
        rejectButtonText: "Cancel"

        onAccepted: {
            if (settings.DeleteCache) {
                notifyController.showMessage("Cache deleted successfully")
            } else {
                notifyController.showMessage("Failed to delete cache")
            }
        }
    }
    /*
    Component {
        id: clearCacheContent
        Item {
            height: childrenRect.height
            width: dialogLoader.status ==  Loader.Ready ? dialogLoader.item.contentWidth : 0

            Text {
                id: text
                width: parent.width
                text: "Are you sure you want to clear the cache? All offline playlists and cached music will be deleted, which may cause unnecessary bandwidth usage.\n\nWe recommed that you only do this if you are about to uninstall Spotify from your phone.\n\nIf you wish to remove offline playlists, you may do so from the Playlists view (clicking the Offline playlists button) instead."
                font.pixelSize: UI.FONT_SIZE_NORMAL
                font.family: UI.FONT_FAMILY
                color: "white"
                wrapMode: Text.WordWrap
            }

            Item {
                anchors.top: text.bottom
                anchors.topMargin: 30
                width: parent.width
                height: 60
                anchors.left: parent.left
                anchors.leftMargin: 30

                Button {
                    id: ok
                    objectName: "okButton"
                    anchors.left: parent.left
                    anchors.leftMargin: 80
                    width: 140
                    text: "Ok"
                    platformStyle: ButtonStyle { inverted: true }
                    onClicked: {
                        if (settings.DeleteCache) {
                            notifyController.showMessage("Cache deleted successfully")
                        } else {
                            notifyController.showMessage("Failed to delete cache")
                        }
                        rootWindow.hideDialog()
                    }
                }
            }
        }
    }
*/
    Component {
        id: aboutContent

        Item {
            height: childrenRect.height
            width: dialogLoader.status ==  Loader.Ready ? dialogLoader.item.contentWidth : 0

            Column {
                width: parent.width
                height: childrenRect.height

                Text {
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.leftMargin: 15
                    anchors.rightMargin: 15
                    font.pixelSize: UI.FONT_SIZE_NORMAL
                    font.family: UI.FONT_FAMILY
                    color: "grey"
                    text: settings.GetCopyrightText
                    wrapMode: Text.Wrap
                    horizontalAlignment: Text.AlignHCenter
                }
                Components.ListItem {
                    labelText1: "Version"
                    labelText2: settings.GetAppVersion
                    showIcon: false
                    inverted: true
                    anchors.horizontalCenter: parent.horizontalCenter
                }
                Components.ListItem {
                    labelText1: "Available"
                    labelText2: settings.GetAvailableDiskSpace
                    showIcon: false
                    inverted: true
                    anchors.horizontalCenter: parent.horizontalCenter
                }
                Components.ListItem {
                    labelText1: "Total Disk Space"
                    labelText2: settings.GetTotalDiskSpace
                    showIcon: false
                    inverted: true
                    anchors.horizontalCenter: parent.horizontalCenter
                }
                Components.ListItem {
                    labelText1: "Used Disk Space"
                    labelText2: settings.GetUsedDiskSpace
                    showIcon: false
                    inverted: true
                    anchors.horizontalCenter: parent.horizontalCenter
                }
            }
        }
    }

    Sheet {
        id: scrobbleSheet
        z: 4
        visualParent: settingsPage
        acceptButtonText: "Save"
        rejectButtonText: "Cancel"

        onAccepted: {
            settings.setUsernameLastFM(userNameField.text)
            settings.setPasswordLastFM(passwordField.text)
            if (settings.scrobbling) {
                nowPlayingPageController.authenticateLastFMSession()
            }
        }

        content: Rectangle {
            id: scrobbleContainer
            anchors.fill: parent
            color: "black"
            property string newUser: ""
            property string newPwd: ""

            MouseArea {
                anchors.fill: scrobbleContainer
            }

            Components.ViewTitlePanel {
                id: scrobbleSheetTitle
                backgroundVisible: false
                anchors.top: parent.top
                height: 72
                headerText: "Scrobble Last.fm"
            }

            Rectangle {
                height: 1
                width: parent.width
                color: "gray"
                anchors.top: scrobbleSheetTitle.bottom
            }

            Text {
                id: userName
                anchors.top: scrobbleSheetTitle.bottom
                anchors.topMargin: 20
                anchors.left: parent.left
                anchors.leftMargin: 15
                width: parent.width
                text: "Username:"
                font.pixelSize: UI.FONT_SIZE_NORMAL
                font.family: UI.FONT_FAMILY
                color: "white"
                wrapMode: Text.WordWrap
            }

            TextField {
                id: userNameField
                anchors.top: userName.bottom
                anchors.topMargin: 15
                anchors.left: userName.left
                width: parent.width - 30
                font.pixelSize: UI.FONT_SIZE_NORMAL
                inputMethodHints: Qt.ImhNoPredictiveText | Qt.ImhNoAutoUppercase
                text: settings.usernameLastFM
                onTextChanged: scrobbleContainer.newUser = userNameField.text
                Keys.onReturnPressed: {
                    userNameField.platformCloseSoftwareInputPanel()
                    scrobbleContainer.focus = true
                }
            }

            Text {
                id: password
                anchors.top: userNameField.bottom
                anchors.topMargin: 15
                anchors.left: userName.left
                width: parent.width
                text: "Password:"
                font.pixelSize: UI.FONT_SIZE_NORMAL
                font.family: UI.FONT_FAMILY
                color: "white"
                wrapMode: Text.WordWrap
            }

            TextField {
                id: passwordField
                anchors.top: password.bottom
                anchors.topMargin: 15
                anchors.left: userName.left
                width: parent.width - 30
                font.pixelSize: UI.FONT_SIZE_NORMAL
                text: settings.passwordLastFM
                onTextChanged: scrobbleContainer.newPwd = passwordField.text
                echoMode: TextInput.Password
                inputMethodHints: Qt.ImhNoPredictiveText | Qt.ImhNoAutoUppercase
                Keys.onReturnPressed: {
                    passwordField.platformCloseSoftwareInputPanel()
                    scrobbleContainer.focus = true
                }
            }
        }
    }
}

