import QtQuick 1.1
import com.meego 1.0
import "../Components" as Components
import "../Components/constants.js" as UI

Page {
    id: wrapper
    property string pageName: "SpotifySharePage"
    property int longTappedIndex
    property bool listViewEnabled: true
    property string recipientText: ""
    property int friendsCount: shareController.friendCount

    orientationLock: 1 // PageOrientation.LockPortrait

    onFriendsCountChanged: {
        if (!friendsSpinner.visible &&friendsCount == 0) {
            noFriendsText.visible = true
        }
    }

    onStatusChanged: {
        switch(status) {
            case PageStatus.Activating: {
                viewTitlePanel.headerText = "Send to Inbox"
                shareController.setVisible(true)
            }
            case PageStatus.Deactivating: {
                shareController.setVisible(false)
            }
        }
    }

    Text {
        id: noFriendsText
        visible: !shareController.ListLoading && friendsCount == 0
        text: "No Spotify People available"
        width: parent.width - 20
        anchors.centerIn: parent
        font.pixelSize: UI.FONT_SIZE_LARGE
        font.family: UI.FONT_FAMILY
        color: "Black"
        horizontalAlignment: Text.AlignHCenter
        wrapMode: Text.WordWrap
    }

    Components.LoadingSpinner {
        z: 1
        id: friendsSpinner
        visible: shareController.ListLoading
        anchors.fill: parent
    }

    Components.ScrollBar {
        id: scrollBar
        z: 1
        objectName: "scrollBar"
        scrollArea: friendList
    }

    Components.ViewTitlePanel {
        id: viewTitlePanel
        z: 2
        objectName: "viewTitlePanel"
        anchors.top: parent.top
        anchors.topMargin: rootWindow.statusBarHeight
        height: 72
        headerText: "SpotifySharePage"
        onlineMode: engineLoggedIn
        imageSource: nowPlayingPageController.HasValidTrack ?
                "/usr/share/spotify/pics/Spotify-31.png" : ""
        imageSourcePressed: nowPlayingPageController.HasValidTrack ?
                        "/usr/share/spotify/pics/Spotify-61.png" : ""
        onClicked: {
            if (nowPlayingPageController.HasValidTrack) {
                rootWindow.showNPSheet()
            }
        }
    }

    ListView {
        id: friendList
        objectName: "friendList"
        width: parent.width
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: viewTitlePanel.bottom
        anchors.bottom:  parent.bottom
        anchors.bottomMargin: albumAndArtistPageToolBar.height
        model: shareToFriendModel
        clip: true
        enabled: listViewEnabled
        header: shareDetails
        cacheBuffer: 800

        delegate: Components.ListItem {
            objectName: "friendItem"
            labelText1: FriendTitle
            imageSource: FriendProfilePicture ? FriendProfilePicture : "/usr/share/spotify/pics/Spotify-07.png"
            allowRightImage: false
            showProductImage: true
            onClicked: {
                shareController.resetMessageLength()
                rootWindow.showDialog("Send to "+labelText1, sendMessageContent)
                recipientText = labelText1
                shareController.listClicked(index)
            }
        }

        Component {
            id: shareDetails

            Rectangle {
                id: details
                objectName: "detailsArea"
                color: "#2d292a"
                height: childrenRect.height + 30
                width: parent.width

                Image {
                    id: albumArt
                    objectName: "albumArt"
                    anchors.top: parent.top
                    anchors.topMargin: 15; anchors.leftMargin: 15
                    anchors.left: parent.left
                    width: 115; height: width
                    source: shareController.ImagePath
                }
                Text {
                    id: mainItem
                    objectName: "mainItem"
                    anchors.top: albumArt.top
                    anchors.left: albumArt.right; anchors.right: parent.right
                    anchors.leftMargin: 15
                    font.bold: true; font.family: UI.FONT_FAMILY
                    text: shareController.MainText
                    color: "white"; font.pointSize: 16
                    elide: Text.ElideRight
                }
                Text {
                    id: secondaryItem
                    objectName: "secondaryItem"
                    anchors.top: mainItem.bottom
                    anchors.left: albumArt.right; anchors.right: parent.right
                    anchors.topMargin: 10
                    anchors.leftMargin: 15
                    font.family: UI.FONT_FAMILY
                    text: shareController.SecondaryText
                    color: "lightgrey"; font.pointSize: 16
                    elide: Text.ElideRight
                }
                Text {
                    id: secondaryItem2
                    objectName: "secondaryItem2"
                    anchors.top: secondaryItem.bottom
                    anchors.left: albumArt.right; anchors.right: parent.right
                    anchors.topMargin: 5
                    anchors.leftMargin: 15
                    anchors.bottomMargin: 30
                    font.family: UI.FONT_FAMILY
                    text: shareController.SecondaryText2
                    color: "lightgrey"; font.pointSize: 16
                    elide: Text.ElideRight
                }
            }
        }
    }

    Component {
        id: sendMessageContent
        Item {
            property bool showTextArea: true
        }
    }
}
