import QtQuick 1.1
import com.meego 1.0
import "../Components" as Components
import "../Components/constants.js" as UI

Page {
    id: wrapper
    property string pageName: "TracksOfPlaylistPage"
    property bool listIsVisible: false
    property int collage: tracksOfPlaylistPageController.Collage
    property string artistOfLongTappedTrack: ""
    property int longTappedIndex
    property bool playlistIsCollaborative: tracksOfPlaylistPageController.PlaylistCollaborative

    orientationLock: 1 // PageOrientation.LockPortrait

    onStatusChanged: {
        switch(status) {
            case PageStatus.Activating: {
                if (tracksOfPlaylistModel.playlistOwner == "") {
                    viewTitlePanel.headerText = tracksOfPlaylistModel.playlistName
                } else {
                    viewTitlePanel.headerText = tracksOfPlaylistModel.playlistName + " (" + tracksOfPlaylistModel.playlistOwner + ")"
                }
                tracksOfPlaylistPageController.setVisible(true)
            }

            case PageStatus.Deactivating: {
                tracksOfPlaylistPageController.setVisible(false)
            }

            case PageStatus.Active: {
                viewMenu.menuData = menuDataEditPlaylist
                if (tracksOfPlaylistModel.playlistOwner == "") {
                    if (tracksOfPlaylistModel.playlistName != "Inbox" &&
                        tracksOfPlaylistModel.playlistName != "Starred") {
                    }
                }
            }
        }
    }

    onPlaylistIsCollaborativeChanged: {
        menuDataEditPlaylist.model.set(1, {"actionOn": playlistIsCollaborative})
    }

    Components.ContextMenuData {
        id: menuDataEditPlaylist
        model: ListModel {
//            ListElement {
//                action: "Delete Playlist"
//                subText: ""
//                hasCheckBox: false
//                actionOn: false
//            }
            ListElement {
                action: "Rename Playlist"
                subText: ""
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Set As Collaborative"
                subText: ""
                hasCheckBox: true
                actionOn: false
            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
//                case "Delete Playlist": {
//                    tracksOfPlaylistPageController.deletePlaylist();
//                }
                case "Rename Playlist": {
                    renameSheet.open()
                }
                case "Set As Collaborative": {
                    tracksOfPlaylistPageController.changePlaylistCollaborativity()
                }
            }
        }
    }

    Components.ContextMenuData {
        id: longTapMenuData
        model: ListModel {
            ListElement {
                action: "Queue"
                subText: "Add this track to Queue"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Remove Track"
                subText: "Remove track from playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "View Album"
                subText: "View tracks of this album"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "View Artist"
                subText: "More from this artist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share Track"
                subText: "Share this track with others"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share to Spotify People"
                subText: "Send to a friend's Inbox"
                hasCheckBox: false
                actionOn: false
            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Queue": {
                    tracksOfPlaylistPageController.handleAddToQueue(longTappedIndex)
                }
                case "Remove Track": {
                    tracksOfPlaylistPageController.removeTrack(longTappedIndex)
                }
                case "View Album": {
                    albumPageController.resetAlbumPage()
                    goToPage("AlbumPage")
                    tracksOfPlaylistPageController.getAlbumData(longTappedIndex)
                }
                case "View Artist": {
                    artistPageController.clearData()
                    goToPage("ArtistPage")
                    tracksOfPlaylistPageController.getArtistData(longTappedIndex)
                }
                case "Share Track": {
                    tracksOfPlaylistPageController.shareLongTappedTrack(longTappedIndex, false)
                }
                case "Share to Spotify People": {
                    tracksOfPlaylistPageController.shareLongTappedTrack(longTappedIndex, true)
                    goToPage("SpotifySharePage")
                }
            }
        }
        onMenuClosed: {
            viewMenu.menuData = menuDataEditPlaylist
        }
    }

    Components.ContextMenuData {
        id: longTapMenuDataFriend
        model: ListModel {
            ListElement {
                action: "Queue"
                subText: "Add this track to Queue"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Add to Playlist"
                subText: "Save this track to Playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "View Album"
                subText: "View tracks of this album"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "View Artist"
                subText: "More from this artist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share Track"
                subText: "Share this track with others"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share to Spotify People"
                subText: "Send to a friend's Inbox"
                hasCheckBox: false
                actionOn: false
            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Queue": {
                    tracksOfPlaylistPageController.handleAddToQueue(longTappedIndex)
                }
                case "Add to Playlist": {
                    rootWindow.showDialog("Save to playlist", saveToPlaylistContent)
                }
                case "View Album": {
                    albumPageController.resetAlbumPage()
                    goToPage("AlbumPage")
                    tracksOfPlaylistPageController.getAlbumData(longTappedIndex)
                }
                case "View Artist": {
                    artistPageController.clearData()
                    goToPage("ArtistPage")
                    tracksOfPlaylistPageController.getArtistData(longTappedIndex)
                }
                case "Share Track": {
                    tracksOfPlaylistPageController.shareLongTappedTrack(longTappedIndex, false)
                }
                case "Share to Spotify People": {
                    tracksOfPlaylistPageController.shareLongTappedTrack(longTappedIndex, true)
                    goToPage("SpotifySharePage")
                }
            }
        }
        onMenuClosed: {
            viewMenu.menuData = menuDataEditPlaylist
        }
    }

    Components.ScrollBar {
        id: scrollBar
        z: 2
        objectName: "scrollBar"
        scrollArea: myList
    }


    Components.ViewTitlePanel {
        id: viewTitlePanel
        z: 2
        objectName: "viewTitlePanel"
        anchors.top: parent.top
        anchors.topMargin: rootWindow.statusBarHeight
        height: 72
        headerText: "TracksOfPlaylistPage"
        onlineMode: engineLoggedIn
        imageSource: nowPlayingPageController.HasValidTrack ?
                "/usr/share/spotify/pics/Spotify-31.png" : ""
        imageSourcePressed: nowPlayingPageController.HasValidTrack ?
                        "/usr/share/spotify/pics/Spotify-61.png" : ""
        onClicked: {
            if (nowPlayingPageController.HasValidTrack) {
                rootWindow.showNPSheet()
            }
        }
    }

    Component {
        id: headerComponent

        Rectangle {
            id: details
            objectName: "playlistInfo"
            color: UI.APP_BACKGROUND_COLOR
            height: 170/*tracksOfPlaylistPageController.LoggedUserIsSubscriber &&
                    tracksOfPlaylistModel.playlistOwner != "" ? 210 : 170 */
            width: parent.width

            GridView {
                id: gridView
                objectName: "collageImages"
                interactive: false
                cellHeight: if (collage < 4) {
                        return 140
                    } else {
                        return 70
                    }
                cellWidth: cellHeight
              //  anchors.verticalCenter: parent.verticalCenter
                anchors.top: parent.top
                anchors.topMargin: 12
                anchors.leftMargin: 15
                anchors.left: parent.left
                width: 140
                height: width
                focus: true
                model: collageImageModel
                delegate: Image {
                    objectName: "collageImage"
                    width: if (collage < 4) {
                           return 140
                        } else {
                           return 70
                        }
                    height: width
                    source: collage == 0 ? "" : TrackAlbumArt
                    //asynchronous: true
                    MouseArea {
                        objectName: "collageImageMouseArea"
                        anchors.fill: parent
                        onClicked: {
                           // goToPage("NowPlayingPage")
                        }
                    }
                }
            }

            Components.TitlePanel{
                id: playlistName
                objectName: "playlistName"
                anchors.top: gridView.top
                anchors.left: gridView.right
                anchors.right: parent.right
                anchors.leftMargin: 15
                anchors.rightMargin: 15
                headerText: tracksOfPlaylistModel.playlistName
            }
            Components.PropertiesDetailPanel {
                id: playlislOwner
                objectName: "playlislOwner"
                anchors.top: playlistName.bottom
                anchors.left: gridView.right
                anchors.leftMargin: 15
                visible: tracksOfPlaylistModel.playlistOwner == "" ? false : true
                headerText: visible ? "by " + tracksOfPlaylistModel.playlistOwner :
                ""
            }
            Components.PropertiesDetailPanel {
                id: trackCount
                objectName: "trackCount"
                anchors.top: playlislOwner.bottom
                anchors.left: gridView.right
                anchors.leftMargin: 15
                headerText: myList.count + " tracks, " + tracksOfPlaylistModel.duration
            }

            Button {
                id: subscribeButton
                objectName: "subscribeButton"
                property string previousButtonText: ""
                visible: tracksOfPlaylistModel.playlistOwner != "" && tracksOfPlaylistModel.playlistName != "Starred"
                anchors.right: parent.right
                anchors.rightMargin: 15
                anchors.bottom: gridView.bottom
                width: 200
                text: tracksOfPlaylistPageController.LoggedUserIsSubscriber ? "Unsubscribe" : "Subscribe"
                platformStyle: ButtonStyle { inverted: false }
                onClicked:{
                    if (previousButtonText == text) {
                        return
                    } else {
                        previousButtonText = text
                        tracksOfPlaylistPageController.changeSubscribing()
                        myList.positionViewAtBeginning()
                    }
                }
            }

            Components.TitlePanel {
                id: offlinePlaylist
                visible: false // subscribeButton.text !=  "Subscribe" || !subscribeButton.visible
                objectName: "offlinePlaylist"
                anchors.bottom: subscribeButton.visible ? line.top : gridView.bottom
                anchors.topMargin: 15
                anchors.bottomMargin: 15
                anchors.left: subscribeButton.visible ? parent.left : gridView.right
                anchors.leftMargin: 15
                headerText: "Available offline"
                fontBold: false
            }

            Switch {
                objectName: "switch"
                visible: offlinePlaylist.visible
                anchors.right: parent.right
                anchors.rightMargin: 15
                anchors.verticalCenter: offlinePlaylist.verticalCenter
                platformStyle: SwitchStyle { inverted: false }
                onCheckedChanged: {
                    console.log("offlinePlaylist switch " + checked)
                }
            }

            Rectangle {
                id: line
                height: 1
                color: "grey"
                anchors.bottom: parent.bottom
                width: parent.width - 30
                anchors.horizontalCenter: parent.horizontalCenter
            }
        }
    }

    ListView {
        id: myList
        objectName: "tracksOfPlaylist"
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: viewTitlePanel.bottom
        anchors.bottom:  parent.bottom
        anchors.bottomMargin: albumAndArtistPageToolBar.height
        header: headerComponent
        model: tracksOfPlaylistModel
        delegate: tracksOfPlaylistModel.playlistName == "Inbox" ? expanding : normal
        cacheBuffer: 1200
        Component.onCompleted: {
            myList.positionViewAtBeginning()
        }
    }

    Component {
        id: expanding
        Components.ExpandingListItem {
            objectName: "tracksOfPlaylistItem"
            labelText1: TrackTitle
            labelText2: tracksOfPlaylistModel.getTrackSender(index) + " | " +
                    tracksOfPlaylistModel.getTrackAge(index) + " | " + TrackArtist
            starred: TrackStarred
            isSeen: TrackIsSeen
            trackSender: tracksOfPlaylistModel.getTrackSender(index)
            trackMessage: tracksOfPlaylistModel.getTrackMessage(index)
            enableAddToQueue: true
            onClicked: timer.start()
            onStarClicked: tracksOfPlaylistPageController.setStarred(index)
            onSeenClicked: tracksOfPlaylistPageController.setSeen(index)
            onPressAndHold: {
                viewMenu.menuData = longTapMenuData
                artistOfLongTappedTrack = TrackArtist
                longTappedIndex = index;
                viewMenu.open(labelText1, TrackArtist, TrackAlbumArt)
            }
            onSwiped: tracksOfPlaylistPageController.handleAddToQueue(index)

            Timer {
                id: timer
                interval: 550; running: false; repeat: false
                onTriggered: myList.positionViewAtIndex(index, ListView.Contain)
            }
        }
    }
    Component {
        id: normal
        Components.ListItem {
            objectName: "tracksOfPlaylistItem"
            labelText1: TrackTitle
            labelText2: TrackSubHeader
            starred: TrackStarred
            listReady: TrackAvailable
            enableAddToQueue: true
            onClicked: {
                if (tracksOfPlaylistPageController.hasValidTrack(index)) {
                    tracksOfPlaylistPageController.listClicked(index)
                    goToPage("NowPlayingPage")
                } else {
                    notifyController.showMessage("Track is not available")
                }
            }
            onStarClicked: tracksOfPlaylistPageController.setStarred(index)
            onPressAndHold: {
                viewMenu.menuData = tracksOfPlaylistModel.playlistOwner == "" ? longTapMenuData :
                                    longTapMenuDataFriend
                artistOfLongTappedTrack = TrackArtist
                longTappedIndex = index;
                viewMenu.open(labelText1, TrackArtist, TrackAlbumArt)
            }
            onSwiped: tracksOfPlaylistPageController.handleAddToQueue(index)
        }
    }

    Component {
        id: saveToPlaylistContent
        Item {
            height: childrenRect.height
            width: dialogLoader.status ==  Loader.Ready ? dialogLoader.item.contentWidth : 0

            ListView {
                id: playlists
                objectName: "playlists"
                clip: true
                width: parent.width
                height: Math.max(150, Math.min(600, count * 80 + 80))
                model: editablePlaylistsModel
                header: headerComponent
                delegate: Components.ListItem {
                    inverted: true
                    labelText1: playlistName
                    showIcon: false
                    onClicked: {
                        tracksOfPlaylistPageController.saveToPlaylist(index, longTappedIndex)
                        rootWindow.hideDialog()
                    }
                }

                Component {
                    id: headerComponent
                    Components.ListItem {
                        inverted: true
                        labelText1: "<Add as a new playlist>"
                        labelText2: tracksOfPlaylistPageController.longTappedItem(longTappedIndex)
                        showIcon: false
                        onClicked: {
                            tracksOfPlaylistPageController.saveToPlaylist(-1, longTappedIndex)
                            rootWindow.hideDialog()
                        }
                    }
                }
            }
        }
    }

    Sheet {
        id: renameSheet
        z: 4
        visualParent: wrapper
        acceptButtonText: "Save"
        rejectButtonText: "Cancel"

        onAccepted: {
            console.log("onAccepted")
            if (mainContainer.newName != "") {
                tracksOfPlaylistPageController.renamePlaylist(mainContainer.newName)
                viewTitlePanel.headerText = mainContainer.newName
            }
        }

        content: Rectangle {
            id: mainContainer
            anchors.fill: parent
            color: "black"
            property string newName

            MouseArea {
                anchors.fill: mainContainer
            }

            // Title panel
            Components.ViewTitlePanel {
                id: renameSheetTitle
                anchors.top: parent.top
                height: 72
                headerText: "Rename Playlist"
            }

            TextField {
                id: text
                width: parent.width - 30
                anchors.left: parent.left
                anchors.leftMargin: 15
                anchors.top: renameSheetTitle.bottom
                anchors.topMargin: 10
                font.pixelSize: UI.FONT_SIZE_NORMAL
                text: tracksOfPlaylistModel.playlistName
                onTextChanged: mainContainer.newName = text.text
                Keys.onReturnPressed: {
                    if (mainContainer.newName != "") {
                        renameSheet.accept()
                    }
                }
            }
        }
    }
}
