import QtQuick 1.1
import com.meego 1.0
import "Components" as Components
import "Views"

import "Components/constants.js" as UI

Window {
    id: rootWindow
    color: UI.APP_BACKGROUND_COLOR

    property string viewsLocation: "Views/"
    property string startingPage: "PlaylistsPage"
    property string loginDialogVisible: loginDialog.state
    property real pageStackDepth: pageStack.depth
    property bool busy: pageStack.busy
    property int engineLoggedIn: 0
    property bool loadingLinkedPlaylist: false
    property int statusBarHeight: 0

    property string toolBarController: pageStack.currentPage.pageName
    property bool nPtoolBarController: nowPlayingPageController.Visible

    property bool loginIn:  false




    PageStack {
        id: pageStack
        anchors.top: statusBar.bottom
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.bottom: parent.bottom
    }

    Loader {
        id: npsheetloader
        sourceComponent: undefined
        anchors.fill: parent

        onLoaded: {
            z = item.z
            npsheetloader.item.visualParent = npsheetloader
            npsheetloader.item.open()
        }
    }

    // Now playing-Sheet
    Component {
        id: npsheet
        // For some reason this won't load from separate file
        Sheet {
            id: nowPlayingPage
            z:3

            property bool cancelDragEvent: nowPlayingPageController.DragCancelingNeeded
            property int npIndex: nowPlayingPageController.NowPlayingIndex
            property int artIndex: 0
            property int oldNpIndex: 0
            property int highlightMoveDuration: 400
            property int timeFromPreviousIndexChange: 0
            property int correctIndex: -1
            property bool previousButtonClicked: false
            property bool nextButtonClicked: false
            property bool albumArtInit: false

            acceptButtonText: "Hide"

            onNpIndexChanged: {
                // Update viewMenu
                if (viewMenu.visible && nowPlayingPage.visible) {
                    viewMenu.menuData = detailsMenu
                    viewMenu.menuData.model.set(0, {"action": nowPlayingPageController.NowPlayingArtist})
                    viewMenu.menuData.model.set(1, {"action": nowPlayingPageController.NowPlayingAlbum})
                    viewMenu.open(nowPlayingPageController.NowPlayingTrack, nowPlayingPageController.NowPlayingArtist, nowPlayingPageController.getCurrentAlbumArt())
                }

                timeIncreaser.start()
                if (timeFromPreviousIndexChange == 0) {
                    nowPlayingPage.highlightMoveDuration = 400
                } else {
                    nowPlayingPage.highlightMoveDuration = 80
                }

                if (oldNpIndex == 0 && npIndex == nowPlayingPageController.PlaylistTrackCount - 1 && albumArtInit) {
                    if (previousButtonClicked) {
                        indexSetter.start()
                    }
                    if (nextButtonClicked && nowPlayingPageController.PlaylistTrackCount == 2) {
                        artIndex = npIndex + 1
                        albumArtImage.playIndex = artIndex
                        albumArtImage.currentIndex = artIndex
                    } else {
                        correctIndex = npIndex
                        artIndex--
                        albumArtImage.playIndex = artIndex
                        albumArtImage.currentIndex = artIndex
                    }
                } else if (oldNpIndex == nowPlayingPageController.PlaylistTrackCount - 1 && npIndex == 0 && albumArtInit) {
                    if (nextButtonClicked) {
                        indexSetter.start()
                    }
                    if (previousButtonClicked && nowPlayingPageController.PlaylistTrackCount == 2) {
                        artIndex = npIndex + 1
                        albumArtImage.playIndex = artIndex
                        albumArtImage.currentIndex = artIndex
                    } else {
                        correctIndex = npIndex
                        artIndex++
                        albumArtImage.playIndex = artIndex
                        albumArtImage.currentIndex = artIndex
                    }
                } else {
                    artIndex = npIndex + 1
                    albumArtImage.playIndex = artIndex
                    albumArtImage.currentIndex = artIndex
                }
                previousButtonClicked = false
                nextButtonClicked = false

                if (currentTrackIndexText.state == "hidden") {
                    currentTrackIndexText.state = "visible"
                    timer.start()
                } else {
                    timer.restart()
                }
                oldNpIndex = npIndex
            }

            Timer {
                id: indexSetter
                interval: nowPlayingPage.highlightMoveDuration; running: false; repeat: false
                onTriggered: {
                    artIndex = correctIndex + 1
                    albumArtImage.playIndex = artIndex
                    albumArtImage.positionViewAtIndex(artIndex, ListView.Center)
                    correctIndex = -1
                }
            }

            Timer {
                id: timeIncreaser
                interval: 10; running: false; repeat: true
                onTriggered: {
                    timeFromPreviousIndexChange += 10
                    if (timeFromPreviousIndexChange > 500) {
                        timeFromPreviousIndexChange = 0
                        timeIncreaser.stop()
                    }
                }
            }

            onStatusChanged: {
                switch(status) {
                    case PageStatus.Activating: {
                            nowPlayingPageController.setVisible(true)
                            rootWindow.showNPSheet()
                        }
                    case PageStatus.Deactivating: {
                            nowPlayingPageController.setVisible(false)

                           // npToolbar.privateVisibility = true
                        if (toolBarController == "PlaylistsPage" ||
                                toolBarController == "SearchPage" ||
                                toolBarController == "DiscoverPage") {
                            toolbar.privateVisibility = false
                        } else {
                            albumAndArtistPageToolBar.privateVisibility = false
                        }
                        }
                }
            }

            onCancelDragEventChanged: {
                progressBar.tabPressed = false
                progressBar.disableDragTarget()
            }

            Timer {
                id: timer
                interval: 3000; running: false; repeat: false
                onTriggered: {
                    currentTrackIndexText.state = "hidden"
                }
            }

            content: Rectangle {
                id: mainContainer
                anchors.fill: parent
                radius: 20
                color: "#232323"
                // Album art
                ListView {
                    id: albumArtImage
                    property bool songChangeAllowed: false
                    property bool flickStarted: false
                    property int playIndex: -1

                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.top: parent.top
                    height: parent.width
//                    anchors.bottom: parent.bottom
                    model: nowPlayingAlbumArtModel
                    orientation: ListView.Horizontal
                    snapMode: ListView.SnapOneItem
                    clip: true
                    highlightRangeMode: ListView.StrictlyEnforceRange
                    highlightMoveDuration: nowPlayingPage.highlightMoveDuration
                    boundsBehavior: Flickable.StopAtBounds
                    interactive: nowPlayingPageController.PlaylistTrackCount > 1

/*                    Components.LoadingSpinner {
                        id: albumArtLoadingSpinner
                        invertedSpinner: true
                        loadingText: "Loading album art..."
                        anchors.top: parent.top
                        anchors.left: parent.left
                        anchors.right: parent.right
                        anchors.margins: 20
                        height: albumArtImage.currentItem != null ? albumArtImage.currentItem.height : 0
                        visible: !nowPlayingPageController.LocalPlayback &&
                                 albumArtImage.currentItem != null &&
                                 !albumArtImage.currentItem.imageReady ? true : false
                    }
*/
                    delegate:
                    Item {
                        id: nowPlayingPageAlbumArt
                        width: nowPlayingPage.width
                        height: width
                        property bool imageReady: false
                        state: imageReady ? "visible" : "hidden"

                        Image {
                            id: albumArt
                            anchors.top: parent.top
                            width: parent.width - 40
                            height: width
                            source: TrackAlbumArt != "" ? TrackAlbumArt : "/usr/share/spotify/pics/def_pics-album_448.png"
                            anchors.centerIn: parent
                            smooth: true
                            onStatusChanged: {
                                if (status == Image.Ready) {
                                    nowPlayingPageAlbumArt.imageReady = true
                                } else if (status == Image.Error) {
                                    nowPlayingPageAlbumArt.imageReady = true
                                    console.log("\n\n\n IMAGE NOT FOUND FOR NP PUT DEFAULT\n\n\n")
                                }
                            }
                        }
            /*            Image {
                            id: mirroredAlbumArt
                            anchors.top: albumArt.bottom
                            anchors.horizontalCenter: albumArt.horizontalCenter
                            anchors.topMargin: 2
                            opacity: 0.9
                            source: TrackAlbumArt
                            width: parent.width - 40
                            height: width
                            smooth: true
                            transform: [
                                Rotation { origin.x: mirroredAlbumArt.width/2; origin.y: mirroredAlbumArt.height/2; axis { x: 1; y: 0; z: 0 } angle: 180 },
                                Rotation { origin.x: mirroredAlbumArt.width/2; origin.y: 0; axis { x: 1; y: 0; z: 0 } angle: 60 }
                            ]
                        }
                        Rectangle {
                            id: mirrorAlbumArtShader
                            anchors.fill: mirroredAlbumArt
                            transform: [
                                Rotation { origin.x: mirrorAlbumArtShader.width/2; origin.y: mirrorAlbumArtShader.height/2; axis { x: 0; y: 0; z: 1; } angle: 0; },
                                Rotation { origin.x: mirroredAlbumArt.width/2; origin.y: 0; axis { x: 1; y: 0; z: 0 } angle: 60 }
                            ]
                            gradient: Gradient {
                                GradientStop { position: 0.0; color: "transparent" }
                                GradientStop { position: 0.6; color: "#393536" }
                            }
                        }
            */            states: [
                            State {
                                name: "visible"
                                PropertyChanges { target: albumArt; opacity: 1 }
            //                    PropertyChanges { target: mirroredAlbumArt; opacity: 1 }
                            },
                            State {
                                name: "hidden"
                                PropertyChanges { target: albumArt; opacity: 0 }
            //                    PropertyChanges { target: mirroredAlbumArt; opacity: 0 }
                            }
                        ]
                        transitions: [
                            Transition {
                                from: "hidden"; to: "visible"
                                NumberAnimation { target: albumArt; properties: "opacity"; duration: 400; easing.type: Easing.InCubic }
            //                    NumberAnimation { target: mirroredAlbumArt; properties: "opacity"; duration: 400; easing.type: Easing.InCubic }
                            }
                        ]
                    }
                    onModelChanged: {
                        songChangeAllowed = false
                        artIndex = npIndex + 1
                        currentIndex = artIndex
                        playIndex = artIndex
                        positionViewAtIndex(artIndex, ListView.Center)
                        albumArtInit = true
                        songChangeAllowed = true
                    }
                    onCurrentIndexChanged: {
                        if (currentItem != null) {
                            currentItem.z = 2
                        }

                        if (songChangeAllowed) {
                            artIndex = currentIndex
                            if (currentIndex < playIndex - 1) {
                                currentIndex = playIndex - 1
                            } else if (currentIndex > playIndex + 1) {
                                currentIndex = playIndex + 1
                            }
                        }
                    }
                    onMovementStarted: playIndex = currentIndex
                    onMovementEnded: {
                        if (playIndex != artIndex && !flickStarted) {
                            if (playIndex > artIndex && songChangeAllowed) {
                                nowPlayingPageController.previous(true)
                                playIndex = artIndex
                                currentIndex = artIndex
                            } else if (playIndex < artIndex && songChangeAllowed) {
                                nowPlayingPageController.next()
                                playIndex = artIndex
                                currentIndex = artIndex
                            }
                            if (correctIndex != -1) {
                                artIndex = correctIndex + 1
                                playIndex = artIndex
                                currentIndex = artIndex
                                albumArtImage.positionViewAtIndex(artIndex, ListView.Center)
                                correctIndex = -1
                            }
                        }
                    }

                    Timer {
                        id: flickEndedTimer
                        interval: 10; running: false; repeat: false
                        onTriggered: {
                            if (!albumArtImage.moving) {
                                albumArtImage.flickStarted = false
                            } else {
                                flickEndedTimer.start()
                            }
                        }
                    }

                    onFlickStarted: {
                        if (!moving) {
                            flickStarted = true
                        }
                    }
                    onFlickEnded: {
                        flickEndedTimer.start()
                        if (playIndex > artIndex && songChangeAllowed) {
                            nowPlayingPageController.previous(true)
                            playIndex = artIndex
                            currentIndex = artIndex
                        } else if (playIndex < artIndex && songChangeAllowed) {
                            nowPlayingPageController.next()
                            playIndex = artIndex
                            currentIndex = artIndex
                        }
                        if (correctIndex != -1) {
                            artIndex = correctIndex + 1
                            playIndex = artIndex
                            currentIndex = artIndex
                            albumArtImage.positionViewAtIndex(artIndex, ListView.Center)
                            correctIndex = -1
                        }
                    }
                }

                // Progress bar
                Components.Progress {
                    id: progressBar
                    objectName: "progressBar"
                    progress: nowPlayingPageController.NowPlayingProgressBarPosition
                    playedTime: nowPlayingPageController.NowPlayingProgressBarPlayedTime
                    duration: nowPlayingPageController.NowPlayingProgressBarSongDuration
                    anchors.left: albumArtImage.left
                    anchors.leftMargin: 25
                    anchors.right: albumArtImage.right
                    anchors.rightMargin: 25
                    anchors.top: albumArtImage.bottom
                    anchors.topMargin: 5
                    onSetPosition: nowPlayingPageController.seek(progressBar.songPosition)
                }

                // Buttons
                Item {
                    id: buttonPanel
                    objectName: "buttonPanel"
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.top: progressBar.bottom
                    anchors.topMargin: 10
                    height: 75
                    Components.Button {
                        id: button1
                        objectName: "previous"
                        enabled: nowPlayingPageController.PlaylistTrackCount > 1 || nowPlayingPageController.Repeat ? true : false
                        imagePath: enabled ? "/usr/share/spotify/pics/pb_control-prev.png" : "/usr/share/spotify/pics/pb_control-prev-dimmed.png"
                        anchors.left: buttonPanel.left
                        width: parent.width / 3
                        imageSize: 48
                        onClicked: {
                            if (!indexSetter.running) {
                                previousButtonClicked = true
                                nowPlayingPageController.previous()
                            }
                        }
                    }

                    Components.Button {
                        id: button2
                        objectName: "pause"
                        imagePath: {
                            if (nowPlayingPageController.PlayingState) {
                                return "/usr/share/spotify/pics/pb_control-pause.png"
                            } else {
                                return "/usr/share/spotify/pics/pb_control-play.png"
                            }
                        }
                        imageSize: 48
                        anchors.horizontalCenter: buttonPanel.horizontalCenter
                        width: parent.width / 3
                        onClicked: nowPlayingPageController.pauseOrResume()
                    }

                    Components.Button {
                        id: button3
                        objectName: "next"
                        enabled: nowPlayingPageController.PlaylistTrackCount > 1 || nowPlayingPageController.Repeat ? true : false
                        imagePath: enabled ? "/usr/share/spotify/pics/pb_control-next.png" : "/usr/share/spotify/pics/pb_control-next-dimmed.png"
                        anchors.right: buttonPanel.right
                        width: parent.width / 3
                        imageSize: 48
                        onClicked: {
                            if (!indexSetter.running) {
                                nextButtonClicked = true
                                nowPlayingPageController.next()
                            }
                        }
                    }
                }

                // Track title
                Text {
                    id: songText
                    objectName: "songText"
                    anchors.top: buttonPanel.bottom; anchors.left: parent.left; anchors.right: parent.right
                    anchors.topMargin: 10; anchors.leftMargin: 10; anchors.rightMargin: 10
                    text: nowPlayingPageController.NowPlayingTrack
                    color: "white"
                    font.pixelSize: 32
                    font.bold: true
                    elide: Text.ElideRight
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignBottom
                }

                // Artist text
                Text {
                    id: artistText
                    objectName: "artistText"
                    anchors.top: songText.bottom; anchors.left: parent.left; anchors.right: parent.right
                    anchors.leftMargin: 10; anchors.rightMargin: 10
                    text: "by " + nowPlayingPageController.NowPlayingArtist
                    color: "white"
                    font.pixelSize: UI.FONT_SIZE_SMALL
                    font.family: UI.FONT_FAMILY
                    elide: Text.ElideRight
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignBottom
                }

                // Album text
                Text {
                    id: albumText
                    objectName: "albumText"
                    anchors.top: artistText.bottom; anchors.left: parent.left; anchors.right: parent.right
                    anchors.topMargin: 15; anchors.leftMargin: 10; anchors.rightMargin: 10
                    text: nowPlayingPageController.NowPlayingAlbum
                    color: "grey"
                    font.pixelSize: UI.FONT_SIZE_SMALL
                    elide: Text.ElideRight
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignBottom
                }

                // Show info area
                MouseArea {
                    id: mouseRegion
                    objectName: "mouseRegion"
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.top: songText.top
                    anchors.bottom: albumText.bottom
                    onClicked:
                        {
                            viewMenu.menuData = detailsMenu
                            viewMenu.menuData.model.set(0, {"action": nowPlayingPageController.NowPlayingArtist})
                            viewMenu.menuData.model.set(1, {"action": nowPlayingPageController.NowPlayingAlbum})

                            viewMenu.open(nowPlayingPageController.NowPlayingTrack, nowPlayingPageController.NowPlayingArtist, nowPlayingPageController.getCurrentAlbumArt())
                        }
                    enabled: !nowPlayingPageController.LocalPlayback
                }

                Rectangle {
                    id: line
                    height: 2
                    color: "black"
                    anchors.bottom: npToolbar.top
                    anchors.topMargin: 15
                    anchors.left: parent.left
                    anchors.leftMargin: 15
                    anchors.right: parent.right
                    anchors.rightMargin: 15
                }
                Rectangle {
                    id: npToolbar
                    visible: true
                    radius: 20
                    height: 68
                    z: parent.z
                    color:  "#232323"
                    anchors.bottom: parent.bottom
                    anchors.left: parent.left
                    anchors.right: parent.right
                    Item {
                        id: npAddToPlaylistBtItem
                        anchors.left:  npToolbar.left
                        anchors.verticalCenter: parent.verticalCenter
                        visible: !nowPlayingPageController.LocalPlayback
                        width: parent.width / 5
                        height: parent.height
                        Image {
                            id: npAddToPlaylistBt
                            anchors.verticalCenter: parent.verticalCenter
                            anchors.horizontalCenter: parent.horizontalCenter
                            enabled: !shareMenu.visible
                            source:"/usr/share/spotify/pics/Spotify-17.png"
                        }
                        MouseArea {
                            objectName: "npAddToPlaylistBtMouseArea"
                            anchors.fill: parent
                            onClicked: {
                                rootWindow.showDialog("Save to playlist", saveToPlaylistContent)
                            }
                        }
                    }
                    Item {
                        id:npShareBtnItem
                        anchors.left: npAddToPlaylistBtItem.right
                        anchors.verticalCenter: parent.verticalCenter
                        visible: !nowPlayingPageController.LocalPlayback
                        width: parent.width / 5
                        height: parent.height
                        Image {
                            id: npShareBtn
                            anchors.verticalCenter: parent.verticalCenter
                            anchors.horizontalCenter: parent.horizontalCenter
                            source: enabled ? "/usr/share/spotify/pics/Spotify-11.png" :
                            "/usr/share/spotify/pics/Spotify-11-dimmed.png"
                         }
                        MouseArea {
                            objectName: "npShareBtnMouseArea"
                            anchors.fill: parent
                            onClicked: {
                                { shareMenu.visible ? shareMenu.close() : shareMenu.open(2) }
                            }
                        }
                    }
                    Item {
                        id:npFavouriteBtnItem
                        anchors.left: npShareBtnItem.right
                        anchors.verticalCenter: parent.verticalCenter
                        visible: !nowPlayingPageController.LocalPlayback
                        width: parent.width / 5
                        height: parent.height
                        Image {
                            id: npFavouriteBtn
                            anchors.verticalCenter: parent.verticalCenter
                            anchors.horizontalCenter: parent.horizontalCenter
                            enabled: !shareMenu.visible
                            source:nowPlayingPageController.Starred ? "/usr/share/spotify/pics/Spotify-15.png" :
                            "/usr/share/spotify/pics/Spotify-13.png"
                        }
                        MouseArea {
                            objectName: "npFavouriteBtnMouseArea"
                            anchors.fill: parent
                            onClicked: {
                                nowPlayingPageController.setStarred()
                            }
                        }
                    }
                    Item {
                        id:npRandomBtnItem
                        anchors.left: npFavouriteBtnItem.right
                        anchors.verticalCenter: parent.verticalCenter
                        visible: !nowPlayingPageController.LocalPlayback
                        width: parent.width / 5
                        height: parent.height
                        Image {
                            id: npRandomBtn
                            anchors.verticalCenter: parent.verticalCenter
                            anchors.horizontalCenter: parent.horizontalCenter
                            enabled: nowPlayingPageController.PlaylistTrackCount > 2
                            source:nowPlayingPageController.Shuffled ? "/usr/share/spotify/pics/Spotify-16.png" :
                            "/usr/share/spotify/pics/Spotify-08.png"
                         }
                        MouseArea {
                            objectName: "npFavouriteBtnMouseArea"
                            anchors.fill: parent
                            onClicked: {
                                if (nowPlayingPageController.PlaylistTrackCount > 2) {
                                    nowPlayingPageController.shuffle()
                                }
                            }
                        }
                    }
                    Item {
                        id:npRepeatBtnItem
                        anchors.left: npRandomBtnItem.right
                        anchors.verticalCenter: parent.verticalCenter
                        width: parent.width / 5
                        height: parent.height
                        Image {
                            id: npRepeatBtn
                            anchors.verticalCenter: parent.verticalCenter
                            anchors.horizontalCenter: parent.horizontalCenter
                            enabled: !shareMenu.visible
                            source:nowPlayingPageController.Repeat ? "/usr/share/spotify/pics/Spotify-22.png" :
                            "/usr/share/spotify/pics/Spotify-14.png"
                         }
                        MouseArea {
                            objectName: "npFavouriteBtnMouseArea"
                            anchors.fill: parent
                            onClicked: {
                                nowPlayingPageController.changeRepeatMode()
                            }
                        }
                    }
                }

                // Current track index
                Text {
                    id: currentTrackIndexText
                    anchors.top: albumArtImage.top
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.topMargin: 40
                    height: 100
                    width: 300
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignVCenter
                    text: nowPlayingPageController.CurrentPositionInPlaylist
                    color: "white"
                    font.pixelSize: 60
                    font.family: UI.FONT_FAMILY
                    font.bold: true
                    style: Text.Outline; styleColor: "#393536"
                    z: 2
                    state: "hidden"

                    states: [
                        State {
                            name: "visible"
                            PropertyChanges { target: currentTrackIndexText; opacity: 1 }
                        },
                        State {
                            name: "hidden"
                            PropertyChanges { target: currentTrackIndexText; opacity: 0 }
                        }
                    ]

                    transitions: [
                        Transition {
                            from: "visible"; to: "hidden"
                            NumberAnimation { target: currentTrackIndexText; properties: "opacity"; duration: 1500; easing.type: Easing.OutCubic }
                        },
                        Transition {
                            from: "hidden"; to: "visible"
                            NumberAnimation { target: currentTrackIndexText; properties: "opacity"; duration: 500; easing.type: Easing.InCubic }
                        }
                    ]
                }

                // This prevents user to pan more than 1 index per time.
                MouseArea {
                    anchors.fill: albumArtImage
                    visible: albumArtImage.moving
                }

                // Info menu
                Components.ContextMenuData {
                    id: detailsMenu
                    model: ListModel {
                        ListElement {
                            action: ""
                            subText: ""
                            hasCheckBox: false
                            actionOn: false
                        }
                        ListElement {
                            action: ""
                            subText: ""
                            hasCheckBox: false
                            actionOn: false
                        }
                        ListElement {
                            action: "See related Artists"
                            subText: ""
                            hasCheckBox: false
                            actionOn: false
                        }
                    }
                    onItemActivated: {
                        switch(model.get(index).action)
                        {
                            case nowPlayingPageController.NowPlayingArtist: {
                                if (nowPlayingPageController.artistDataReady()) {
                                    nowPlayingPage.close();
                                    artistPageController.clearData()
                                    goToPage("ArtistPage")
                                    artistPageController.setTab("overview")
                                    nowPlayingPageController.getArtistData()
                                }
                            }
                            case nowPlayingPageController.NowPlayingAlbum: {
                                if (nowPlayingPageController.albumDataReady()) {
                                   nowPlayingPage.close();
                                   albumPageController.resetAlbumPage()
                                   goToPage("AlbumPage")
                                   nowPlayingPageController.getAlbumData()
                               }
                            }
                            case "See related Artists": {
                                if (nowPlayingPageController.artistDataReady()) {
                                   nowPlayingPage.close();
                                   artistPageController.clearData()
                                   goToPage("ArtistPage")
                                   artistPageController.setTab("related")
                                   nowPlayingPageController.getArtistData()
                               }
                            }
                        }
                    }
                }
            }
        }
    }

    function showNPSheet() {
        if (npsheetloader.item) {
            rootWindow.z = npsheetloader.z -1
            npsheetloader.item.open()
            return;
        }

       // toolbar.anchors.bottomMargin = toolbar.height
        npsheetloader.sourceComponent = npsheet// = "Views/NowPlayingPage.qml"
    }

    function hideNPSheet() {
        npsheetloader.item.close()
    }

    Components.ViewMenu {
        id: viewMenu
        visible: false
        z: 4
        anchors.top: parent.top
        anchors.bottom: parent.bottom
    }


    Components.LongTapMenu {
        id: longTapMenu
        visible: false
        z: 4
        anchors.top: parent.top
        anchors.bottom: parent.bottom
    }

    Components.ShareMenu {
        id: shareMenu
        visible: false
        z: 4
        anchors.top: parent.top
        anchors.bottom: parent.bottom
        onCloseNPPage: {
            rootWindow.hideNPSheet()
        }
    }

    StatusBar {
        id: statusBar
        z: 2
        anchors.top: parent.top
        Rectangle {
            visible: playlistsPageController.syncing
            width: parent.width -60
            height: parent.height
            color: "black"
            Components.ListItemProgress {
                id: progressBar
                height: 20
                visible: true
                objectName: "progressBar"
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.verticalCenter: parent.verticalCenter
                progress: playlistsPageController.syncCopiedCount /
                        playlistsPageController.syncTotalCount
            }
            Text {
                anchors {
                    left: parent.left
                    right: parent.right
                    verticalCenter: parent.verticalCenter
                    leftMargin: 10
                }
                text: "Syncing " + (playlistsPageController.syncCopiedCount +1) +
                        " of " + playlistsPageController.syncTotalCount
                elide: Text.ElideRight
                font.pixelSize: UI.FONT_SIZE_NORMAL
                font.family: UI.FONT_FAMILY
                color: "white"
            }
        }
    }

    Components.Banner {
        id: banner
        z: 4
    }

    Loader {
        id: dialogLoader
        anchors.fill: parent
        z: 3
    }

    // Save as playlist dialog
    Component {
        id: saveToPlaylistContent
        Item {
            height: childrenRect.height
            width: dialogLoader.status ==  Loader.Ready ? dialogLoader.item.contentWidth : 0

            ListView {
                id: playlists
                objectName: "playlists"
                clip: true
                width: parent.width
                height: Math.max(150, Math.min(600, count * 80 + 80))
                model: editablePlaylistsModel
                header: headerComponent
                delegate: Components.ListItem {
                    inverted: true
                    labelText1: playlistName
                    showIcon: false
                    onClicked: { if (nPtoolBarController || toolBarController == "PlaylistsPage") {
                                    nowPlayingPageController.saveToPlaylist(index)
                                    rootWindow.hideDialog()
                                } else {
                                    albumPageController.handleAddAlbumToPlaylist(index)
                                    rootWindow.hideDialog()
                                }
                    }
                }

                Component {
                    id: headerComponent
                    Components.ListItem {
                        inverted: true
                        labelText1: "<Add as a new playlist>"
                        labelText2: (nPtoolBarController || toolBarController == "PlaylistsPage") ? nowPlayingPageController.NowPlayingTrack :
                                albumPageController.AlbumName
                        showIcon: false
                        onClicked: {
                            (nPtoolBarController || toolBarController == "PlaylistsPage") ?
                                    nowPlayingPageController.saveToPlaylist(-1) :
                                    albumPageController.saveToPlaylist(-1)
                            rootWindow.hideDialog()
                        }
                    }
                }
            }
        }
    }

    Components.ContextMenuData {
        id: addMenuForAlbumPage
        model: ListModel {
            ListElement {
                action: "Add Album To Playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Add Album To Play Queue"
                hasCheckBox: false
                actionOn: false
            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Add Album To Playlist": {
                    rootWindow.showDialog("Save to playlist", saveToPlaylistContent)
                }
                case "Add Album To Play Queue": {
                    albumPageController.handleAddAlbumToQueue()
                }
            }
        }
    }

    // ************ Handler ************
    onToolBarControllerChanged: {
        if (toolBarController == "PlaylistsPage" ||
            toolBarController == "SearchPage" ||
            toolBarController == "DiscoverPage" ||
            toolBarController == "SettingsPage") {
            albumAndArtistPageToolBar.privateVisibility = true
            if (nPtoolBarController) {
              toolbar.privateVisibility = true
             } else {
              toolbar.privateVisibility = false
             }
        } else {
            toolbar.privateVisibility = true
            albumAndArtistPageToolBar.privateVisibility = false
            toolbar.privateVisibility = true
            albumAndArtistShareBtn.visible = true
            if (toolBarController == "AlbumPage") {
                albumAndArtistEditBtn.visible = false
                albumAndArtistaddBtn.visible = true
            } else if (toolBarController == "FriendListPage" ||
                       toolBarController == "ProfilePage" ||
                       toolBarController == "PlayQueuePage" ||
                       toolBarController == "LocalSongsPage" ||
                       toolBarController == "SpotifySharePage" ||
                       nowPlayingPageController.LocalPlayback) {
                        albumAndArtistEditBtn.visible = false
                        albumAndArtistShareBtn.visible = false
                        albumAndArtistaddBtn.visible = false
            }else if (toolBarController == "ArtistPage" ||
                       (tracksOfPlaylistModel.playlistOwner != "" &&
                       tracksOfPlaylistModel.playlistName != "Starred")) {
               albumAndArtistEditBtn.visible = false
               albumAndArtistaddBtn.visible = false
            }
            // Starred, Inbox
            else if (toolBarController == "TracksOfPlaylistPage" &&
                     (tracksOfPlaylistModel.playlistName == "Starred" ||
                     tracksOfPlaylistModel.playlistName == "Inbox"))
            {
                albumAndArtistEditBtn.visible = false
                albumAndArtistaddBtn.visible = false
            }
            else if (toolBarController == "TracksOfPlaylistPage") {
                albumAndArtistEditBtn.visible = true
                albumAndArtistaddBtn.visible = false
            }
       }
   }

    // ************ Handler ************

    // Toolbar for album- and artistPage
    ToolBar {
        id: albumAndArtistPageToolBar
        z: 2
        platformStyle: ToolBarStyle { inverted: false; background: "/usr/share/spotify/pics/toolbar_background.png"; visibilityTransitionDuration: 200 }
        anchors.bottom: parent.bottom
        anchors.bottomMargin: inputContext.softwareInputPanelVisible == true ? -albumAndArtistPageToolBar.height : 0
        tools: ToolBarLayout {
            ToolIcon {
                id: albumAndArtistBackBtn
                objectName: "albumAndArtistBackBtn"
                iconSource:"/usr/share/spotify/pics/icon-m-toolbar-back-white.png"
                onClicked: goToPage("goBack")
            }
            ToolIcon {
                id: albumAndArtistEditBtn
                objectName: "albumAndArtistEditBtn"
                iconSource:"/usr/share/spotify/pics/Spotify-09.png"
                onClicked:{
                   viewMenu.open("", "", "")
                }
            }

            ToolIcon {
                id: albumAndArtistaddBtn
                objectName: "albumAndArtistaddBtn"
                iconSource:"/usr/share/spotify/pics/Spotify-17.png"
                onClicked:{
                    viewMenu.menuData = addMenuForAlbumPage
                    viewMenu.open("", "", "")
                }
            }

            ToolIcon {
                id: albumAndArtistShareBtn
                objectName: "albumAndArtistShareBtn"
                enabled: pageStack.currentPage.pageName == "AlbumPage" ?
                        !albumPageController.LoadingAlbumData :
                        pageStack.currentPage.pageName == "ArtistPage" ?
                        !artistPageController.LoadingArtistData : true
                iconSource: enabled ?  "/usr/share/spotify/pics/Spotify-11.png" :
                        "/usr/share/spotify/pics/Spotify-11-dimmed.png"
                onClicked: if (shareMenu.visible) {
                    shareMenu.close()
                } else if (enabled) {
                    if (pageStack.currentPage.pageName == "AlbumPage") {
                        shareMenu.open(1)
                    } else if (pageStack.currentPage.pageName == "ArtistPage") {
                        artistPageController.shareArtist()
                    } else if (pageStack.currentPage.pageName == "TracksOfPlaylistPage") {
                        tracksOfPlaylistPageController.sharePlaylistToOthers()
                    }
                }
            }
        }
    }

    //Main Views Toolbar
    ToolBar {
        id: toolbar
        z: 2
        platformStyle: ToolBarStyle { inverted:false; background: "/usr/share/spotify/pics/maintoolbar_1.png"; visibilityTransitionDuration: 200 }
        anchors.bottom: parent.bottom
        anchors.bottomMargin: inputContext.softwareInputPanelVisible == true ? -toolbar.height : 0
        tools: ToolBarLayout {

            Item {
                id: playListsRect
                width: parent.width / 4
                height: toolbar.height
                MouseArea {
                    id: playListsRectMouse
                    objectName: "playListsRectMouse"
                    anchors.fill: parent
                    onClicked: {
                        if (pageStack.currentPage.pageName != "PlaylistPage" && !rootWindow.busy) {

                            switchView("PlaylistsPage")
                        }
                    }
                }
                Image {
                    id: playListsBtn
                    anchors.top: playListsRect.top
                    anchors.topMargin: 7
                    anchors.horizontalCenter: parent.horizontalCenter
                    source: "/usr/share/spotify/pics/tab_bar_playlists.png"
                }
                Text {
                    id: playListsText
                    z: 1
                    anchors.bottom: playListsRect.bottom
                    anchors.bottomMargin: 7
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.leftMargin: 15
                    text: "Playlists"
                    color: "white"
                    font.pixelSize: 16
                    font.family: UI.FONT_FAMILY
                }
            }
            Item {
                id: searchRect
                width: parent.width / 4
                height: toolbar.height
                anchors.left: playListsRect.right
                MouseArea {
                    id: searchRectMouse
                    objectName: "searchRectMouse"
                    anchors.fill: parent
                    onClicked: {
                        if (pageStack.currentPage.pageName != "SearchPage" && !rootWindow.busy) {

                            switchView("SearchPage")
                        }
                    }
                }
                Image {
                    id: searchBtn
                    anchors.top: searchRect.top
                    anchors.topMargin: 7
                    anchors.horizontalCenter: parent.horizontalCenter
                    source: "/usr/share/spotify/pics/tab_bar_search.png"
                }

                Text {
                    id: searchText
                    anchors.bottom: searchRect.bottom
                    anchors.bottomMargin: 7
                    anchors.horizontalCenter: parent.horizontalCenter
                    text: "Search"
                    color: "grey"
                    font.pixelSize: 16
                }
            }
            Item {
                id: discoverRect
                width: parent.width / 4
                height: toolbar.height
                anchors.left: searchRect.right
                MouseArea {
                    id: discoverRectMouse
                    objectName: "discoverRectMouse"
                    anchors.fill: parent
                    onClicked: {
                        if (pageStack.currentPage.pageName != "DiscoverPage" && !rootWindow.busy) {

                            switchView("DiscoverPage")
                        }
                    }
                }
                Image {
                    id: discoverBtn
                    anchors.top: discoverRect.top
                    anchors.topMargin: 7
                    anchors.horizontalCenter: parent.horizontalCenter
                    source: "/usr/share/spotify/pics/Spotify-27.png"


                }
                Text {
                    id: discoverText
                    anchors.bottom: discoverRect.bottom
                    anchors.bottomMargin: 7
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.rightMargin: 15
                    text: "What's New"
                    color: "grey"
                    font.pixelSize: 16
                }

            }
            Item {
                id: settingsRect
                width: parent.width / 4
                height: toolbar.height
                anchors.left: discoverRect.right
                MouseArea {
                    id: settingsRectMouse
                    objectName: "settingsRect"
                    anchors.fill: parent
                    onClicked: {
                        if (pageStack.currentPage.pageName != "SettingsPage" && !rootWindow.busy) {

                            switchView("SettingsPage")
                        }
                    }
                }
                Image {
                    id: settingsBtn
                    anchors.top: settingsRect.top
                    anchors.topMargin: 7
                    anchors.horizontalCenter: parent.horizontalCenter
                    source: "/usr/share/spotify/pics/tab_bar_settings.png"


                }
                Text {
                    id: settingsText
                    anchors.bottom: settingsRect.bottom
                    anchors.bottomMargin: 7
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.rightMargin: 15
                    text: "Settings"
                    color: "grey"
                    font.pixelSize: 16
                }
            }
            Item {
                id: allowClick
                visible: true
                anchors.fill: parent
                MouseArea {
                    anchors.fill: parent
                    onClicked: console.log("Wait")

                }
            }
        }
    }

    Components.LoadingSpinner {
        id: loadingSpinner
        z: 1
        visible: loadingLinkedPlaylist
        loadingText: "Loading playlist..."
        invertedSpinner: true
        anchors.fill:  parent
    }

    Component.onCompleted: {
        pageStack.push(Qt.createComponent(viewsLocation + startingPage + ".qml"))
        theme.inverted = true
        loginTimer.start()
    }

    Timer {
        id: loginTimer
        interval: 250; repeat: false; running: false;
        onTriggered: {
            if (loginDialog.username != "" && loginDialog.password != "" && settings.autoLogin) {
                // Do autologin
                engine.createSession();
                engine.login(loginDialog.username, loginDialog.password)
            } else {
                // Show Login Dialog
                loginDialog.state = "visible";
            }
        }
    }

    Connections {
        target: engine
        onLoggedStatusChanged: {
            engineLoggedIn = engineLoginStatus
            if (engineLoggedIn == 0) {
                // LoggedOut
                console.log("Logged out")
                if (pageStack.currentPage.pageName != startingPage) {
                    switchView(startingPage)
                }
                loginDialog.state = "visible";
            } else if (engineLoggedIn == 1) {
                // LoggedIn
                loginDialog.state = "hidden";
                allowClick.visible = false
                loginIn = true
                console.log("Logged in")
            } else if (engineLoggedIn == 2) {
                // LoggingIn
                loginDialog.state = "loggingIn";
            } else if (engineLoggedIn == 3) {
                // LoggingOut
                loginDialog.state = "loggingOut";
            } else {}
        }
    }

    Components.LoginDialog {
        id: loginDialog
        z: 3
        button1Label: "Login"
        state: "hidden"
        username: settings.username
        password: settings.password
        tos: settings.tos

        onLogIn: {
            engine.createSession();
            engine.login(loginDialog.username, loginDialog.password)
        }
    }

    function findPlaylistsPage(page) {
        return page.pageName == "PlaylistsPage"
    }
    function findSearchPage(page) {
        return page.pageName == "SearchPage"
    }
    function findDiscoverPage(page) {
        return page.pageName == "DiscoverPage"
    }
    function findAlbumPage(page) {
        return page.pageName == "AlbumPage"
    }
    function findArtistPage(page) {
        return page.pageName == "ArtistPage"
    }
    function findNowPlayingPage(page) {
        return page.pageName == "NowPlayingPage"
    }
    function findSettingsPage(page) {
        return page.pageName == "SettingsPage"
    }
    //Switch views and amend the navigation history
    function switchView(viewName) {
        if (pageStack.currentPage.pageName == viewName && viewName != "goBack" || busy) {
            return
        }

        if (viewName == "goBack") {
            pageStack.pop()
        } else if (viewName == "PlaylistsPage") {
            pageStack.pop(pageStack.find(findPlaylistsPage))
            toolbar.platformStyle.background = "/usr/share/spotify/pics/maintoolbar_1.png"
            playListsText.color = "white"
            searchText.color = "grey"
            discoverText.color = "grey"
            settingsText.color = "grey"
        } else if (viewName == "SearchPage") {
            if (pageStack.find(findSearchPage) == null) {
                pageStack.push(Qt.createComponent(viewsLocation + viewName + ".qml"))
            } else {
                pageStack.pop(pageStack.find(findSearchPage))
            }
            toolbar.platformStyle.background = "/usr/share/spotify/pics/maintoolbar_2.png"
            playListsText.color = "grey"
            searchText.color = "white"
            discoverText.color = "grey"
            settingsText.color = "grey"
        } else if (viewName == "DiscoverPage") {
            if (pageStack.find(findDiscoverPage) == null) {
                pageStack.push(Qt.createComponent(viewsLocation + viewName + ".qml"))
            } else {
                pageStack.pop(pageStack.find(findDiscoverPage))
            }
            toolbar.platformStyle.background = "/usr/share/spotify/pics/maintoolbar_3.png"
            playListsText.color = "grey"
            searchText.color = "grey"
            discoverText.color = "white"
            settingsText.color = "grey"
        } else if (viewName == "SettingsPage") {
            if (pageStack.find(findSettingsPage) == null) {
                pageStack.push(Qt.createComponent(viewsLocation + viewName + ".qml"))
            } else {
                pageStack.pop(pageStack.find(findSettingsPage))
            }
            toolbar.platformStyle.background = "/usr/share/spotify/pics/maintoolbar_4.png"
            playListsText.color = "grey"
            searchText.color = "grey"
            discoverText.color = "grey"
            settingsText.color = "white"
        } else if (viewName == "AlbumPage") {
            if (pageStack.find(findAlbumPage) == null) {
                pageStack.push(Qt.createComponent(viewsLocation + viewName + ".qml"))
            } else {
                pageStack.pop(pageStack.find(findAlbumPage))
            }
        } else if (viewName == "ArtistPage") {
            if (pageStack.find(findArtistPage) == null) {
                pageStack.push(Qt.createComponent(viewsLocation + viewName + ".qml"))
            } else {
                pageStack.pop(pageStack.find(findArtistPage))
            }
        } else if (viewName == "NowPlayingPage") {
            showNPSheet()
        } else {
            pageStack.push(Qt.createComponent(viewsLocation + viewName + ".qml"))

        }
    }

    function linkedPlaylistReady(ready) {
        if (ready) {
            loadingLinkedPlaylist = false
        } else {
            loadingLinkedPlaylist = true
        }
    }

    function showDialog(title, content) {
        if (dialogLoader.item) {
            dialogLoader.item.state = "hidden"
            dialogLoader.source = ""
        }

        dialogLoader.source = "Components/Dialog.qml"
        var newContent = content.createObject(dialogLoader)

        dialogLoader.item.headerText = title
        dialogLoader.item.content = newContent
        dialogLoader.item.state = "visible"
        dialogLoader.item.contentHeight = newContent.height
        if (newContent.showTextArea) {
            dialogLoader.item.showTextArea = true
        }
        notifyController.initDialog()
    }

    function hideDialog() {
        dialogLoader.item.state = "hidden"
        timer.start()
    }

    Timer {
        id: timer
        interval: 300; running: false; repeat: false
        onTriggered: dialogLoader.source = ""
    }

    function showPlayTokenLostDialog() {
        var content = Qt.createComponent("Components/PlayTokenLostContent.qml")
        if (content.status == Component.Ready) {
            showDialog("Music Paused", content)
        }
    }
}
